package RSServiceSOAPServer;

use SOAP::Lite; # + trace => all;

use strict;
use POSIX qw(setsid);
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use IPC::ShareLite;
use Pod::WSDL;
use Env;
use Tie::ShareLite qw( :lock );


our $conffileOK = readConfiguration("/etc/RS.conf");
if(!$conffileOK) {
      print "RSServiceSOAPServer: configuration file not found or non-readable, exiting now!\n";
      exit(1);
}

push(@INC, $ENV{"RESERVATION_SYSTEM_DIRECTORY"});


require "RSService.pm";
# load the ReservationDatabase - Implements the access to the database containing the sensor data
require "ReservationDatabase.pm";
# some utility functions...
require "RSUtils.pm";

print "starting...\n";

RSServiceSOAPServer();

sub RSServiceSOAPServer {

   print "Starting WISEBED Reservation System ...\n";


   # read the configuration
   my $conffileOK = &RSUtils::readConfiguration("/etc/RS.conf");
   
   my $outfile = "$ENV{'RESERVATION_SYSTEM_DIRECTORY'}"."RSServiceSOAPServer.out";
   my $errfile = "$ENV{'RESERVATION_SYSTEM_DIRECTORY'}"."RSServiceSOAPServer.err";

   defined(my $pid = fork) or die "Can't fork: $!";
   exit if $pid;
   setsid                  or die "Can't start a new session: $!";
     
      
   chdir '/'                   or die "Can't chdir to /: $!";
   open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
   open STDOUT, ">$outfile"  or die "Can't write to $outfile: $!"; 
   open STDERR, ">$errfile"  or die "Can't write to $errfile: $!";

   
   my $rs_ip   = &RSUtils::getHostFromURL($ENV{"RESERVATION_SYSTEM_SERVICE_URL"});
   my $rs_port = &RSUtils::getPortFromURL($ENV{"RESERVATION_SYSTEM_SERVICE_URL"});
   
   print "rs service ip:port = $rs_ip:$rs_port\n";
   
   &RSUtils::redirectOutput($outfile,$errfile);

   if(!&RSUtils::portChecker($rs_ip, $rs_port)) {
	   $0 = "RSServiceSOAPServer_" . $rs_port;
	   print "starting RSServiceSOAPServer_$rs_port ...\n";
	   use SOAP::Transport::TCP;
	   while(1) {
	       my $daemon = SOAP::Transport::HTTP::Daemon
	                    ->new(LocalAddr => $rs_ip, LocalPort => $rs_port, Listen => 50, ReuseAddr => 1);
	       $daemon->dispatch_to('RSService');
	       $daemon->myuri("urn:RSService");
	       $daemon->deserializer(MyDeserializer->new); 
           $daemon->serializer(MySerializer->new);
           
	       
	       print "SOAP HTTP server listening...\n";
	       print "   Host: ", $daemon->sockhost, "\n";
	       print "   Port: ", $daemon->sockport, "\n";
	       $daemon->handle();
	   }
	  
	   BEGIN {
          package MySerializer;
          @MySerializer::ISA = 'SOAP::Serializer';
          sub envelope {
             # print "in envelope: 1=" . $_[1] . "\n";
             # print "in envelope: 2=" . $_[2] . "\n";
             $_[2] = SOAP::Data->name($_[2])->prefix('urn')->uri('urn:RSService')
             if $_[1] =~ /^(?:method|response)$/;
             shift->SUPER::envelope(@_);
          }
	   }
	   
       BEGIN {
          package MyDeserializer; @MyDeserializer::ISA = 'SOAP::Deserializer';
          use SOAP::Lite;
          use vars qw(@ISA);
          
          sub deserialize {
            &RSService::setRawXML_SOAPRequest($_[1]);
            shift->SUPER::deserialize(@_);
          }
       }
	   
   }
   else {
       print "RSServiceSOAPServer: port $rs_port is occupied! kill existing RS deamon and restart!\n";
   }
   
   
}


sub readConfiguration {
     my $conf = $_[0];
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

1;