package SOAP::Transport::HTTP::Daemon::ReapForkOnAccept;

use strict;
use vars qw(@ISA);
use SOAP::Transport::HTTP;
use POSIX ":sys_wait_h";


@ISA = qw(SOAP::Transport::HTTP::Daemon);

sub handle {
  my $self = shift->new;

  CLIENT:
  while (my $c = $self->accept) {

    while (waitpid(-1, &WNOHANG) != -1) {} # reap any unreaped forks
    
    my $pid = fork();

    #  1. The fork failed ($pid is undefined)
    #  2. We are the parent ($pid != 0)
    unless( defined $pid && $pid == 0 ) {
      if (!defined $pid) {
        print STDERR "YIKES; SOAP::Transport::HTTP::Daemon::ReapForkOnAccept Failing to fork handlers.\n";
        print STDERR "Requests are quietly going unanswered (they are likely getting code 500).\n";
      }
      $c->close;
      next;
    }
    
    # From this point on, we are the child.
    $self->close;  # Close the listening socket (always done in children)
    while (my $r = $c->get_request) {
      $self->request($r);
      $self->SOAP::Transport::HTTP::Server::handle;
      $c->send_response($self->response);
    }
    $c->close;
    return;
  }
}

1;
