package ControllerService;


use IPC::ShareLite;
use HTTP::Request;
use HTTP::Response;

use Pod::WSDL;
use Data::Dumper;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use Time::HiRes qw(gettimeofday usleep);
use XML::Simple;
use File::Copy;
use SOAP::Lite;

require "DatabaseInterface.pm";

our $raw_xml_soap_request;
our $raw_xml_soap_response;
our $outfile;
our $errfile;

our $MAX_LINES 						=     2000;
our $VERSION_STRING                 =     "2.3";

require "Utils.pm";

our $WSDL_FILE = &Utils::correctPath($ENV{"TARWIS_DIRECTORY"} . "WSDL/WSNService.wsdl");
    


&Utils::readConfiguration("/etc/TARWIS.conf");
push(@INC, $ENV{"TARWIS_DIRECTORY"});

use strict;



sub setOutfile {
	$outfile = $_[0];
}

sub setErrfile {
    $errfile = $_[0];
}

sub setRawXML_SOAPRequest {
	$raw_xml_soap_request = $_[0];
}

sub setRawXML_SOAPResponse {
	$raw_xml_soap_response = $_[0];
}


sub checkSOAPRequest {
    if($_[0] eq "ControllerService") {
       for(my $i=0; $i<(@_-1); $i++) {
          $_[$i] = $_[$i+1];
       }
    }
}

sub generateWSNServiceInstanceWSDL {
    my $instance_url  = $_[0];
    my $experiment_id = $_[1];
    my $INSTANCE_WSDL_FILE = $WSDL_FILE . "_expid=$experiment_id";
    copy($WSDL_FILE,$INSTANCE_WSDL_FILE); 
    &Utils::replaceEndpointURLinWSDL($INSTANCE_WSDL_FILE, $instance_url);
    print "ControllerService: setting WSNService Instance URL to: $instance_url in file: $INSTANCE_WSDL_FILE\n";
    return $INSTANCE_WSDL_FILE;
}

sub getWSNServiceDriver {
    my $expid  = $_[0];
    my $client = SOAP::Lite->new();
    my $WSNServiceDriver = $client->service("file:" . &Utils::correctPath($ENV{"TARWIS_DIRECTORY"} . "WSDL/WSNService.wsdl_expid=$expid"));
    return $WSNServiceDriver;
}

sub getVersion { 
    my $version = SOAP::Data->name("return")->type("string")->value($VERSION_STRING);
    return $version;
}

sub receiveNotification { 
    print "ControllerService::receiveNotification called\n"; 
    checkSOAPRequest(@_);
    
    print Dumper(@_);
    
    print "ControllerService::receiveNotification end\n"; 
}

sub experimentEnded { 
    print "ControllerService::experimentEnded called\n"; 
    checkSOAPRequest(@_);
    
    print Dumper(@_);
    
    print "ControllerService::experimentEnded end\n"; 
}

sub getVersion_WSNService { 
    my $expid  = $_[0];
    return getWSNServiceDriver($expid)->getVersion();
}


sub resetNodeController {
    print "ControllerService::resetNodeController called\n"; 
    checkSOAPRequest(@_);
    my $node_urn = $_[0];
    my @nodes = ($node_urn);
    # reset - but do not wait for the op to complete!
    resetNodesController(\@nodes, 0);
    print "ControllerService::resetNodeController end\n"; 
}

sub resetNodesController {
   
    print "ControllerService::resetNodesController\n"; 
    my @urns = @{$_[0]};
    # if true, function waits for nodes to wake up!
    my $wait_for_nodes = $_[1];
   
    my $database = new DatabaseInterface();
    my $experiment_id = $database->getCurrentExperimentOfNode(@urns[0]);
   
   
   
    my @urns_soap_request = ();
   
    for(my $i=0; $i<@urns; $i++) {
   	  print "urn: @urns[$i]\n";
      my $soap_obj = SOAP::Data->name("nodes")->type("string")->value(@urns[$i]);
      push(@urns_soap_request, $soap_obj);
    }
      
    print "calling resetNodes at WSNService (expid=$experiment_id)\n";
   
    my $requestID = getWSNServiceDriver($experiment_id)->resetNodes(@urns_soap_request);
    
    $database->setAsynchronousRequestType($requestID, "resetNodes");
   
    if(!($experiment_id =~ m/NOEXPERIMENT/)) {
	    if(@urns > 1) {
	      $database->writeExperimentControllerStatusLine($experiment_id, "multiple nodes are currently being reset");
	    }
	    else {
	      my $node = @urns[0];
	      $database->writeExperimentControllerStatusLine($experiment_id, "$node is currently being reset"); 
	    }
    }
   
    print "resetNodesController req_id=$requestID\n";
      
   
    for(my $i=0; $i<@urns; $i++){
       my $node_urn = @urns[$i];
      
       if($experiment_id =~ m/NOEXPERIMENT/) {
      	  print "ControllerService::resetNodesController: experiment id = NOEXPERIMENT\n";
       }
       else {
          $database->writeNodeCommentToDB($node_urn, $experiment_id, "#############################");
          $database->writeNodeCommentToDB($node_urn, $experiment_id, "node $node_urn is being reset");
          $database->writeNodeCommentToDB($node_urn, $experiment_id, "#############################");
       }
       $database->setAsynchronousStateOfNode($requestID, $node_urn, 0); 
    }

   
    if($wait_for_nodes) {
	   waitForNodesToBeReset(\@urns, $requestID);
    }
    print "ControllerService::resetNodesController end\n"; 
}

sub waitForNodesToBeReset {
	 
   my @urns 		= @{$_[0]};
   my $requestID 	= $_[1];
   
   my $WAIT_PERIOD = $ENV{'WAIT_PERIOD_AFTER_REBOOT'} * 1000000;    
   if($WAIT_PERIOD <= 0) {
   	  $WAIT_PERIOD = 0;
   }
   
   my $MAX_WAITCYCLES = 20;
   my $WAITCYCLE = $WAIT_PERIOD/$MAX_WAITCYCLES;
     
   my $waitForResetOperations = 1;
   my $cycles = 0;
   my $done = 1;
   
   my $database = new DatabaseInterface();
   my $experiment_id = $database->getCurrentExperimentOfNode(@urns[0]);
   
   while($waitForResetOperations==1 && $cycles < $MAX_WAITCYCLES) {
       my $waiting = ($WAITCYCLE * $cycles) / 1000000;
       $done = 1;
       if(!($experiment_id =~ m/NOEXPERIMENT/)) {
           $database->writeExperimentControllerStatusLine($experiment_id, "... time elapsed since reboot: $waiting s (of max. $ENV{'WAIT_PERIOD_AFTER_REBOOT'} s WAIT_PERIOD_AFTER_REBOOT)");
       }
       usleep($WAITCYCLE);
       for(my $i=0; $i<@urns; $i++) {
           my $resetState = $database->getAsynchronousStateOfNode($requestID, @urns[$i]);
           $done = $done && ($resetState == 1); 
       }
       if($done == 1) {
           $waitForResetOperations = 0;
       }
       $cycles++;
   } 
   if($done == 1) { 
   	   if(!($experiment_id =~ m/NOEXPERIMENT/)) {
	       if(@urns > 1) {
	          $database->writeExperimentControllerStatusLine($experiment_id,"... nodes successfully reset");
	       }
	       else {
	          my $node = @urns[0];
	          $database->writeExperimentControllerStatusLine($experiment_id,"... $node successfully reset"); 
	       }
   	   }
   }
}


sub flashProgramsController { 
 
    # receive arguments
    my @urns = @{$_[0]};
    my @program_indices = @{$_[1]};
    my @programs = @{$_[2]};
    my $experiment_id = $_[3];
    
    print "flashProgramsController #nodes: " . @urns . "\n";
      
    
    my $database = new DatabaseInterface();
    
    # prepare for SOAP call
    my @urns_soap_request = ();
    for(my $i=0; $i<@urns; $i++) {
   	  print "flashProgramsController urn: @urns[$i]\n";
      my $soap_obj = SOAP::Data->name("nodeIds")->type("string")->value(@urns[$i]);
      push(@urns_soap_request, $soap_obj);
    }
    
    my @program_indices_soap_request = ();
    for(my $i=0; $i<@program_indices; $i++) {
   	  print "program_indices urn: @program_indices[$i]\n";
      my $soap_obj = SOAP::Data->name("programIndices")->type("int")->value(@program_indices[$i]);
      push(@program_indices_soap_request, $soap_obj);
    }
   
    my @programs_soap_request = ();
    for(my $i=0; $i<@programs; $i++) {
   	  my $soap_obj = SOAP::Data->name("programs")->type("tns:program")->value(@programs[$i]);
      push(@programs_soap_request, $soap_obj);
    }
    
    print "calling flashPrograms (exp_id=$experiment_id)\n";
    my $wsndriver = getWSNServiceDriver($experiment_id); 
    my $requestID = $wsndriver->flashPrograms(@urns_soap_request, @program_indices_soap_request, @programs_soap_request);
    
    # $wsndriver->deserializer(MyCustomDeserializer->new); 
    #       
    # BEGIN {
    #    package MyCustomDeserializer; @MyCustomDeserializer::ISA = 'SOAP::Deserializer';
    #    use SOAP::Lite;
    #    use vars qw(@ISA);
    #    require "Utils.pm";
    #       
    #    sub deserialize {
    #       print "SOAP response: " . $_[1] . "\n";
    #       shift->SUPER::deserialize(@_);
    #    }
    # }
    
    print "obtained REQUEST_ID from getWSNServiceDriver: $requestID\n"; 
    
    print "now waiting for asynchronous replies\n";
    $database->setAsynchronousRequestType($requestID, "flashPrograms");
   
    # wait for maximum WAIT_PERIOD_BEFORE_FLASH_RETRY until reattempting to flash failed nodes
    my @returnvalues = ();
    foreach my $urn (@urns) {
       push(@returnvalues, 0);
       $database->setAsynchronousStateOfNode($requestID, $urn, 0); 
    }
    
    # WAIT_PERIOD_BEFORE_FLASH_RETRY in TARWIS.conf denotes how long the controller waits 
    # after a flashPrograms-call for the next retry!  
    my $WAIT_PERIOD = $ENV{'WAIT_PERIOD_BEFORE_FLASH_RETRY'} * 1000000;   
    if($WAIT_PERIOD<=0) {
    	$WAIT_PERIOD = 0;
    }
    my $MAX_WAITCYCLES = 20;
    my $WAITCYCLE = $WAIT_PERIOD/$MAX_WAITCYCLES;
    
    # print "flashProgramsController: waitcycle= " . $WAITCYCLE . "\n";
    # print "time=" . &Utils::getTimeInMySQLFormat() . "\n";
    
    my $waitForFlashOperations = 1;
    my $cycles = 0;
    
    while($waitForFlashOperations==1 && $cycles < $MAX_WAITCYCLES) {
       my $done = 1;
       usleep($WAITCYCLE);
       # print "i=$cycles time=" . &Utils::getTimeInMySQLFormat() . " waiting for reply to req_id: $requestID\n";
       my $time_elapsed = ($WAITCYCLE * $cycles) / 1000000;
       # print "waited $time_elapsed now of maximal $ENV{'WAIT_PERIOD_BEFORE_FLASH_RETRY'}...\n";
       if( ! ($experiment_id =~ m/NOEXPERIMENT/) ) {
           $database->writeExperimentControllerStatusLine($experiment_id,"... waited $time_elapsed s (of max. $ENV{'WAIT_PERIOD_BEFORE_FLASH_RETRY'} s WAIT_PERIOD_BEFORE_FLASH_RETRY) for status updates");
       }
       for(my $i=0; $i<@urns; $i++) {
           my $flashState = $database->getAsynchronousStateOfNode($requestID, @urns[$i]);
           my $urn = @urns[$i];
           # print "state of node $urn is $flashState\n";
           if($flashState == 100) {
              @returnvalues[$i]=1;
              $done = (1 && $done);
           }
           elsif($flashState == -1) {
              $done = (1 && $done);
           }
           else {
              $done = 0;
           } 
       }
       if($done == 1) {
           $waitForFlashOperations = 0;
       }
       $cycles++;
       # print "end while: done=$done waitForFlashOperations=$waitForFlashOperations\n";
    }    
    
    &Utils::truncateFile($outfile, $MAX_LINES);
    &Utils::truncateFile($errfile, $MAX_LINES);
    
    print "flashProgramsControllerEnd\n"; 
    return @returnvalues;
}


#############################################################
# gsoap (C Programming Language) sends:
#
#   <?xml version="1.0" encoding="UTF-8"?>
#       <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" 
#       xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" 
#       xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
#       xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
#       xmlns:ns1="urn:ControllerService">
#           <SOAP-ENV:Body>
#               <ns1:receiveStatus>
#                   <status>
#                       <requestId>alive:0</requestId>
#                       <status>
#                           <nodeId>urn:wisebed:node:ulanc:1</nodeId>
#                           <value>1</value>
#                       </status>
#                   </status>
#               </ns1:receiveStatus>
#           </SOAP-ENV:Body>
#   </SOAP-ENV:Envelope>   

# perl SOAP::Lite sends:
# <?xml version="1.0" encoding="UTF-8"?>
# <soap:Envelope soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
#   xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
#   xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
#   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
#   xmlns:tns="urn:ControllerService"
#   xmlns:xsd="http://www.w3.org/2001/XMLSchema">
#   <soap:Body>
#       <tns:receiveStatus>
#           <status>
#               <requestId xsi:type="xsd:string">request#1</requestId>
#               <status>
#                   <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:18</nodeId>
#                   <value xsi:type="xsd:int">1</value>
#                   <msg xsi:type="xsd:string">status update</msg>
#               </status>
#           </status>
#       </tns:receiveStatus>
#   </soap:Body>
# </soap:Envelope>
   
# 27.04.2010 Request with multiple status' subtags   
# <?xml version="1.0" encoding="UTF-8"?>
# <soap:Envelope soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
# xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
# xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
# xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
# mlns:tns="urn:ControllerService" 
# xmlns:xsd="http://www.w3.org/2001/XMLSchema">
#   <soap:Body>
#       <tns:receiveStatus>
#              <status>
#                  <status>
#                     <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeId>
#                     <value xsi:type="xsd:int">1</value>
#                     <msg xsi:type="xsd:string">status update</msg>
#                  </status>
#                  <status>
#                     <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeId>
#                     <value xsi:type="xsd:int">1</value>
#                     <msg xsi:type="xsd:string">status update</msg>
#                  </status>
#                  <status>
#                     <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:3</nodeId>
#                     <value xsi:type="xsd:int">1</value>
#                     <msg xsi:type="xsd:string">status update</msg>
#                  </status>
#                  <requestId xsi:type="xsd:string">request#1</requestId>
#              </status>
#        </tns:receiveStatus>
#  </soap:Body>
# </soap:Envelope>

# <tns:receiveStatus>
#     <status>
#         <status>
#         <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeId>
#         <value xsi:type="xsd:int">1</value>
#         <msg xsi:type="xsd:string">status update 1</msg>
#         </status>
#         <status>
#         <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeId>
#         <value xsi:type="xsd:int">1</value>
#         <msg xsi:type="xsd:string">status update 1</msg>
#         </status>
#         <requestId xsi:type="xsd:string">req#1</requestId>
#     </status>
#     <status>
#         <status>
#         <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeId>
#         <value xsi:type="xsd:int">1</value>
#         <msg xsi:type="xsd:string">status update 1</msg>
#         </status>
#         <status>
#         <nodeId xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeId>
#         <value xsi:type="xsd:int">1</value>
#         <msg xsi:type="xsd:string">status update 1</msg>
#         </status>
#         <requestId xsi:type="xsd:string">req#1</requestId>
#     </status>
# </tns:receiveStatus>

    
our $MAX_DB_QUERY_ATTEMPTS = 3;

sub receiveStatus {
	print "ControllerService::receiveStatus called " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n"; 
	# print "SOAP: $raw_xml_soap_request\n";
    checkSOAPRequest(@_);
    my $method_name = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "receiveStatus");
    # print "MethodName: $method_name\n";
    
    my @status_messages = &Utils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "status");
    
    my @requestIds = ();
    my @urns = ();
    my @states = ();
    
    my @experiment_ids = ();
    my @statusLines = ();
    my @times = ();
    
    my $time = &Utils::getTimeInMySQLFormat();
    
    my $database; 
    
    if(@status_messages>0){
       $database = new DatabaseInterface();                
    } 
        
    foreach my $status (@status_messages) {
    	# print Dumper($status);
	    
	    my $requestId 	= &Utils::trimchomp(&Utils::getPropertyValue($status,"requestId"));
	    # print "requestId: $requestId\n";
	    
	    my @items = (); 
	    
	    $status = $status->{"status"};
	    
		if(ref($status) eq 'HASH' ){
			# print "is hash!\n";
	        push(@items, $status);
	    }
	    else {
	    	# print "is array!\n";
	        foreach my $record (@{$status})  {
	            push(@items, $record);
	        }
	    }
	    
	    #print "items: " . @items . "\n";
	    #print "looping through items\n";
	   
	    foreach my $item (@items){
	    	my $nodeId = undef;
	    	my $value = undef;
	    	
	    	# print "item: $item\n";
	    	
	    	my $nodeIdExists = &Utils::checkTagExists($item, "nodeId");
	    	my $valueExists  = &Utils::checkTagExists($item, "value");
	    	
	    	# print "nodeIdExists: $nodeIdExists\n";
	        # print "valueExists: $valueExists\n";
	        
	        if( $nodeIdExists && $valueExists ) {
		    	$nodeId      = &Utils::getPropertyValue($item, "nodeId");
		    	$value       = &Utils::getPropertyValue($item, "value");
	            
		    	if($value eq "" || length($value) == 0) {
	    			$value = 0;
	    		}
		    	print "request: $requestId: got status from node: $nodeId, value: $value\n";
		    	    	
		    	my $requestType;
	  
			    if($value == 1 || $value == 100 || $value == 0) {
				   	my $succeeded = 0;
				   	my $attempts = 0;
				   	
				    while($attempts < $MAX_DB_QUERY_ATTEMPTS && !$succeeded ) {
					    $requestType = $database->getAsynchronousRequestType($requestId);
					    if( ! ($requestType =~ m/UNKNOWNREQUESTID/) ) {
						    
						    my $experiment_id = $database->getCurrentExperimentOfNode($nodeId);
						  	if( ! ($requestType =~ m/areNodesAlive/) && ! ($experiment_id =~ m/NOEXPERIMENT/) ) {
						        push(@experiment_ids, $experiment_id);
						        push(@statusLines, "status of request \"$requestId\" ($requestType) ($nodeId): $value");	
						   	    push(@times, $time);
						  	}
						   	push(@requestIds,$requestId);
	                        push(@urns,$nodeId);
	                        push(@states,int($value));
							$succeeded = 1;
					    }
					    else {
					    	print "receiveStatus: error when trying to match received requestId: $requestId to any operation (attempt $attempts)\n";      
					    	usleep(100000);
					    	$attempts++;
					    }
				    }
			    }
			    else {
			    	print "value: $value (not saving to DB)\n";
			    }
			        
	    	}
	    	else {
	    		print "status must have nodeId AND value! one attribute is missing! nodeId=$nodeId value=$value\n";
	    	}
	    }
    }
    
    if(@experiment_ids>0) {
    	print "statusLines: writing " . @experiment_ids ." lines into the DB\n";
        $database->writeExperimentControllerStatusLine_arrays(\@experiment_ids, \@statusLines, \@times);
    }
    
    if(@requestIds>0){
    	print "states: writing " . @requestIds ." lines into the DB\n";
    	$database->setAsynchronousStateOfNode_arrays(\@requestIds, \@urns, \@states);
    }
    
    print "truncating $outfile\n";
    
    &Utils::truncateFile($outfile, $MAX_LINES);
    &Utils::truncateFile($errfile, $MAX_LINES);
   
    print "ControllerService::receiveStatusEnd " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";    
    
    return;
}

sub extractUsecs {
    my $string = $_[0];
    if($string =~ /\./) {
      $string =~ s/.*\.//;
      $string =~ s/[A-Z]//;
      $string =~ s/\+.*//;
      my $l = length($string);
      for(my $i=$l; $i<6; $i++){
      	 $string .= "0";
      }
      
    }
    else {
      $string = "0";
    }
    return $string;
}

sub removeUsecs {
    my $string = $_[0];
    if($string =~ /\./) {
      $string =~ s/\.[0-9]*//;
    }
    return $string;
}


sub removeStuff {
    my $string = $_[0];
    $string =~ s/\$VAR[0-9]*\ =\ '//;
    $string =~ s/';$//;
    return &Utils::trimchomp($string);
}

#               if($textMessage =~ m/\<data\ +key\ *=/ && $textMessage =~ m/\<\/data\>/){
#                   my $temp_text = $textMessage;
#                   $temp_text =~ s/\<data\ +key\ *=\ *//;
#                   $temp_text =~ s/\ *\<\/data\>//;
#                   $temp_text =~ s/\"//;
#                   $temp_text =~ s/\"//;
#                   $temp_text =~ s/\'//;
#                   $temp_text =~ s/\'//;
#                   $temp_text = &Utils::trimchomp($temp_text);
#                   my @tokens = split('>', $temp_text);
#                   if(@tokens==2){
#                       my $key   =@tokens[0];
#                       my $value =@tokens[1];
#                       my @properties = $database->getDecimalPropertiesOfSensorNode($sourcenodeID);
#                       for(my $k=0; $k<@properties; $k++){
#                           my $property = &Utils::stripePrefixFromURN(@properties[$k]);
#                           my $propID = $database->getPropertyID(@properties[$k]);
#                           if($key =~ m/$property/ ){
#                               # print "key: $key matched!\n";
#                               $database->insertSensorNodeValues($sourcenodeID, $propID, $value, $timestamp, $usecs);
#                           }
#                       }
#                   }
#               }


#    <soap:Body>
#        <tns:receive>
#            <msg>
#                <sourceNodeId xsi:type="xsd:string">
#                    urn:wisebed:node:ubern:msb5
#                </sourceNodeId>
#                <timestamp xsi:type="xsd:dateTime">
#                    2011-03-17T10:10:32.961547
#                </timestamp>
#                <binaryData xsi:type="xsd:base64Binary">
#                    ZXN0aW1hdGlvbiByY3Y6IDc4Ljk3OCBzbHA6IDI0MTcuNDk1IHR4OiAzMy40NTcgcmN2X2NoZzogNTY0MSBzbHBfY2hnOiA1NDM3IHR4X2NoZzogMjA2IHByb2dyYW1fdGltZTogMjUzMC00
#                </binaryData>
#            </msg>
#            <msg>
#                <sourceNodeId xsi:type="xsd:string">
#                    urn:wisebed:node:ubern:msb4
#                </sourceNodeId>
#                <timestamp xsi:type="xsd:dateTime">
#                    2011-03-17T10:10:32.959191
#                </timestamp>
#                <binaryData xsi:type="xsd:base64Binary">
#                    ZXN0aW1hdGlvbiByY3Y6IDgwLjg5MyBzbHA6IDI0MzUuMTU5IHR4OiAxMy44ODEgcmN2X2NoZzogNTc4MyBzbHBfY2hnOiA1NTI2IHR4X2NoZzogMjYyIHByb2dyYW1fdGltZTogMjUzMC00
#                </binaryData>
#            </msg>
#            <msg>
#                <sourceNodeId xsi:type="xsd:string">
#                    urn:wisebed:node:ubern:msb1
#                </sourceNodeId>
#                <timestamp xsi:type="xsd:dateTime">
#                    2011-03-17T10:10:32.958255
#                </timestamp>
#                <binaryData xsi:type="xsd:base64Binary">
#                    ZXN0aW1hdGlvbiByY3Y6IDk2LjA1OSBzbHA6IDI0MTUuNzk1IHR4OiAxOC4wNzcgcmN2X2NoZzogNjIyNCBzbHBfY2hnOiA1NzE2IHR4X2NoZzogNTA5IHByb2dyYW1fdGltZTogMjUzMC00
#                </binaryData>
#            </msg>
#            <msg>
#                <sourceNodeId xsi:type="xsd:string">
#                    urn:wisebed:node:ubern:msb7
#                </sourceNodeId>
#                <timestamp xsi:type="xsd:dateTime">
#                    2011-03-17T10:10:33.048172
#                </timestamp>
#                <binaryData xsi:type="xsd:base64Binary">
#                    ZXN0aW1hdGlvbiByY3Y6IDExMS4zMzIgc2xwOiAyNDA5LjU3NSB0eDogOS4wMTggcmN2X2NoZzogNTM1MSBzbHBfY2hnOiA1MTQ3IHR4X2NoZzogMjA1IHByb2dyYW1fdGltZTogMjUzMC00
#                </binaryData>
#            </msg>
#            <msg>
#                <sourceNodeId xsi:type="xsd:string">
#                    urn:wisebed:node:ubern:msb2
#                </sourceNodeId>
#                <timestamp xsi:type="xsd:dateTime">
#                    2011-03-17T10:10:32.941610
#                </timestamp>
#                <binaryData xsi:type="xsd:base64Binary">
#                    ZXN0aW1hdGlvbiByY3Y6IDg5LjY5NSBzbHA6IDI0MzIuNTQ3IHR4OiA3LjY5MSByY3ZfY2hnOiA1Mzk4IHNscF9jaGc6IDUyMjEgdHhfY2hnOiAxNzggcHJvZ3JhbV90aW1lOiAyNTMwLTQ=
#                </binaryData>
#            </msg>
#        </tns:receive>
#    </soap:Body>

our $messagesCounter = 0;

sub escapeBadEncoding {
    my $data = $_[0];
    my $puredata = "";
    my @chars=split(//,$data);
    for(my $i=0; $i<@chars; $i++){
       my $n = ord(@chars[$i]);
       if( $n == 9 || $n == 10 || ($n >= 32 && $n <= 126) ) {
         $puredata .=  @chars[$i];
       }
       else {
         $puredata .=  "[0x" . sprintf ("%02x", ord(@chars[$i])) . "]"; 
       }
    }
    return $puredata;
}

# RECEIVE

sub receive {
    checkSOAPRequest(@_);
    print "ControllerService::receive called " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
    
    print "SOAP: $raw_xml_soap_request\n";
    my $method_name = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "receive");
    # print "MethodName: $method_name\n";
    my @messages = &Utils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "msg");
    
    my @sourcenodeID = (); 
    my @experimentID = ();
    my @textMessage = ();
    my @timestamp = ();
    my @usecs = ();
    
    my $database;
    
    if(@messages>0) {
        # print "intiating db connection\n";
        $database = new DatabaseInterface();
    }
    
    print "messages: " . @messages . "\n";
    
    foreach my $message (@messages) {
        
        my $sourcenodeID        = &Utils::trimchomp(&Utils::getPropertyValue($message,"sourceNodeId"));
        my $timestamp           = &Utils::trimchomp(&Utils::getPropertyValue($message,"timestamp"));
        my $textMessage         = &Utils::trimchomp(&Utils::decodeMessage(&Utils::getPropertyValue($message, "binaryData")));
        my $timestamp_original  = $timestamp;
        
        if($textMessage eq "") {
           $textMessage  = &Utils::trimchomp(&Utils::getPropertyValue($message->{textMessage}, "msg"));
        }
        
        if($textMessage eq "") {
           $textMessage  = &Utils::trimchomp(&Utils::decodeMessage(&Utils::getPropertyValue($message->{binaryMessage}, "binaryData")));
        }
        
        # my $binaryData = &Utils::getPropertyValue($message, "binaryData");
        # print "binary = $binaryData\n";
        # print "before = $textMessage\n";
        $textMessage = escapeBadEncoding($textMessage);
        # print "after = $textMessage\n";
        # print "sourcenodeID = $sourcenodeID\n";
        # print "timestamp = $timestamp\n";
        # print "textMessage = $textMessage\n";
        
        my $experiment_id = $database->getCurrentExperimentOfNode($sourcenodeID);
        my $experiment_start_time_secs = 0;
        my $experiment_end_time_secs = 0;
        
        if( !($experiment_id eq "") && !($experiment_id eq "NOEXPERIMENT") )  {
            $experiment_start_time_secs = &Utils::parseDate($database->getStartOfExperimentWithID($experiment_id));
            $experiment_end_time_secs  =  &Utils::parseDate($database->getEndOfExperimentWithID($experiment_id));
        
            print "ControllerService::receive sourcenodeID = $sourcenodeID\n";
            print "ControllerService::receive message = \"$textMessage\"  now: ". &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
            print "timestamp = $timestamp\n";
            # print "textMessage = $textMessage\n";
            
            my $usecs = extractUsecs($timestamp);
            # print "usecs = $usecs\n";
            
            my $timestamp = removeUsecs($timestamp);
            # print "timestamp = $timestamp\n";
            
		    my $received_time_secs = &Utils::parseDate($timestamp);
		    # print "received_time_secs = $received_time_secs\n";
            
		    if( !$sourcenodeID eq "" && validLine($textMessage)) { # && validTime($timestamp) && validUsecs($usecs)) {
		    	   
			    if($experiment_start_time_secs > $received_time_secs || $experiment_end_time_secs < $received_time_secs) {
	                print "ControllerService::receive: time not within defined experiment duration range!\n";
	                # alter the text message: 
	                $textMessage = "(invalid timestamp) " . $textMessage;
	                # IMPORTANT: OUTPUT WILL BE DELETED WHEN EXPORTED TO WISEML!
	                print "timestamp_original = " . $timestamp_original . "\n";
	            }
			    
			    
			    $timestamp = &Utils::convertToMySQLTimestamp($received_time_secs);
			    # print "timestamp converted to MySQL format: " . $timestamp . "\n";
			      
			    push(@sourcenodeID, $sourcenodeID);
			    push(@experimentID, $experiment_id);
	            push(@textMessage, $textMessage);
	            push(@timestamp, $timestamp);
	            push(@usecs, $usecs);
	            
			}
			else {
			    print "not saving to DB: invalid data:\n";
			    print "sourcenodeID=$sourcenodeID\n";
			    print "experiment_id=$experiment_id\n";
		        print "timestamp=$timestamp\n";
		        print "usecs=$usecs\n";
		        print "textMessage=$textMessage\n";
		    }
		        
		} # if there is no experiment
        else {
            print "no experiment of node $sourcenodeID found\n";
        }   
	} # foreach message
    
    # after the output from different messages has been collected in the arrays, write them alltogether into the db
    if(@sourcenodeID>0) {
    	print "writing into the db: lines: ". @sourcenodeID . "\n";
        $database->writeTimestampedSensorNodeOutputToDB_arrays(\@sourcenodeID, \@experimentID, \@textMessage, \@timestamp, \@usecs);
    }            
    
    if($messagesCounter > 20) {
    	&Utils::truncateFile($outfile, $MAX_LINES);
        &Utils::truncateFile($errfile, $MAX_LINES);
        $messagesCounter = 0;
    }
    $messagesCounter++;
        
    
    print "ControllerService::receive end " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
    
    return;
}   

sub validTime {
   return length($_[0])>0 && $_[0] =~ /-/;
}

sub validUsecs {
   return length($_[0])>0 && length($_[0])<=6 && &Utils::isInteger($_[0]);
}

sub isUndefined {
   return $_[0] =~ /undef;/ && $_[0] =~ /\$VAR/;
}

sub validLine {
   return length($_[0])>0 && !isUndefined($_[0]);
}

sub getNeighbourhoodController {
    my $node = $_[0];
    
    print "getNeighbourhoodController called at node $node" . &Utils::getTimeInMySQLFormat() . "\n";
        
    my $soap_obj = SOAP::Data->new(name => "node", value => $node, type => "string");
    
    my $soap_client = getWSNServiceDriver();
    $soap_client->deserializer(MyCustomDeserializer->new); 
          
    BEGIN {
       package MyCustomDeserializer; @MyCustomDeserializer::ISA = 'SOAP::Deserializer';
       use SOAP::Lite;
       use vars qw(@ISA);
	   require "Utils.pm";
          
       sub deserialize {
          &ControllerService::setRawXML_SOAPResponse($_[1]);
          shift->SUPER::deserialize(@_);
       }
    }
    
    $soap_client->getNeighbourhood($soap_obj);
    
    # print "SOAP: $raw_xml_soap_response\n";
    
    my $method_name = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_response, "getNeighbourhoodResponse");
    # print "MethodName: $method_name\n";
    my @items = &Utils::getItemsFromSOAPBody($raw_xml_soap_response, $method_name, "return");
    my @neighbours = ();
    
    # print "items: @items\n";
    
    foreach my $item (@items){
    	# print "item: $item\n";
    	$item = &Utils::getValue($item);
        $item = &Utils::trimchomp($item);
        if(! $item eq "" && length($item) > 0) {
            print "neighbour found: $item\n";
            push(@neighbours, $item);
        }
    }
    
    print "getNeighbourhoodController end " . &Utils::getTimeInMySQLFormat() . "\n";
   
    return @neighbours;
}

sub getLinkQualityController {
    my $node_src = $_[0];
    my $node_dst = $_[1]; 
    my $src_soap = SOAP::Data->new(name => "src", value => $node_src, type => "string");
    my $dst_soap = SOAP::Data->new(name => "dst", value => $node_dst, type => "string");
    
    my $database = new DatabaseInterface();
    my $experiment_id = $database->getCurrentExperimentOfNode($node_src);
    if(!($experiment_id =~ m/NOEXPERIMENT/)) {
        my $linkQuality = getWSNServiceDriver($experiment_id)->getLinkQuality($src_soap, $dst_soap);
        return $linkQuality;
    }   
    return -1;
}

sub runCommandController { 
    checkSOAPRequest(@_);
    print "ControllerService::runCommandController called " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  

    my $urn = $_[0];
    my $command = $_[1];
    
    my $database = new DatabaseInterface();
    my $experiment_id = $database->getCurrentExperimentOfNode($urn);
    $database->addUpdateCommandEntry($urn, $command, &Utils::getTimeInMySQLFormat(), &Utils::getUsecs(), $experiment_id, 0);
    
    print "ControllerService::runCommandController end " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  
}

sub runBroadcastCommandController { 
    checkSOAPRequest(@_); 
    print "ControllerService::runBroadcastCommandController called " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  
    
    my @nodes_urns = @{$_[0]};
    my $command    = $_[1];
    
    if(@nodes_urns > 0 ) {
	    my $urn = "broadcast|" . @nodes_urns[0];
	    my $database = new DatabaseInterface();	
	    my $experiment_id = $database->getCurrentExperimentOfNode(@nodes_urns[0]); 
        $database->addUpdateCommandEntry($urn, $command, &Utils::getTimeInMySQLFormat(), &Utils::getUsecs(), $experiment_id, 0);   
    }
    else {
    	print "ControllerService::nodes_urns size = " . @nodes_urns . "\n";
    }
    
    print "ControllerService::runBroadcastCommandController end " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  
}

sub resetNodesBroadcastCommandController { 
    checkSOAPRequest(@_); 
    print "ControllerService::resetNodesBroadcastCommandController called " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  
    
    my @nodes_urns = @{$_[0]};
     
    foreach my $node (@nodes_urns){
        print "resetting node: $node\n";
    }
    
    resetNodesController(\@nodes_urns, 0);
    
    print "ControllerService::resetNodesBroadcastCommandController end " . &Utils::getTime_Hours_Minutes_Secs_Usecs   () . "\n";  
}

sub runCommand_execute {
    print "ControllerService::runCommand_execute called " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";  

    my $urns          = $_[0];
    my $command       = $_[1];
    my $database      = $_[2];
    my $experiment_id = $_[3];
    
    my @sortedurns = &Utils::sortURNs(@$urns);
    
    if(@sortedurns==0) {
    	print "ERROR! number of urns = 0\n";
        return; 
    }
    
    print "current experiment id: $experiment_id\n";            
                
    if($experiment_id =~ m/NOEXPERIMENT/) {
        print "runCommand_execute: experiment_id is $experiment_id ... experiment is over\n";
        return;
    }
    
    my $instance_url = $database->getEndpointURLofRunningExperiment($experiment_id);
    if( $instance_url =~ m/NOSUCHEXPERIMENT/ ) { 
        print "runCommand_execute: instance_url = $instance_url - maybe experiment is over? \n";
        return;
    }
    
        	
    my @urns_soap_request = ();
    foreach my $urn (@sortedurns){
        my $soap_obj = SOAP::Data->name("nodeIds")->type("string")->value($urn);
        push(@urns_soap_request, $soap_obj);
    }
    my $message = generateComplexTypeMessage_send($command);
    print "calling send at $instance_url\n"; 
    
    eval{
        getWSNServiceDriver($experiment_id)->send(@urns_soap_request,$message);
    };
    if ($@) {
        print "runCommand_execute: no WSNService present at $instance_url - maybe experiment is over? \n";
        return;
    }
    
    
    print "ControllerService::runCommand_execute end " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";  
}

sub getPropertyValueOf {
    my $node_urn = $_[0];
    my $property = $_[1];
    
    my $database = new DatabaseInterface();
    my $experiment_id = $database->getCurrentExperimentOfNode($node_urn);
   
    
    my $node_soap = SOAP::Data->new(name => "node", value => $node_urn, type => "string");
    my $property_soap = SOAP::Data->new(name => "propertyName", value => $property, type => "string");
    
    if($experiment_id =~ m/NOEXPERIMENT/) {
        return getWSNServiceDriver($experiment_id)->getPropertyValueOf($node_soap, $property_soap);
    }
    else {
    	return -1;
    }
}

sub addUpdateSensorNode {
    checkSOAPRequest(@_);
    my $urn = $_[0];
    my $x = int($_[1]);
    my $y = int($_[2]);
    my $z = int($_[3]);
    print "received a request to register node $urn at $x $y $z\n";    
    return 1;
}

sub generateComplexTypeMessage_send {
  
   my $textData      = $_[0];
   
   my $soap_datetime = generateSOAPDateTimeObject(&Utils::getTimeInMySQLFormat(), &Utils::getUsecs());
   
   my $soap_message = SOAP::Data->name("message" => \SOAP::Data->value( 
                                                    SOAP::Data->new(name => "sourceNodeId",  value => "user input", type => "string"), 
                                                    SOAP::Data->new(name => "timestamp",     value => $soap_datetime, type => "dateTime"), 
                                                    SOAP::Data->name("textMessage" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "msg", value => $textData, type => "string"),
                                                                    )
                                                                  )
                                                                    )
                                       ); 
   return $soap_message;                                    
}

sub generateSOAPDateTimeObject {
   my $myTime = $_[0];
   my $microseconds = $_[1];
   # print "myTime=" . $myTime . "\n";
   my $date = ParseDate($myTime);
   # print "date=" . $date . "\n";
   my $datestr = UnixDate($date, "%a %b %e %H:%M:%S %z %Y"); 
   # print "datestr=" . $datestr . "\n";
   my $soap_datetime = ConvertDate($datestr);
   my $date_including_microseconds;
   if(int($microseconds)>0){
     $date_including_microseconds = sprintf("%s.%06d", $soap_datetime, $microseconds);
   }
   else {
     $date_including_microseconds = $soap_datetime;
   }
   # print "soap_datetime=" . $date_including_microseconds . "\n";
   return $date_including_microseconds;
}

sub generateComplexTypeProgram {
  
   my $program  = $_[0];
   my $version  = $_[1];
   my $name     = $_[2];
   my $platform = $_[3];
   my $other    = $_[4];
   my $soap_program = SOAP::Data->name("programs" => \SOAP::Data->value( 
                                                      SOAP::Data->new(name => "program",     value => $program, type => "base64Binary"), 
                                                      SOAP::Data->name("metaData" => \SOAP::Data->value( 
                                                                                 SOAP::Data->new(name => "version", value => $version, type => "string"),
                                                                                 SOAP::Data->new(name => "name", value => $name, type => "string"),
                                                                                 SOAP::Data->new(name => "platform", value => $platform, type => "string"),
                                                                                 SOAP::Data->new(name => "other", value => $other , type => "string")
                                                                                                  )->type("tns:programMetaData")
                                                                   ),                                                 
                                                                       )
                                       )->type("tns:program"); 
   return $soap_program;                                    
}

sub generateSOAPDateTimeObject {
   my $myTime = $_[0];
   my $microseconds = $_[1];
   my $date = ParseDate($myTime);
   my $datestr = UnixDate($date, "%a %b %e %H:%M:%S %z %Y"); 
   my $soap_datetime = ConvertDate($datestr);
   my $date_including_microseconds;
   if(int($microseconds)>0){
     $date_including_microseconds = sprintf("%s.%06d", $soap_datetime, $microseconds);
   }
   else {
     $date_including_microseconds = $soap_datetime;
   }
   return $date_including_microseconds;
}

sub isAlive {
   print "ControllerService::isAlive called " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
   return 1;
}








# our $TURNON_DB_ACCESS = 1;
#
# sub receiveStatus_raw {
#
#    my $database;
#    if($TURNON_DB_ACCESS) {
#        $database = new DatabaseInterface();
#    }
#
#    while(@_ >= 4) {
#        my $requestId   = shift;
#        my $nodeId      = shift;
#        my $value       = shift;
#        my $msg         = shift;
#                
#        if($value eq "" || length($value) == 0) {
#           $value = 0;
#        }
#    
#        print "request: $requestId: got status from node: $nodeId, value: $value, msg: $msg\n";
#         
#        my $requestType;
#                 
#        if($TURNON_DB_ACCESS) {       
#            if($value == 1 || $value == 100) {
#                my $succeeded = 0;
#                my $attempts = 0;
#                        
#                while($attempts < $MAX_DB_QUERY_ATTEMPTS && !$succeeded ) {
#                   $requestType = $database->getAsynchronousRequestType($requestId);
#                   if( ! ($requestType =~ m/UNKNOWNREQUESTID/) ) {
#                   my $experiment_id = $database->getCurrentExperimentOfNode($nodeId);
#                   if( ! ($requestType =~ m/areNodesAlive/) && ! ($experiment_id =~ m/NOEXPERIMENT/) ) {
#                       $database->writeExperimentControllerStatusLine($experiment_id, "status of request \"$requestId\" ($requestType) ($nodeId): $value");
#                   }
#                   $database->setAsynchronousStateOfNode($requestId, $nodeId, int($value));
#                       $succeeded = 1;
#                   }
#                   else {
#                       print "receiveStatus_raw: error when trying to match received requestId: $requestId to any operation (attempt $attempts)\n";      
#                       usleep(100000);
#                       $attempts++;
#                   }
#                }
#            } 
#            else {
#                print "receiveStatus_raw value: $value (not saving to DB)\n";
#            }
#        }
#    }
#}
#       &Utils::readConfiguration("/etc/TARWIS.conf");
#       $outfile = &Utils::correctPath($ENV{"TARWIS_DIRECTORY"} . "/ControllerServiceSOAPServer.out");
#       $errfile = &Utils::correctPath($ENV{"TARWIS_DIRECTORY"} . "/ControllerServiceSOAPServer.err");
#       &Utils::truncateFile($outfile, $MAX_LINES);   
#       &Utils::truncateFile($errfile, $MAX_LINES);
# 
#
#
# sub receive_raw {
#    
#    print "receive_raw: called " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
#    
#    my $database;    
#    
#    my @sourcenodeID = (); 
#    my @experimentID = ();
#    my @textMessage = ();
#    my @timestamp = ();
#    my @usecs = ();
#    
#    if($TURNON_DB_ACCESS) {
#        $database = new DatabaseInterface();
#    }
#    
#    my $i=0;
#    
#    while(@_ >= 3) {
#        my $sourcenodeID        = shift;
#        my $timestamp           = shift;
#        $timestamp              = &Utils::trimchomp($timestamp);    
#        $sourcenodeID           = &Utils::trimchomp($sourcenodeID);
#        my $textMessage         = shift;
#        
#        $i++;
#        
#        # print "message #$i\n";
#        # print "sourcenodeID = $sourcenodeID\n";
#        # print "timestamp    = $timestamp\n";
#        # print "textMessage  = $textMessage\n";
#            
#        if($TURNON_DB_ACCESS) {
#            my $experiment_id = $database->getCurrentExperimentOfNode($sourcenodeID);
#            # print "experiment_id = $experiment_id\n";
#            my $experiment_start_time_secs = 0;
#            my $experiment_end_time_secs = 0;
#            
#            if( !($experiment_id eq "") && !($experiment_id eq "NOEXPERIMENT") )  {
#               $experiment_start_time_secs = &Utils::parseDate($database->getStartOfExperimentWithID($experiment_id));
#               $experiment_end_time_secs  =  &Utils::parseDate($database->getEndOfExperimentWithID($experiment_id));
#            } 
#            else {
#               print "no experiment of node found\n";
#               print "ControllerService::receive_raw end " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
#               return;
#            }
#            
#            my $usecs              = 0;
#            my $received_time_secs = &Utils::parseDate($timestamp);
#            
#            if( !$sourcenodeID eq "" && validLine($textMessage) ) {       
#               if($experiment_start_time_secs > $received_time_secs || $experiment_end_time_secs < $received_time_secs) {
#                   print "ControllerService::receive: time not within defined experiment duration range!\n";
#                   print "sourcenodeID               = $sourcenodeID\n";
#                   print "experiment_id              = $experiment_id\n";
#                   print "experiment_start_time_secs = $experiment_start_time_secs\n";
#                   print "experiment_end_time_secs   = $experiment_end_time_secs\n";
#                   $textMessage = "(invalid timestamp) " . $textMessage;
#                   # IMPORTANT: OUTPUT WILL BE DELETED WHEN EXPORTED TO WISEML!
#               }             
#               
#               push(@sourcenodeID, $sourcenodeID);
#               push(@experimentID, $experiment_id);
#               push(@textMessage, $textMessage);
#               push(@timestamp, $timestamp);
#               push(@usecs, $usecs);
#            }
#            else {
#               print "not saving to DB: invalid data:\n";
#               print "sourcenodeID=$sourcenodeID\n";
#               print "experiment_id=$experiment_id\n";
#               print "timestamp=" . &Utils::getTimeInMySQLFormat() . "\n";
#               print "usecs=$usecs\n";
#               print "textMessage=$textMessage\n";
#            }   
#            
#        } # if turnon db
#    } # while loop
#    
#    if(@sourcenodeID>0) {
#        $database->writeTimestampedSensorNodeOutputToDB_arrays(\@sourcenodeID, \@experimentID, \@textMessage, \@timestamp, \@usecs);
#    }            
#    
#    print "ControllerService::receive_raw end " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
#}



1;