package EmailClient;

use strict;
use POSIX qw(setsid);
use POSIX ();
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use Carp::Assert;
use Proc::Simple;
use Env;

require "Utils.pm";
require "DatabaseInterface.pm";

# read the configuration
&Utils::readConfiguration("/etc/TARWIS.conf");

our $SENDER_ADDRESS = "wisebed-testbed-do-not-reply@".$ENV{"SMTP_RELAY"};
        

sub notifyOwnerOfExperimentWithID {
    my $experimentID        = $_[0];
    my $resultsfilesize     = $_[1];
    my $MESSAGE_BODY_FILE   = $ENV{'TARWIS_DIRECTORY'} . "notification_" . $experimentID . ".txt";
    
    if(-e $MESSAGE_BODY_FILE ) {
       unlink($MESSAGE_BODY_FILE);
    }
    
    print "file: " . $MESSAGE_BODY_FILE . "\n";
    
    my $database = new DatabaseInterface();
    
    my $email_address = $database->getEmailOfExperimentWithID($experimentID);
    
    if(!&Utils::validEmail($email_address)) {
    	print "EmailClient::notifyOwnerOfExperimentWithID invalid email address $email_address, aborting...\n";
    	return;
    }
    
    my $experiment_name = &Utils::trimchomp($database->getNameOfExperimentWithID($experimentID));
    my $user_unique_id = $database->getUserUniqueIDOfExperimentWithID($experimentID);
    my $description = $database->getDescriptionOfExperimentWithID($experimentID);
    $description = &Utils::trimchomp($description);
    
    my $start_time = $database->getStartOfExperimentWithID($experimentID);
    my $end_time = $database->getEndOfExperimentWithID($experimentID);
    my $subject = "experiment #id=$experimentID has ended";  
     
    if(-e $MESSAGE_BODY_FILE ) {
       unlink($MESSAGE_BODY_FILE);
    }
    open(MESSAGE, ">>$MESSAGE_BODY_FILE")  or die "Can't write to $MESSAGE_BODY_FILE :$!";         
    print MESSAGE "Dear WISEBED User ($user_unique_id)\n\n\n";
    print MESSAGE "Your experiment (#id=$experimentID)" ;
    print MESSAGE " was successfully executed on the " . $ENV{"TESTBED_NAME"} . " at " . $ENV{"ORGANIZATION_NAME"} . "\n\n";
    if(!($experiment_name eq "")) {
        print MESSAGE "Experiment name: $experiment_name\n\n";
    }
    print MESSAGE "Experiment description:\n\n";
    if(length(&Utils::trimchomp($description))>0) {
       print MESSAGE "\"$description\"\n\n";
    }
    else{
       print MESSAGE "<NO DESCRIPTION SUPPLIED>\n\n";
    }
    print MESSAGE "The experiment started at $start_time UTC and ";
    print MESSAGE "expired at $end_time UTC.\n\n";
    print MESSAGE "You can fetch your experiment results at the WISEBED website of " . $ENV{"TESTBED_NAME"} . " at " . $ENV{"ORGANIZATION_NAME"} . "\n";
    print MESSAGE "under the \"My Experiments\" tab ( " . $ENV{"TARWIS_GUI_URL"} ."/ExperimentSetup/expSetupFinExp.php#own" . $experimentID . " )\n\n\n";
    print MESSAGE "This message was generated by the TARWISServer Software of the WISEBED Testbed facilities at " . $ENV{"ORGANIZATION_NAME"} . "\n";
    close(MESSAGE);  
      
    my $attach = int($ENV{'ATTACH_RESULTS_TO_EMAIL'});
    my $max_size = int($ENV{'ATTACH_RESULTS_TO_EMAIL_MAX_SIZE'});
     
    if($attach && $resultsfilesize<$max_size) {  
	    my $zipfile = $database->getExperimentResultZipfile($experimentID);
	    my $filepath = &Utils::correctPath($ENV{"TARWIS_DIRECTORY"}."/".getFileName($experimentID, $experiment_name, $start_time, $end_time));
	    print "sending email WITH data attachement\n";  
	    print "results filepath=$filepath filesize=$resultsfilesize\n";  
	    &Utils::writeStringToFile($filepath, $zipfile);
	    sendEmailWithAttachement($email_address, $ENV{"SMTP_RELAY"}, $subject,  $MESSAGE_BODY_FILE, $filepath, $experimentID);
    }
    else {
    	print "sending email WITHOUT data attachement\n";  
        sendEmail($email_address, $ENV{"SMTP_RELAY"}, $subject,  $MESSAGE_BODY_FILE, $experimentID);
    }
}
    

sub getFileName {
    my $exp_id          = &Utils::trimchomp($_[0]);
    my $name            = &Utils::trimchomp($_[1]);
    my $start_time      = &Utils::replaceSomeChars(&Utils::trimchomp($_[2]));
    my $end_time        = &Utils::replaceSomeChars(&Utils::trimchomp($_[3]));

    my $expFilename = "experiment#("; 
    
    if(length($name) < 200) {
          $name = &Utils::replaceSomeChars($name);
          $expFilename = $expFilename . "name=" . $name . ",";
    }
    $expFilename = $expFilename . "id=$exp_id,start=$start_time,end=$end_time).zip";	
    return $expFilename;
}

sub sendEmail {
    my $receiver        = $_[0];
    my $smtp_relay      = $_[1];
    my $subject         = $_[2];
    my $message_file    = $_[3];
    my $expid           = $_[4];
    
    
    system("perl " . $ENV{'TARWIS_DIRECTORY'} . "sendEmail.pm -f $SENDER_ADDRESS -t $receiver -u \"$subject\" -o message-file=$message_file -s $smtp_relay:25");
    if(-e $message_file){
    	unlink($message_file);
    }
    
    system ("rm -rf $ENV{'TARWIS_DIRECTORY'}/experiment#*,id=$expid*");
}
        
        
sub sendEmailWithAttachement {
    my $receiver        = $_[0];
    my $smtp_relay      = $_[1];
    my $subject         = $_[2];
    my $message_file    = $_[3];
    my $attachement     = $_[4];
    my $expid           = $_[5];
    
    
    my $cmd = "perl " . $ENV{'TARWIS_DIRECTORY'} . "sendEmail.pm -f $SENDER_ADDRESS -t $receiver -u \"$subject\" -o message-file=$message_file -s $smtp_relay:25";
    $cmd .= " -a \"$attachement\" ";
    # print "EmailClient::sendEmailWithAttachement calling: $cmd\n";
    system($cmd);
   
      
    if(-e $attachement){
       unlink($attachement);
    }
    if(-e $message_file){
       unlink($message_file);
    }
    
    system ("rm -rf $ENV{'TARWIS_DIRECTORY'}/experiment#*,id=$expid*");
}
      
1;
