#!/usr/bin/perl

use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use IPC::ShareLite;
use Env;
use Pod::WSDL;
use File::ReadBackwards;
use Time::HiRes qw(gettimeofday usleep);

our $MSB_COMMAND_PORT               = 4000;   

readConfiguration("/etc/TARWIS.conf");

push(@INC, $ENV{"TARWIS_DIRECTORY"});
push(@INC, $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION");

require "UBERNUtils.pm";


our $CURRENT_DIRECTORY = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/LOCAL/");
our $RESET_SCRIPT = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/ResetMSBNode.pl");
our $FLASH_SCRIPT = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/DeployMSBImage.pl");

push(@INC, $CURRENT_DIRECTORY);

if($ARGV[0] eq "" || $ARGV[0] !~ m/msb/ ) {
    print "usage: Viewer.pl <msbX>\n"; 
    exit(1)
}


my $MILLIS = 100;
my $ID     = $ARGV[0];
my $FILE   = $ID.".txt";
my $LINES  = 10;

our $answer = `/bin/ps ax | grep \"node_output_handler_node_$ID\" | wc -l`;
if(int($answer) <= 1){
	print "ERROR! no output handler running!\n";
	print "start corresponding node output handler first (runLocal.sh)!\n";
	exit(1);
}

$SIG{INT}  = \&exitProcess;
$SIG{KILL} = \&exitProcess;
$SIG{TERM} = \&exitProcess;

sub exitProcess {
   print "signal caught! exiting\n";
   exit(1);  
}

our $lastLine = -1;


my $pid = fork();
if($pid!= 0){

    print "####################################################################\n";
    print "STARTING INTERFACE TO: $ID \n";
	print "####################################################################\n";
    
	while(1) {
		&UBERNUtils::flushFile($CURRENT_DIRECTORY.$FILE) unless( -e $CURRENT_DIRECTORY.$FILE );
		my $bw = File::ReadBackwards->new($CURRENT_DIRECTORY. $FILE ) or die "can't read '$FILE' $!" ;
	    my $log_line;
	    my $count = 0;
        if(defined( $log_line = $bw->readline ) ) {
        	my @tokens      = split(/\|/,$log_line);
	        my $line_number = int(@tokens[0]);
            my $time        = int(@tokens[1]);
            my $line       = &UBERNUtils::trimchomp(@tokens[2]) . "\n";
            my @lines = ();
            my @line_numbers = ();
            if($line_number > $lastLine) {
                push(@lines, $line); 
                push(@line_numbers, $line_number); 
            }    
	        while(defined($log_line = $bw->readline) && ($line_number-1) > $lastLine){
	            my @tokens     = split(/\|/,$log_line);
                $line_number   = int(@tokens[0]);
                my $time       = int(@tokens[1]);
                my $line       = &UBERNUtils::trimchomp(@tokens[2]) . "\n";
                push(@lines, $line); 
                push(@line_numbers, $line_number); 
	        }
	        if(@line_numbers>0) {
	           @line_numbers = reverse @line_numbers;
               @lines = reverse @lines;
	           $lastLine = @line_numbers[@line_numbers-1];
	           for(my $i=0; $i<@lines; $i++){
	           	   print sprintf("%06d",@line_numbers[$i]) . "|" . @lines[$i];
	           }
	        }
	    }
	    usleep($MILLIS * 1000);
	}
}
else {
	 while(1){
	    my $userinput =  <STDIN>;
	    print "entered command: $userinput\n";
	    if ( $userinput =~ m/RESET/ ) {
            system("perl $RESET_SCRIPT $ID");
        }
        elsif ( $userinput =~ m/FLASH\_NODE/ ) {
        	$userinput =~ s/FLASH\_NODE//;
        	$userinput = &UBERNUtils::trimchomp($userinput);
            system("perl $FLASH_SCRIPT $ID $userinput");
        }
        elsif ( $userinput =~ m/FND/ ) {
            $userinput =~ s/FND//;
            $userinput = &UBERNUtils::trimchomp($userinput);
            system("perl $FLASH_SCRIPT $ID $userinput");
        }
        else{
            sendCommand($ID, $userinput);
        }
        usleep($MILLIS * 1000);
	 }
}

sub readConfiguration {
     my $conf = $_[0];
     if(!-e $conf || !-r $conf) {
        print "Configuration file: $conf does not exist!\n";
        return 0;
     }
     open(CONFIG, $conf)  or return 0;   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

sub sendCommand {
    my $id      = $_[0];
    my $command = $_[1];
    my $number = $id;
    $number =~ s/msb//;
    $number = int($number)-1;
    my $hostname = "ee0" . int($number) . ".unibe.ch";
    &UBERNUtils::sendData_flush($hostname, $MSB_COMMAND_PORT, $command); 
}
