require Win32::SerialPort;
use Time::HiRes qw(usleep);
use Env;
use IO::Socket::INET;

# use Win32::MMF::Shareable;

require "Utils.pm";

&Utils::readConfiguration("C:\\msb\\msb.conf");
my $COMPORT = $ENV{'COMPORT'};
print "COMPORT is: $COMPORT\n";

$listenPort = 4000;

print "process id is: $$\n";

# install signal handler
$SIG{QUIT} = \&gotSignal;

sub gotSignal {
	print "SIGQUIT! received signal QUIT\n";
	print "exiting!\n";
	exit(0);
}

# save the process id here
my $id_file = "C:\\msb\\procid.txt";
if ( -e $id_file ) {
	unlink($id_file);
}

open( PROCID, ">>C:\\msb\\procid.txt" );
print PROCID $$;
close(PROCID);

my $DESTINATION_IP   = $ENV{'DESTINATION_IP'};
my $DESTINATION_PORT = $ENV{'DESTINATION_PORT_BASE'};

my $hostID = &Utils::getHostID();
print "HostID: $hostID\n";
my $sensorID = "msb" . ( int($hostID) + 1 );
print "SensorID: $sensorID\n";
my $hostIP = &Utils::getHostIP();
print "HostIP: $hostIP\n";

$timestamp = &Utils::getTimeInMySQLFormat();
$usecs     = &Utils::getUsecs();
print "time: $timestamp usecs: $usecs\n";

$DESTINATION_PORT += ( int($hostID) + 1 );

print "DESTINATION_IP: " . $DESTINATION_IP . "\n";
print "DESTINATION_PORT: " . $DESTINATION_PORT . "\n";

our $BINARY_FILE     = "C:\\msb\\file.hex";
our $RESET_FLAG_FILE = "C:\\msb\\resetScriptAfterFlash.dat";

$served = &Utils::verifyPort( $DESTINATION_IP, $DESTINATION_PORT );
if ($served) {
	print "destination port is served!\n";
}
else {
	print "destination port is NOT served!\n";
}

if ( -e $RESET_FLAG_FILE ) {
	unlink($RESET_FLAG_FILE);
}

my $pid = fork();
if ( $pid == 0 ) {
	open( $lockfile, ">lock.dat" );
	my $PortObj = new Win32::SerialPort( "$COMPORT", $quiet, $lockfile )
	  or die "Can't open $PortName: $!\n";
	my $configuration = "settings.conf";
	$PortObj->baudrate(115200);
	$PortObj->parity("none");
	$PortObj->databits(8);
	$PortObj->stopbits(1);
	$PortObj->write_settings;
	$PortObj->save($configuration);
	$PortObj->close();
	$PortObj = tie( *FILEHANDLE, 'Win32::SerialPort', $configuration ) || die;

	my $command_input_time                      = 0;
	my $command_input_done_waiting_for_reaction = 0;

	tie my $received_command_lock, "Win32::MMF::Shareable",'received_command_lock';
	tie my $received_data, "Win32::MMF::Shareable", 'received_data';

	tie my $sensor_output_lock, "Win32::MMF::Shareable", 'sensor_output_lock';
	tie my @sensor_output_data, "Win32::MMF::Shareable", 'sensor_output_data';

	my $char;

	while (1) {
		$char = $PortObj->lookfor();
		if ($char) {
			$command_input_done_waiting_for_reaction = 0;
			@lines                                   = ();
			$text = &Utils::finalizeToken($char);
			push( @lines, $text );
			while ($sensor_output_lock) {
				usleep(10000);
				$char = $PortObj->lookfor();
				if ($char) {
					$text = &Utils::finalizeToken($char);
					push( @lines, $text );
				}
			}
			@sensor_output_data = @lines;
			$sensor_output_lock = 1;
		}
		else {
			if ($command_input_done_waiting_for_reaction) {
				my $dt = ( time() - $command_input_time );

				# print "WAITING! waiting for reaction time dt=$dt\n";
				if ( time() - $command_input_time > 5 ) {
					$PortObj->close();
					system( "echo flashed > " . $RESET_FLAG_FILE );
					killFather();
				}
			}

			if ($received_command_lock) {
				my $record = $received_data;
				if ( $record =~ m/RESET/ ) {
					system("C:\\msb\\resetNode.bat");
				}
				elsif ( $record =~ m/FLASH\_NODE/ || $record =~ m/FND/ ) {
					print "flashing node\n";
					system( "echo flashed > " . $RESET_FLAG_FILE );
					my $flashpid = fork();
					if ( $flashpid == 0 ) {
						system("C:\\msb\\flashNode.bat");
					}
					else {
						sleep(20);
						$PortObj->close();
						killFather();
					}
				}
				else {
					print "writing to port: $COMPORT\n";
					print "command: $record\n";
					$PortObj->write( $record . "\n" );
					$command_input_time                      = time();
					$command_input_done_waiting_for_reaction = 1;
				}

				# release token
				$received_command_lock = 0;
			}
			else {
				usleep(10000);
			}
		}
	}
}
else {
	my $pid = fork();
	if ( $pid == 0 ) {

		tie my $received_command_lock, "Win32::MMF::Shareable",'received_command_lock';
		tie my $received_data, "Win32::MMF::Shareable", 'received_data';

		print "Starting to listen at $hostIP:$listenPort for COMMANDS\n";
		my $sock = new IO::Socket::INET(
			LocalHost => $hostIP,
			LocalPort => $listenPort,
			Type      => SOCK_STREAM,
			Listen    => 50,
			Reuse     => 1,
			Proto     => 'tcp'
		);
		while ( my $client = $sock->accept() ) {
			my $data = "";
			while ( defined( my $buf = <$client> ) ) {
				$data .= $buf;
			}

			# wait while still locked!
			while ($received_command_lock) {
				usleep(10000);
			}

			# write data when lock is released
			$received_data = $data;

			# set lock to signalize finish status!
			$received_command_lock = 1;
		}
	}
	else {
		my $pid = fork();
		if ( $pid == 0 ) {
			print "Starting to listen at $hostIP:" . int( $listenPort + 1 ). " for binary code files\n";
			my $sock = new IO::Socket::INET(
				LocalHost => $hostIP,
				LocalPort => ( $listenPort + 1 ),
				Type      => SOCK_STREAM,
				Listen    => 50,
				Reuse     => 1,
				Proto     => 'tcp'
			);
			while ( my $client = $sock->accept() ) {
				my $data = "";
				while ( defined( my $buf = <$client> ) ) {
					$data .= $buf;
				}
				if ( length($data) > 1000 ) {
					print "received binary size: " . length($data) . "\n";
					&Utils::flushFile($BINARY_FILE);
					&Utils::decodeAndWriteToFile( $data, $BINARY_FILE );
					print "received and decoded binary - done\n";
				}
			}
		}
		else {

			tie my $sensor_output_lock, "Win32::MMF::Shareable",'sensor_output_lock';
			tie my @sensor_output_data, "Win32::MMF::Shareable",'sensor_output_data';

			while (1) {
				if ($sensor_output_lock) {
					my $success = &Utils::sendDataChunks_flush(
						$DESTINATION_IP, $DESTINATION_PORT,
						$sensorID,       \@sensor_output_data
					);
					if ( !$success ) {
						print "could not deliver data to $DESTINATION_IP:$DESTINATION_PORT \n";
					}
					else {
						print "sent data. time: " . &Utils::getTimeInMySQLFormat() . " " . &Utils::getUsecs() . "\n";
					}
					$sensor_output_lock = 0;
				}
				usleep(50000);
			}
		}
	}
}

sub killFather {
	my $id_file = "C:\\msb\\procid.txt";
	open FILE, "<", $id_file or die $!;
	my @lines = <FILE>;
	if ( @lines > 0 ) {
		my $proc = int( @lines[0] );
		print "killing proc: $proc\n";
		my $status = kill( 3, $proc );
		exit(0);
	}
	else {
		print "ERROR: no pid in file $id_file!\n";
	}
}

sub reopenSerial {
	my $p = tie( *FILEHANDLE, 'Win32::SerialPort', $configuration ) || die;
	return $p;
}
