use SOAP::Lite + 'trace'; 

use strict;
use IPC::ShareLite;
use HTTP::Request;
use HTTP::Response;
use Env;
use Pod::WSDL;
use Data::Dumper;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use Time::HiRes qw(gettimeofday);

require "UBERNUtils.pm";


my @urns     = ( "urn:wisebed:node:ubern:1",
                 "urn:wisebed:node:ubern:2" 
               );
my @status   = ( 1,
                 1 
                 );

my @urns_2   = ( "urn:wisebed:node:ubern:3",
                 "urn:wisebed:node:ubern:4" );
my @status_2 = ( 0,0 );


my $soap_message1 = generateMultipleComplexStatusMessage("req#1", \@urns, \@status, "status update 1");
my $soap_message2 = generateMultipleComplexStatusMessage("req#2", \@urns_2, \@status_2, "status update 2");
my $soap_message3 = generateMultipleComplexStatusMessage("req#3", \@urns_2, \@status_2, "status update 2");
my $soap_message4 = generateMultipleComplexStatusMessage("req#4", \@urns_2, \@status_2, "status update 2");
my $soap_message5 = generateMultipleComplexStatusMessage("req#5", \@urns_2, \@status_2, "status update 2");

getControllerServiceDriver()->receiveStatus($soap_message1);

sub getControllerServiceDriver {
   my $driver = SOAP::Lite->service("file:/opt/TARWISServer/WSDL/ControllerService.wsdl");
   $driver->uri('urn:ControllerService');
   return $driver;
}

sub generateMultipleComplexStatusMessage {
  
   my $requestId     = $_[0];
   my @urns          = @{$_[1]};
   my @status        = @{$_[2]};
   my $msg           = $_[3];
 
   my @soap_objects = ();
    for(my $i=0; $i<@urns; $i++) {
       my $soap_status = generateSoapStatus(@urns[$i],@status[$i], $msg);
       push(@soap_objects, $soap_status);
   }
   my $soap_obj_urn = SOAP::Data->new(name => "requestId",   value => $requestId,     type => "string");
   push(@soap_objects, $soap_obj_urn);
  
   my @result = ();
   for(my $i=0; $i<@soap_objects; $i++) {
     my $soap_obj = SOAP::Data->value(@soap_objects[$i]);
     push(@result, $soap_obj);
   }                                       
   my $finalMessage = SOAP::Data->name("status" =>  \SOAP::Data->value(  @result ) ) ;
   return $finalMessage;
}

sub generateSoapStatus {
    my $nodeId = $_[0];
    my $value  = $_[1];
    my $msg    = $_[2];
    
    my $soap_obj = SOAP::Data->name("status" => \SOAP::Data->value( SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                    SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                    SOAP::Data->new(name => "msg", value => $msg , type => "string"))); 
    return $soap_obj;
}

sub deliverMessageSOAP {
   my $urn       = $_[0];
   my $msg       = $_[1];
   my $soap_message = generateComplexTypeMessageText($urn, $msg);
   getControllerServiceDriver()->receive($soap_message);
}

sub generateComplexStatusMessage {
  
   my $requestId     = $_[0];
   my $nodeId        = $_[1];
   my $value         = $_[2];
   my $msg           = $_[3];
 
   my $soap_message = SOAP::Data->name("status" => \SOAP::Data->value( 
                                                  SOAP::Data->new(name => "requestId",  value => $requestId), 
                                                  SOAP::Data->name("status" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                                SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                                SOAP::Data->new(name => "msg", value => $msg , type => "string")
                                                                                                  )
                                                                  ) )
                                       ); 
   return $soap_message;                                    
}

sub generateComplexTypeMessageText {
  
   my $node_urn      = $_[0];
   my $textData      = $_[1];
   my $soap_datetime = generateSOAPDateTimeObject(getTime(), 0);
   
   my $soap_message = SOAP::Data->name("msg" => \SOAP::Data->value( 
                                                     SOAP::Data->new(name => "sourceNodeId",  value => $node_urn, type => "string"), 
                                                     SOAP::Data->new(name => "timestamp",     value => $soap_datetime, type => "dateTime"), 
                                                     SOAP::Data->new(name => "binaryData",    value => &UBERNUtils::encodeMessage($textData), type => "base64Binary"), 
                                                     )
                                       ); 
   return $soap_message;                                    
}

sub generateSOAPDateTimeObject {
   my $myTime = $_[0];
   my $microseconds = $_[1];
   my $date = ParseDate($myTime);
   my $datestr = UnixDate($date, "%a %b %e %H:%M:%S %z %Y"); 
   my $soap_datetime = ConvertDate($datestr);
   my $date_including_microseconds;
   if(int($microseconds)>0){
     $date_including_microseconds = sprintf("%s.%06d", $soap_datetime, $microseconds);
   }
   else {
     $date_including_microseconds = $soap_datetime;
   }
   return $date_including_microseconds;
}

sub getTime {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   print "time=" . time() . "\n";
   my $time = sprintf("%4d-%02d-%02dT%02d:%02d:%02dZ", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}
