package SessionManagementService;


use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use LWP::UserAgent;
use File::Slurp; 
use HTTP::Request;
use HTTP::Response;
use Env;
use Pod::WSDL;
use Data::Dumper;
use Net::Ping;
use Coro;
use Storable;
use Data::Dumper;
use XML::Simple;

our $raw_xml_soap_request;
our $outfile;
our $errfile;

our $MAX_LINES                      = 300;
our $CURRENT_DIRECTORY              = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'}."/UBERN_TESTBED_IMPLEMENTATION");  
our $WISEML_FILE                    = &UBERNUtils::correctPath($CURRENT_DIRECTORY."/UBERN_TESTBED.wiseml");
our $WSN_SERVICE_IP_ADDRESS         = &UBERNUtils::getThisMachineIP();
our $WSN_SERVICE_PORT_BASE          = 5000;
our $WSN_SERVICE_PORT_ENTRIES       =  100;

our %instances = ();
our $network_wiseml;

require "UBERNUtils.pm";
require "CommonTypes.pm";

readConfiguration("/etc/RS.conf");
readConfiguration("/etc/SNA.conf");
push(@INC, $ENV{"RESERVATION_SYSTEM_DIRECTORY"});

require "ReservationDatabase.pm";

sub readConfiguration
{
     my $conf = $_[0];
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}


sub setOutfile {    
    $outfile = $_[0];
    print "setting outfile to: $outfile\n";
}

sub setErrfile {    
    $errfile = $_[0];
    print "setting errfile to: $errfile\n";
}

sub setRawXML_SOAPRequest {
    $raw_xml_soap_request = $_[0];
}

sub checkKeyValidity {
	my $key = $_[0];
	my $dbObject = new ReservationDatabase();
	my @rows = $dbObject->dbGetReservationsOfReservationKey($key);
	my $timeNow = &UBERNUtils::get_UTC_time();
	
    	print "checkKeyValidity: checking for reservations with key=$key now=$timeNow\n";
    
	if(@rows > 0) {
		for(my $i=0; $i<@rows; $i++)
	    {
		     my @singlerow = @{$rows[$i]};
		     
		     my $urn       = @singlerow[0];
		     my $fromRes   = @singlerow[1];
		     my $toRes     = @singlerow[2];
		     my $userData  = @singlerow[4];
		     
             my $fromTime  = &UBERNUtils::parseDate($fromRes); 
             my $toTime    = &UBERNUtils::parseDate($toRes); 
             
             print "urn: $urn reserved from $fromRes ($fromTime) to $toRes ($toTime)\n";
             
             if($fromTime <= $timeNow && $toTime >= $timeNow) {
             	 print "key valid to call getInstance NOW!\n";
             }
             else {
             	 print "key invalid to call getInstance NOW!\n";
             	 return 0;
             }
	    }
	    return 1;
	}
	else {
		print "checkKeyValidity: no reservation with key=$key found\n";
		return 0;
	}
}


sub getNetwork {
    checkSOAPRequest(@_);
    
    # perl sends: 
    # <?xml version="1.0" encoding="UTF-8"?><soap:Envelope xmlns:ct="urn:CommonTypes" 
    # soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
    # xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    # xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
    # xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    # xmlns:tns="urn:SessionManagementService"><soap:Body><tns:getNetwork xsi:nil="true" 
    # /></soap:Body></soap:Envelope>
    
    
    # java sends:
    # <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:urn="urn:CommonTypes">
    # <soapenv:Header/>
    # <soapenv:Body>
    # <urn:getNetwork/>
    # </soapenv:Body>
    # </soapenv:Envelope>
    
    print "getNetwork called at the SessionManagementService\n";
    print "WSN_SERVICE_IP_ADDRESS: $WSN_SERVICE_IP_ADDRESS\n";
    if($network_wiseml eq ""){
        $network_wiseml = read_file($WISEML_FILE);
    }
    my $network_soap = SOAP::Data->name("return")->value($network_wiseml);
    return $network_soap;
}

sub getConfiguration { 
    print "SessionManagementService::getConfiguration called " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n"; 
    checkSOAPRequest(@_);
    my $rs_url = SOAP::Data->name("reservationSystemEndpointUrl")->value($ENV{'RESERVATION_SYSTEM_SERVICE_URL'});
    my $sna_url = SOAP::Data->name("snaaEndpointUrl")->value($ENV{'SNA_AUTHORIZATION_URL'});
    return ($rs_url, $sna_url);
    print "SessionManagementService::getConfiguration end " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n"; 
}

sub getNodes {
    checkSOAPRequest(@_);
    print "getNodes called at the SessionManagementService\n";
    
    my $graphDescription = read_file($WISEML_FILE);
    
    my $xml = new XML::Simple;
    my $data;
    eval{
        $data =  $xml->XMLin($graphDescription, KeyAttr => { attributes => "+id" });
    };
    if ($@) {
        print "SessionManagementService::getNetwork returns invalid XML Syntax!\n";
        print "error: $@\n";
        print "exiting...\n";
        exit(1);
    }; 
    
    my $default_type_content = ""; 
    if(   &Utils::checkTagExists($data, "setup") 
       && &Utils::checkTagExists($data->{setup}, "defaults") 
       && &Utils::checkTagExists($data->{setup}->{defaults}, "node") 
       && &Utils::checkTagExists($data->{setup}->{defaults}->{node}, "nodeType") ) {
       $default_type_content = $data->{setup}->{defaults}->{node}->{nodeType};
    }
    
    
    my @node_references = ();
    if(ref($data->{setup}->{node}) eq 'HASH' ){
       push(@node_references, $data->{setup}->{node});
    }
    else {
      foreach my $node (@{$data->{setup}->{node}})  {
         push(@node_references, $node);
      }
    }
    
    my @nodes_soap = ();
    
    # iterate through the found node references and generate simpler tags (e.g. for IPhone APP)
    foreach my $node (@node_references) {
        my $node_type = $default_type_content;
        if( &Utils::checkTagExists($node, "nodeType") ) {
            $node_type = $node->{nodeType};
        } 
        my $id = $node->{id};
        print "adding node: $id\n";
        my $node_soap = SOAP::Data->name("node")->attr({id => $id, nodeType => $node_type}) ;   
        push(@nodes_soap, $node_soap);
    }
    
          
	&UBERNUtils::truncateFile($outfile, $MAX_LINES);   
	&UBERNUtils::truncateFile($errfile, $MAX_LINES);
    
    print "getNodes called at the SessionManagementService end\n";
    return @nodes_soap;
}

our $portCounter = 0;

sub getInstance {
    checkSOAPRequest(@_);
    print "getInstance called at the SessionManagementService\n";
    
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getInstance"); 
    
    # print "SOAP: $raw_xml_soap_request\n"; 
    # print "MethodName: $method_name\n";
  
    
    my @reservationHashes = &UBERNUtils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "secretReservationKey");
    my $reservation_id; 
    
    if(&UBERNUtils::checkIfPropertyIsDefined(@reservationHashes[0], "secretReservationKey")) {
        $reservation_id = &UBERNUtils::getValue(@reservationHashes[0]->{secretReservationKey});
    }
    else {
    	print "ERROR! secretReservationKey is invalid!\n";
    }
     
    my $controller_url = $_[1];
    
    print "retrieved res_id: $reservation_id, url: $controller_url\n";
    
    if(checkKeyValidity(&UBERNUtils::trimchomp($reservation_id))) {  
	    my $instance_url_soap;
	    my $port = $WSN_SERVICE_PORT_BASE + $portCounter;
	    
	    print "checking port: $port\n";
	    my $secret_url = "http://" . $WSN_SERVICE_IP_ADDRESS . ":" . $port;
	    my $instance_url_soap = SOAP::Data->name("return")->value($secret_url);
	    
	    if(!&UBERNUtils::portChecker($WSN_SERVICE_IP_ADDRESS, $port)) {
	        $portCounter = ($portCounter + 1) % $WSN_SERVICE_PORT_ENTRIES;
	        print "port $port is free. portCounter now: $portCounter\n";
	        my $wsnService = &UBERNUtils::daemonize(\&UBERN_SOAPServer::WSNServiceSOAPServer, "WSNServiceSOAPServer_" . $port . "_" . $reservation_id, $port, $reservation_id);
	        $instances{$reservation_id} = $wsnService;
	        sleep(2);
	        print "returning $secret_url\n";       
	        &UBERNUtils::truncateFile($outfile, $MAX_LINES);   
	        &UBERNUtils::truncateFile($errfile, $MAX_LINES);
	        return $instance_url_soap;    
	    }
	    
	    print "ERROR! No empty port found! EXITING!\n";
	    exit(1);	    
    }
    else {
    	print "key invalid... not forking new WSNService!\n";
    	my $secret_url = "ERROR! invalid key!";
    	my $instance_url_soap = SOAP::Data->name("return")->value($secret_url);
    }
}


sub free {
    checkSOAPRequest(@_);
    print "free called at the SessionManagementService\n";   
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "free"); 
    # print "SOAP: $raw_xml_soap_request\n"; 
    # print "MethodName: $method_name\n";
    my @reservationHashes = &UBERNUtils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "secretReservationKey");
    my $reservation_id;
    if(&UBERNUtils::checkIfPropertyIsDefined(@reservationHashes[0], "secretReservationKey")) {
        $reservation_id = &UBERNUtils::getValue(@reservationHashes[0]->{secretReservationKey});
    }
    else {
        print "ERROR! secretReservationKey is invalid!\n";
    }
    print "retrieved res_id: $reservation_id\n";
    $instances{$reservation_id}->kill("SIGUSR1");
    $instances{$reservation_id}->wait();
    print "killed instance with res_id: $reservation_id\n";   
    
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);   
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
}


sub areNodesAlive {
    
    checkSOAPRequest(@_); 
 
    print "SessionManagementService::areNodesAlive called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "areNodesAlive"); 
    
    print "SOAP: $raw_xml_soap_request\n"; 
    print "MethodName: $method_name\n";
    
    my @urns = &UBERNUtils::getItemsFromSOAPBody($raw_xml_soap_request, $method_name, "nodes");
    
    foreach my $urn (@urns){
    	print "urn: $urn\n";
    }
    
    my $requestID = generateRequestID();
    
    my $requestID_soap = SOAP::Data->name("return")->value($requestID);
    
    print "SessionManagementService::areNodesAlive end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    
    return $requestID_soap;
}

our $requestIDCounter = 0;

sub generateRequestID {
    $requestIDCounter++;
    return "request#aliveStatus#$requestIDCounter";
}


sub areNodesAliveOperations {
    
    print "SessionManagementService::areNodesAliveOperations called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    my $node_urns = $_[0];
    my $requestID = $_[1];
    
    my @status = ();
    foreach my $urn (@$node_urns){
       my $id = &UBERNUtils::stripeIDfromURN($urn);
       my $meshnode_hostname = getMeshNodeOfSensorNode($id);
       my $ping_succeeded = pingMeshnode($meshnode_hostname);
       print "ping_succeeded at $meshnode_hostname = $ping_succeeded\n";
       push(@status, $ping_succeeded );
    }
    
    
    deliverMultipleStatus_SingleCall($node_urns, \@status, $requestID);
    
    print "SessionManagementService::areNodesAliveOperations ended " . &UBERNUtils::getTimeInMySQLFormat() . "\n"; 
}  


sub pingMeshnode {
    my $ip_or_hostname = $_[0];
    my $p = Net::Ping->new( "icmp", 1, 64 );
    my $alive = $p->ping($ip_or_hostname);
	$p->close();
	return $alive;    
}


sub getMeshNodeOfSensorNode {
    my $sensorNode_id = $_[0];
    if( $sensorNode_id =~ m/msb/ ) {   
        my $number = $sensorNode_id;
        $number =~ s/msb//;
        $number = int($number)-1;
        # determine the host
        my $hostname = "ee0" . int($number);
    	return $hostname;
    }
    else {
    my $mesh_id = $sensorNode_id-1;
    if($mesh_id<10) {
       $mesh_id = "0".$mesh_id;
    }
    return "wb$mesh_id";
    }
}

our $MAX_ATTEMPTS = 2;

sub deliverMultipleStatus_SingleCall {
   my @urns      = @{$_[0]};
   my @status    = @{$_[1]};
   my $requestID = $_[2];
   
   print "deliverMultipleStatus_SingleCall requestId: $requestID " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
   my $soap_message = generateMultipleComplexStatusMessage($requestID, \@urns, \@status, "status update");
   
   my $failed = 0;
      
   for(my $i=0; $i<$MAX_ATTEMPTS; $i++) {
	   eval{
	       getControllerServiceDriver()->receiveStatus($soap_message);
	   };
	   if ($@) {
	       print STDERR "WSNService::deliverMultipleStatus_SingleCall failed! ". &UBERNUtils::getTimeInMySQLFormat() . "\n";
	       print STDERR "urns: @urns\n";
	       if($i<$MAX_ATTEMPTS) {
	       	   print STDERR "retrying...\n";
	       }
	       $failed = 1;
	   };
	   
	   if(!$failed) {
	   	   print "deliver status succeeded!\n";
           last;
       }
   }
   
   print "deliverMultipleStatus_SingleCall end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
}

our $ControllerServiceDriver = 0;

sub getControllerServiceDriver {
   if($ControllerServiceDriver == 0) {
	   my $client = SOAP::Lite->new();
	   $client->proxy($ENV{'CONTROLLER_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
       $ControllerServiceDriver = $client->service("file:" . &Utils::correctPath($ENV{'TARWIS_DIRECTORY'}. "/WSDL/ControllerService.wsdl"));
           
   }
   return $ControllerServiceDriver;
}


sub generateSoapStatus {
	my $nodeId = $_[0];
    my $value  = $_[1];
    my $msg    = $_[2];
    
	my $soap_obj = SOAP::Data->name("status" => \SOAP::Data->value( SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                    SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                    SOAP::Data->new(name => "msg", value => $msg , type => "string"))); 
    return $soap_obj;
}


sub generateMultipleComplexStatusMessage {
  
   my $requestId     = $_[0];
   my @urns          = @{$_[1]};
   my @status        = @{$_[2]};
   my $msg           = $_[3];
 
   # first the status' 
   my @soap_objects = ();
    for(my $i=0; $i<@urns; $i++) {
       my $soap_status = generateSoapStatus(@urns[$i],@status[$i], $msg);
       push(@soap_objects, $soap_status);
   }
   # then the requestID
   my $soap_obj_urn = SOAP::Data->new(name => "requestId",   value => $requestId,     type => "string");
   push(@soap_objects, $soap_obj_urn);
  
   # then the resulting final message
   my @result = ();
   for(my $i=0; $i<@soap_objects; $i++) {
     my $soap_obj = SOAP::Data->value(@soap_objects[$i]);
     push(@result, $soap_obj);
   }                                       
   my $finalMessage = SOAP::Data->name("status" =>  \SOAP::Data->value(  @result ) ) ;
   return $finalMessage;
}

sub generateComplexStatusMessage {
  
   my $requestId     = $_[0];
   my $nodeId        = $_[1];
   my $value         = $_[2];
   my $msg           = $_[3];
 
   my $soap_message = SOAP::Data->name("status" => \SOAP::Data->value( 
                                                  SOAP::Data->new(name => "requestId",  value => $requestId), 
                                                  SOAP::Data->name("status" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                                SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                                SOAP::Data->new(name => "msg", value => $msg , type => "string")
                                                                                                  )
                                                                  ) )
                                       ); 
   return $soap_message;                                    
}



sub checkSOAPRequest() {
    if($_[0] eq "SessionManagementService" || $_[0] eq "CommonTypes") {
       for(my $i=0; $i<(@_-1); $i++) {
          $_[$i] = $_[$i+1];
       }
    }
}




1;
