package UBERNUtils;
use strict;
use POSIX qw(setsid);
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Date::Parse;
use MIME::Base64 qw( encode_base64 decode_base64 );
use File::Slurp;
use Data::Dumper;
use Time::HiRes qw(gettimeofday); 
use Net::Ping;
use Proc::Simple;
use Date::Parse;
use Time::Zone;
use IO::Socket::INET;
use CGI;
use File::Basename;
use Archive::Zip;
use File::Copy;
use Fcntl qw/ :flock /;

sub appendStringToFile {
   my $filename = $_[0];
   my $string   = $_[1];
   my $output = new IO::File(">>$filename");
   print $output $string;
   $output->close();
}

sub encodeMessage {
   my $text = $_[0];
   my $encoded=encode_base64($text);
   return $encoded;
}

sub decodeMessage {
   my $text = $_[0];
   my $decoded=decode_base64($text);
   return $decoded;
}

    
sub sendData_flush {
    my $DESTINATION_IP   = $_[0];
    my $DESTINATION_PORT = $_[1];
    my $text             = $_[2];

    my $sock = new IO::Socket::INET(PeerAddr=>$DESTINATION_IP,PeerPort=>$DESTINATION_PORT,Proto=>'tcp');
    if($sock) {
        $sock->send($text);
        $sock->close();
        return 1;
    }
    else {
        print "could not send data, destination $DESTINATION_IP:$DESTINATION_PORT not reachable\n";
        return 0;
    }
}


sub parseDate {
   my $time = $_[0];
   return str2time($time); 
}

sub get_UTC_time() {
    use Time::Zone;
    my $offset = tz_offset(0);
    return (time() - $offset);
}

sub pingMeshnode {
    my $ip_or_hostname = $_[0];
    my $p = Net::Ping->new( "icmp", 1, 64 );
    my $alive = $p->ping($ip_or_hostname);
    $p->close();
    return $alive;    
}

sub getEEPCHostnameOf {
	my $urn = $_[0];
	my $id    = &UBERNUtils::stripeIDfromURN($urn);
    my $eepc  = &UBERNUtils::getSensorNodeOfEEPC($id);
    my $number = $id;
    $number =~ s/msb//;
    $number = int($number)-1;
    # determine the host
    my $hostname = "ee0" . $number  . ".unibe.ch";
    return $hostname;
}

sub getMeshNodeHostnameOf {
    my $urn = $_[0];
    my $id    = &UBERNUtils::stripeIDfromURN($urn);
    my $eepc  = &UBERNUtils::getSensorNodeOfMeshNode($id);
    my $number = $id;
    $number =~ s/msb//;
    $number = int($number)-1;
    # determine the host
    my $hostname;
    if($number<10) {
        $hostname = "wb0" . $number  . ".unibe.ch";
    }
    else {
    	$hostname = "wb" . $number  . ".unibe.ch";
    }
    return $hostname;
}

#   Trims and chomps tabs/spaces/newlines

sub trimchomp
{
    my $string = $_[0];
    chomp($string);
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    chomp($string);
    return $string;
}

sub removeQuotes {
    my $string = $_[0];
    $string =~ tr/"//d;
    $string =~ tr/"//d;
    return $string;
}


sub checkTagExists {
    my $record = $_[0];
    my $tagName = $_[1];
    # print "checkTagExists: $tagName\n";
    my $dump = Dumper($record->{$tagName});
    # print "dump=" . $dump . "\n";
    if($dump =~ m/undef/ && scanDump($dump) == 0){
      # print "returning false\n";
      return 0;
    }
    else {
      # print "returning true\n";
      return 1;
    }
}

sub scanDump {
    my $dumpcount = 0; $dumpcount++ while $_[0] =~ /\{/g;
    return $dumpcount;
}


sub checkIfPropertyIsDefined {
    my $ref = $_[0];
    my $propname = $_[1];
    my $value;
    my $returnValue = 0;
    eval{
        $value = removeQuotes(&UBERNUtils::trimchomp($ref->{$propname}));
        if(length($value) > 0) {
            $returnValue = 1;
        }
        else {
            $returnValue = 0;
        }
    };
    if ($@){        
        $returnValue = 0;
    };      
    return $returnValue;
}

sub removeStuff {
    my $string = $_[0];
    $string =~ s/\$VAR[0-9]*\ =\ '//;
    $string =~ s/';$//;
    return &UBERNUtils::trimchomp($string);
}

sub getValue {
    my $reference    = $_[0];
    if( ref($reference) eq "" ){
        return trimchomp($reference);    
    }
    elsif(ref($reference) eq 'HASH' ){
        $reference = $reference->{content};
        return trimchomp($reference);
    }
    elsif(ref($reference) eq 'SCALAR') {
        return trimchomp("$reference");
    }
    die "getValue given wrong argument type!";
}

#   Returns 1 if argument is integer, 0 if not...

sub isInteger {
    if ( $_[0] =~ /^\d+$/ ) { 
       return 1;
    } 
    return 0;
}

sub getSensorNodeOfEEPC {
    my $tmp = $_[0];
    $tmp =~ s/[a-z]*//;
    $tmp =~ s/\..*//;
    my $eepc_id = int($tmp);
    return "msb" . ($eepc_id+1);
}


sub getSensorNodeOfMeshNode {
    my $tmp = $_[0];
    $tmp =~ s/[a-z]*//;
    $tmp =~ s/\..*//;
    my $mesh_id = int($tmp);
    return $mesh_id+1;
}

sub readConfiguration {
	 # read the configuration
     my $conf = $_[0];
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
}

sub generateURN {
   my $id = $_[0];
   my $urn = "urn:wisebed:node:ubern:" . $id;
   return $urn;
}

sub stripeIDfromURN {
   my $urn   = $_[0];
   # stripe away the testbed name
   $urn =~ s/.*://;
   return $urn;
}


#   sort the array according to the IDs in the URNs
sub sortURNs {
    my @urns = @_;
    my @urns_sorted = sort{&URN_sort_function}(@urns);
    return @urns_sorted;
}


sub URN_sort_function {
    if (int(stripeIDfromURN($a)) < int(stripeIDfromURN($b))) { -1; }
    elsif (int(stripeIDfromURN($a))==int(stripeIDfromURN($b))) { 0; }
    else { return 1; }
}
    

sub stripePropertyNamefromURN {
   my $urn   = $_[0];
   # stripe away the testbed name
   $urn =~ s/.*://;
   return $urn;
}

our $FIX_IP_GRIDLAB23 = "130.92.66.182";

sub getThisMachineIP {
   my $ifconfig="/sbin/ifconfig";
   my @lines=qx|$ifconfig eth0| or die("Can't get info from ifconfig: ".$!);
   foreach(@lines){
      if(/inet addr:([\d.]+)/){
         if($1 eq "") {
         	return $FIX_IP_GRIDLAB23;
         }
         return "$1";
      }
   }
}

sub containsElement {
   my $arrayRef   = $_[0];
   my $element = $_[1];
   for(my $i=0; $i<@$arrayRef; $i++){
      if($element eq @$arrayRef[$i]) {
         return 1;
      }
   }
   return 0;
}

sub indexOfElement {
   my $arrayRef   = $_[0];
   my $element = $_[1];
   for(my $i=0; $i<@$arrayRef; $i++){
      if($element eq @$arrayRef[$i]) {
         return $i;
      }
   }
   return 0;
}


#   Fork and daemonize a perl subroutine
#   write output and stderr to files

sub daemonize {
    
    my $service         = $_[0];
    my $serviceName     = $_[1];
    my $port            = $_[2];
    my $experiment_id   = $_[3];
    
    # redirect STDOUT and STDERR to outfiles
    
    my $outfile = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'}."/UBERN_TESTBED_IMPLEMENTATION/$serviceName.out");
    my $errfile = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'}."/UBERN_TESTBED_IMPLEMENTATION/$serviceName.err");
    
    # print "daemonize: outfile: $outfile errfile: $errfile\n";
    
    # fork process to background!
    my $myproc = Proc::Simple->new();  # Create a new process object
    $myproc->start($service, $outfile, $errfile, $port, $experiment_id);  
    return $myproc;
}


#   contains non-ASCII characters
sub containsBadEncoding {
    my $data = $_[0];
    my @chars=split(//,$data);
    for(my $i=0; $i<@chars; $i++){
       my $n = ord(@chars[$i]);
       if( ! ( ( $n >= 0 && $n <= 13) || ($n >= 32 && $n <= 127) ) ) {
           return 1;
       }
       
    }
    return 0;
}

#   contains non-ASCII characters
sub removeBadEncoding {
    my $data = $_[0];
    my $puredata = "";
    my @chars=split(//,$data);
    for(my $i=0; $i<@chars; $i++){
       my $n = ord(@chars[$i]);
       if( $n == 0 || $n == 9 || ($n >= 32 && $n <= 126) ) {
         $puredata .=  @chars[$i];
       }
    }
    return $puredata;
}



sub getTimeInMySQLFormat() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   my $time = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}


sub getTime_Hours_Minutes_Secs_Usecs() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   my ($epochseconds, $microseconds) = gettimeofday; 
   my $time = sprintf("%02d:%02d:%02d:%06d",$hour,$min,$sec, $microseconds);
   return $time;
}


#   for log files: cut off all lines of <arg1> if the file is bigger than <arg2> lines 



sub truncateFile {
    my $file = $_[0];
    my $lines = $_[1];

    if(-e $file) {
        open FILE, "<$file"        or die  "$0 [$$]: open: $!";
        flock FILE, LOCK_EX        or die  "$0 [$$]: flock: $!";
        my(@lines) = <FILE>;
        close(FILE);
        
        if(@lines <= $lines) {
            return;
        }
        else {
            unlink($file);
            chdir '/'                   or die "Can't chdir to /: $!";
            if($file =~ /\.out/) {
               open STDOUT, ">$file"    or die "Can't write to $file: $!"; 
               my $fileLines = @lines;
               for(my $i= $fileLines - ($lines/2); $i<@lines; $i++)
               {
                  print STDOUT @lines[$i];
               }
            }
            else {
               open STDERR, ">$file"    or die "Can't write to $file: $!"; 
               my $fileLines = @lines;
               for(my $i= $fileLines - ($lines/2); $i<@lines; $i++)
               {
                  print STDERR @lines[$i];
               }
            } 
        }
    }   
}



sub portChecker {
    my $host = $_[0];
    my $port = $_[1];
    my $sock = new IO::Socket::INET(PeerAddr=>$host,PeerPort=>$port,Proto=>'tcp');
    if($sock)
    {
       close $sock;
       return 1;
    }
    return 0;
}

sub verifyPort {
    my $host = $_[0];
    my $port = $_[1];
    my $portServed = portChecker($_[0],$_[1]);
    return $portServed;
}

sub readFile {
    my $filepath = $_[0];
    my $string;
    {
	   local $/=undef;
	   open FILE, "$filepath" or die "Couldn't open file: $!";
	   $string = <FILE>;
	   close FILE;
    }
   return $string;
}



sub encodeFile {
   my $filepath = $_[0];
   open INFILE, '<', $filepath;
   binmode INFILE;
   my $buf;
   my $encoded;
   while ( read( INFILE, $buf, 100000000 ) ) {
        $encoded.=encode_base64($buf);
   }
   close INFILE;
   return $encoded;
}

sub decodeFile {
   my $filepath = $_[0];
   open INFILE, '<', $filepath;
   binmode INFILE;
   my $buf;
   my $decoded;
   while ( read( INFILE, $buf, 100000000 ) ) {
        $decoded.=decode_base64($buf);
   }
   close INFILE;
   return $decoded;
}

sub decodeAndWriteToFile {
   my $image = $_[0];
   my $filename = $_[1];
   $image = decode_base64($image);
   
   open (FILE, ">$filename") or die "Can't read $filename !";
   print FILE $image;
   close(FILE);
}


sub writeToFile {
   my $image = $_[0];
   my $filename = $_[1];
   open (FILE, ">$filename") or die "Can't read $filename !";
   print FILE $image;
   close(FILE);
}


sub sendData_flush {
    my $DESTINATION_IP_HOST   = $_[0];
    my $DESTINATION_PORT = $_[1];
    my $text             = $_[2];

    my $sock = new IO::Socket::INET(PeerAddr=>$DESTINATION_IP_HOST,PeerPort=>$DESTINATION_PORT,Proto=>'tcp');
    if($sock) {
        $sock->send($text);
        $sock->close();
        return 1;
    }
    else {
        print "could not send data, destination $DESTINATION_IP_HOST:$DESTINATION_PORT not reachable\n";
        return 0;
    }
}

sub correctPath {
	my $path = $_[0];
	$path =~ s/\/\//\//;
	return $path;	
}

sub flushFile {
   my $file = $_[0];
   open (FILE, ">$file") or die "Can't read $file $!";
   close(FILE);
}

sub redirectOutput {
	my $outfile = $_[0]; 
    my $errfile = $_[1];
	
	flushFile($outfile);
    flushFile($errfile);
    
    chdir '/'                   or die "Can't chdir to /: $!";
    open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
    open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
    open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";
}

sub getHostFromURL {
    my $uri = URI->new($_[0]);
    return $uri->host;
}

sub getPortFromURL {
    my $uri = URI->new($_[0]);
    return $uri->port;
}

sub getDate() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   my $time = sprintf("%4d-%02d-%02d", $year+1900,$mon+1,$mday);
   return $time;
}

sub getMidnight() {
   return getDate() . " 23:59:59";
}

sub getMidnightAsInt() {
   return parseDate(getMidnight());
}

sub parseDate {
   my $time = $_[0];
   return str2time($time);
}





sub getTag_IncludingNamespace_FromRawSOAP {
    my $raw_soap = $_[0];
    my $ns = $_[1];
    
    if($raw_soap =~ m/$ns/ ) {
        my $retVal = $raw_soap;
        $retVal =~ s/\s+//g;

        # print "retVal: " . $retVal . "\n";
        while($retVal =~ m/[\r\n]/) {
            $retVal =~ s/[\r\n]/ /;
            # print "retVal: " . $retVal . "\n";
        }
        while( $retVal =~ m/\>\</ )
        {
            $retVal =~ s/\>\</\>\ \</;
            # print "retVal: " . $retVal . "\n";
        }
        
        # print "retVal: " . $retVal . "\n";
        my @tokens = split(" ", $retVal);
        $retVal = $ns;
        foreach my $token (@tokens){
            if($token =~ m/.*<\/.*:$ns>/){
                # print "token matches: " . $token . "\n";
                $retVal = $token;
                $retVal =~ s/.*\<\///;
                $retVal =~ s/\///;
                $retVal =~ s/\>//;
                $retVal = &UBERNUtils::trimchomp($retVal);
                # print "retVal: " . $retVal . "\n";
                return $retVal;
            }
        }
        return $retVal;     
    }
    else {
        return -1;
    }
}



sub getItemsFromSOAPBody {
    
    # print "getItemsFromSOAPBody\n";
    
    my $raw_soap = $_[0];
    
    my $xml = new XML::Simple;
    my $data = $xml->XMLin($raw_soap);
    
    my $Envelope_ns = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Envelope");
    my $Body_ns     = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Body");
    
    # print "Envelope: $Envelope_ns\n";
    # print "Body: $Body_ns\n";
    
    my $item_name = @_[@_-1];
    
    my @items = ();
    
    my $reference;
    
    if(&UBERNUtils::checkTagExists($data->{$Envelope_ns})){
        $reference = $data->{$Envelope_ns};     
    }
    else {
        $reference = $data;
    }
    
    $reference = $data->{$Body_ns};
    
    # start at 1 because $_[0] is the raw xml
    for(my $i=1; $i<@_-1; $i++) {
        my $tag_name = @_[$i];
        $reference = $reference->{$tag_name};
    }
    
    $reference = $reference->{$item_name};
    
    if( ref($reference) eq "" ){
        # print "adding string: $reference\n";
        if(!&UBERNUtils::trimchomp($reference) eq "")
        {
            push(@items, $reference);
        }
    }
    elsif(ref($reference) eq 'HASH' ){
        # print "adding hash: $reference\n";
        # print Dumper($reference);
        my $record = &UBERNUtils::getValue($reference);
        push(@items, $record);
    }
    elsif(ref($reference) eq 'SCALAR') {
        # print "adding scalar: $reference\n";
        if(!&UBERNUtils::trimchomp("$reference") eq "")
        {
           push(@items, $reference);
        }
    }
    elsif(ref($reference) eq 'ARRAY') {
        # print "adding array: $reference\n";
        # print Dumper($reference);
        foreach my $record (@{$reference})  {
            $record = &UBERNUtils::getValue($record);
            push(@items, $record);
        }
    }
    # print "getItemsFromSOAPBody end\n";
    return @items;
}


sub getItemFromSOAPBody {
    my @items = &UBERNUtils::getItemsFromSOAPBody(@_);
    return @items[0];
}

sub getHashesFromSOAPBody {
    
    # print "getHashesFromSOAPBody\n";
    
    my $raw_soap = $_[0];
    
    my $xml = new XML::Simple;
    my $data = $xml->XMLin($raw_soap);
    
    my $Envelope_ns = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Envelope");
    my $Body_ns     = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Body");
    
    # print "Envelope: $Envelope_ns\n";
    # print "Body: $Body_ns\n";
    
    my $item_name = @_[@_-1];
    
    my @items = ();
    
    my $reference;
    
    if(&UBERNUtils::checkTagExists($data->{$Envelope_ns})){
        $reference = $data->{$Envelope_ns};     
    }
    else {
        $reference = $data;
    }
    
    $reference = $data->{$Body_ns};
    
    # start at 1 because $_[0] is the raw xml
    for(my $i=1; $i<@_-1; $i++) {
        my $tag_name = @_[$i];
        $reference = $reference->{$tag_name};
    }
    
    $reference = $reference->{$item_name};
    
    if(ref($reference) eq 'HASH' ){
        # print "is Hash! " . Dumper($reference) . "\n";
        push(@items, $reference);
    }
    else {
        # print "is array! " . Dumper($reference) . "\n";
        foreach my $record (@{$reference})  {
            push(@items, $record);
        }
    }
    
    # print "getHashesFromSOAPBody end @items\n";
    
    return @items;
}


1;