package UBERN_SOAPServer;

use POSIX qw(setsid);
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use Data::Dumper;

our $MAX_LINES = 200;

# load the MANAGEMENT_INTERFACE_TESTBED_PORT - Interface of the Basic Access Functionalities
# to the deployed stationary Wireless Sensor Network nodes
require "WSNService.pm";
require "UBERNUtils.pm";
require "SessionManagementService.pm";

our $WSN_SERVICE_IP_ADDRESS			= &UBERNUtils::getThisMachineIP();

sub WSNServiceSOAPServer {

   my $outfile        = $_[0]; 
   my $errfile        = $_[1]; 
   my $port           = $_[2];   
   my $reservation_id = $_[3];	
	
   &UBERNUtils::redirectOutput($outfile,$errfile);
   &WSNService::setOutfile($outfile);
   &WSNService::setErrfile($errfile);
   
   # read the configuration
   &UBERNUtils::readConfiguration("/etc/TARWIS.conf");

   $0 = "WSNServiceSOAPServer_". $port;
  
   print "starting WSNServiceSOAPServer...\n";
   my $daemon;
   
   if(&UBERNUtils::verifyPort($WSN_SERVICE_IP_ADDRESS, $port)) {
   		print "host: " . $WSN_SERVICE_IP_ADDRESS . " port: " . $port . " is already occupied, kill the application on this port!\n";
   }
   
   $SIG{USR1} =\&gotSignal;
   
   sub gotSignal {
        print "WSNServiceSOAPServer::got signal, exiting\n";
        exit(0);
   }
   
   
   while(1) {
   
       my $daemon = SOAP::Transport::HTTP::Daemon->new(LocalAddr => $WSN_SERVICE_IP_ADDRESS, LocalPort => $port, Listen => 50, ReuseAddr => 1);
       $daemon->deserializer(WSNServiceDeserializer->new); 
       $daemon->dispatch_to("WSNService", "CommonTypes");
       $daemon->myuri("urn:WSNService");
       
       print "SOAP HTTP server listening...\n";
       print "   Host: ", $daemon->sockhost, "\n";
       print "   Port: ", $daemon->sockport, "\n";
       print "   Time: ". &UBERNUtils::getTimeInMySQLFormat() . "\n";
       $daemon->handle();
       
       BEGIN {
          package WSNServiceDeserializer; @WSNServiceDeserializer::ISA = 'SOAP::Deserializer';
          use SOAP::Lite;
          use vars qw(@ISA);
          
          sub deserialize {
            &WSNService::setExperimentID($reservation_id);
            &WSNService::setRawXML_SOAPRequest($_[1]);
            shift->SUPER::deserialize(@_);
          }
       }
       
   }
   
}

   



sub SessionManagementServiceSOAPServer {

   my $outfile = $_[0]; 
   my $errfile = $_[1]; 
	
   &UBERNUtils::redirectOutput($outfile,$errfile);
   &SessionManagementService::setOutfile($outfile);
   &SessionManagementService::setErrfile($errfile);

   # read the configuration
   
   # read the configuration
   &UBERNUtils::readConfiguration("/etc/TARWIS.conf");
   
   my $ip   = &Utils::getHostFromURL($ENV{"SESSION_MANAGEMENT_SERVICE_URL"});
   my $port = &Utils::getPortFromURL($ENV{"SESSION_MANAGEMENT_SERVICE_URL"});
   
   $0 = "SessionManagementServiceSOAPServer_" . $port;
   print "starting SessionManagementServiceSOAPServer...\n";

   my $daemon;
   use SOAP::Transport::TCP;

   use SOAP::Lite; #  +trace;
   while(1) {
       my $portServed = &UBERNUtils::portChecker($ip, $port);
       if(!$portServed){
          $daemon = SOAP::Transport::HTTP::Daemon->new(LocalAddr => $ip, LocalPort => $port, Listen => 50, ReuseAddr => 1);
       }
       $daemon->dispatch_to('SessionManagementService', 'CommonTypes');
       $daemon->myuri("urn:SessionManagementService");
       $daemon->deserializer(SessionManagementServiceDeserializer->new); 
       
       print "SOAP HTTP server listening...\n";
       print "   Host: ", $daemon->sockhost, "\n";
       print "   Port: ", $daemon->sockport, "\n";
       $daemon->handle();
       
       BEGIN {
          package SessionManagementServiceDeserializer; @SessionManagementServiceDeserializer::ISA = 'SOAP::Deserializer';
          use SOAP::Lite; #  +trace;
          use vars qw(@ISA);
          
          sub deserialize {
            &SessionManagementService::setRawXML_SOAPRequest($_[1]);
            shift->SUPER::deserialize(@_);
          }
       }
       
   }
}

1;