#!/bin/bash

if [ "$(id -u)" != "0" ]; then
    echo "This script must be run as root" 1>&2
    exit 1
fi

cat /proc/version | grep "debian" | grep "lenny" > /dev/null
if [ $? != 0 ]; then
   if [ `cat /etc/debian_version` == "5.0.5" ]; then
        echo "version is ok, yet /proc/version does not contain lenny..."
        echo "continuing..."
   else
   echo "This install script only supports Debian Lenny! If you're using"
   echo "another linux distro you have to install the subsequent Debian"
   echo "Lenny packages manually for your linux distribution!"
   exit 1
   fi
fi

apt-get update

apt-get install -y --force-yes locales
dpkg-reconfigure locales

echo "deb http://ftp.au.debian.org/debian/ lenny main contrib non-free" >> /etc/apt/sources.list
echo "deb http://ftp.de.debian.org/debian/ lenny main contrib non-free" >> /etc/apt/sources.list
echo "deb http://ftp.nl.debian.org/debian/ lenny main contrib non-free" >> /etc/apt/sources.list





apt-get clean all
apt-get update --force-yes
apt-get upgrade -y --force-yes
dpkg --configure -a --force-yes
apt-get -f install -y --force-yes

# fetch and install required debian packages for mysql
apt-get install -y --force-yes gcc make libc6 libncurses-dev
apt-get install -y --force-yes mysql-admin 
apt-get install -y --force-yes mysql-server
apt-get install -y --force-yes mysql-admin-common 
apt-get install -y --force-yes mysql-client 
apt-get install -y --force-yes mysql-navigator 
apt-get install -y --force-yes mysql-query-browser 
apt-get install -y --force-yes mysql-server
apt-get install -y --force-yes mysql-devel
apt-get install -y --force-yes libmysqlclient15-dev

# other stuff
apt-get install -y --force-yes expat
apt-get install -y --force-yes libexpat* 
apt-get install -y --force-yes ruby soap4r libruby1.8 libxmlrpc-ruby


# fetch and install required perl and php modules
apt-get install -y --force-yes perl 
apt-get install -y --force-yes perl-base 
apt-get install -y --force-yes perl-modules 
apt-get install -y --force-yes php-db 
apt-get install -y --force-yes php5 
apt-get install -y --force-yes php5-mysql 
apt-get install -y --force-yes libperl6-slurp-perl 
apt-get install -y --force-yes libperl6-export-perl 
apt-get install -y --force-yes libdatetime-format-mysql-perl 
apt-get install -y --force-yes libdb4.2 
apt-get install -y --force-yes libdb4.3 
apt-get install -y --force-yes libsoap-lite-perl 
apt-get install -y --force-yes php5-common 
apt-get install -y --force-yes php5-mysql 
apt-get install -y --force-yes libio-all-perl
apt-get install -y --force-yes libio-zlib-perl
apt-get install -y --force-yes libio-socket-ssl-perl
apt-get install -y --force-yes libarchive-tar-perl
apt-get install -y --force-yes libmodule-build-perl
apt-get install -y --force-yes libextutils-autoinstall-perl
apt-get install -y --force-yes libmodule-install-perl
apt-get install -y --force-yes libgraphviz-perl
apt-get install -y --force-yes librexml-ruby
apt-get install -y --force-yes rubygems

apt-get install -y --force-yes libhttp-access2-ruby*
apt-get install -y --force-yes libruby*
apt-get install -y --force-yes libxml-parser-ruby1.8
apt-get install -y --force-yes libhttp-access2-ruby1.8
apt-get install -y --force-yes libzlib-ruby
apt-get install -y --force-yes libuconv-ruby1.8
apt-get install -y --force-yes php-soap
apt-get install -y --force-yes php5-cli
apt-get install -y --force-yes phpgroupware-soap
apt-get install -y --force-yes libcgi-perl
apt-get install -y --force-yes imagemagick


apt-get install -y --force-yes libmysql-ruby
apt-get install -y --force-yes libdbd-mysql-ruby

apt-get install -y --force-yes bsdmainutils

rm -rf perlLibs
mkdir perlLibs
cd perlLibs

rm install_output.log
touch install_output.log


# debs for the authorization tool
apt-get install -y libapache2-mod-php5 php5 php5-common php5-curl php5-dev php5-gd php5-mysql php5-mhash


# fetch and install required perl libraries
for lib in version-0.76 \
version-AlphaBeta-0.06 \
version-Limit-0.0301 \
WSDL-Generator-0.02 \
XML-Parser-2.36 \
XML-Writer-0.606 \
Pod-WSDL-0.05 \
Proc-Simple-1.24 \
SOAP-Lite-0.710.08 \
File-Temp-0.22 \
MIME-tools-5.427 \
MIME-Lite-3.024 \
IPC-ShareLite-0.17 \
IPC-Shareable-0.60 \
File-Slurp-9999.13 \
ExtUtils-MakeMaker-6.50 \
Email-Date-Format-1.002 \
DBI-1.607 \
DBI-1.609 \
Class-Hook-0.03 \
Carp-Assert-0.20 \
DateTime-Format-MySQL-0.04 \
XML-Simple-2.18 \
DBD-mysql-4.012 \
IO-1.25 \
libwww-perl-5.829 \
SOAP-0.28 \
TimeDate-1.16 \
Compress-Raw-Bzip2-2.021 \
Compress-Raw-Zlib-2.021 \
Compress-Raw-Zlib-2.024 \
IO-Compress-2.021 \
Tie-ShareLite-0.03 \
SOAP-Data-Builder-0.9 \
SOAP-Data-ComplexType-0.044 \
Pod-Simple-3.10 \
Test-Pod-1.40 \
Date-Manip-5.54 \
SOAP-DateTime-0.02 \
threads-1.75 \
Log-Report-0.25 \
Test-Deep-0.106 \
XML-Compile-Tester-0.05 \
XML-Compile-1.06 \
XML-Compile-SOAP-2.10 \
Template-Toolkit-2.22 \
TermReadKey-2.30 \
Class-Std-0.011 \
Class-Std-Fast-v0.0.8 \
Task-Weaken-1.03 \
SOAP-Lite-0.710.10 \
libwww-perl-5.834 \
Net-Ping-2.36 \
SOAP-WSDL-2.00.10 \
Guard-1.021 \
common-sense-2.01 \
Coro-5.21 \
Storable-2.21 \
Archive-Zip-1.30 \
TimeDate-1.20 \
File-ReadBackwards-1.04 \
HTTP-Async-0.09;
do
        echo "installing lib: $lib"
        wget -q http://www.iam.unibe.ch/wisebed/TARWIS/perlLibs/$lib.tar.gz

        tar xzf $lib.tar.gz > install_output.log
        cd $lib >> install_output.log
        if [ "$lib" == "SOAP-Lite-0.710.08" ]; then
            perl Makefile.PL --noprompt >> install_output.log
        elif [ "$lib" == "MIME-Lite-3.024" ]; then
            touch confirm && echo -e "\n\n\n\n\n" >> confirm 
            perl Makefile.PL < confirm >> install_output.log
            rm confirm
        elif [ "$lib" == "SOAP-Lite-0.710.10" ]; then
            touch confirm && echo -e "\n\n\n\n\n" >> confirm 
            perl Makefile.PL < confirm >> install_output.log
            rm confirm
        else
            perl Makefile.PL >> install_output.log
        fi
        make >> install_output.log
        make install >> install_output.log
        cd ..
        rm -rf $lib
        rm -rf $lib.tar.gz
        
        echo "#################################################"
        echo "CHECK IN THE ABOVE OUTPUT IF THE INSTALLATION OF "
        echo "$lib HAS SUCCEEDED!                              "
        echo "CHECK IF THERE IS ANY ERROR MESSAGE GENERATED BY "
        echo "\"make\" and \"make install\"                    "
        echo "IF THERE ARE ERRORS, TRY TO INSTALL IT MANUALLY! "
        echo "(E.G. USE PERL CPAN ...)                         "
        echo "IGNORE WARNINGS, CONFIRM QUESTIONS WITH ENTER... "
        echo "PRESS ENTER TO CONTINUE!                         "
        echo "#################################################"
        read keystroke
done

# get graphviz modification
# and place it at /usr/share/php/Image/
wget -q http://www.iam.unibe.ch/wisebed/TARWIS/perlLibs/GraphViz_mod.tar.gz
tar xzf GraphViz_mod.tar.gz
rm -rf /usr/share/php/Image
mv Image /usr/share/php
rm -rf GraphViz_mod.tar.gz*

# get newest soap4r

wget http://www.iam.unibe.ch/wisebed/TARWIS/perlLibs/soap4r.tar.gz
tar xzf soap4r.tar.gz
cd soap4r-1.5.8/
ruby install.rb
cd ..

rm -rf soap4r.tar.gz*
rm -rf soap4r-1.5.8
rm -rf perlLibs


reboot



