#!/bin/bash

##########################################################

function readconf() {
    while read line; do
        # skip comments
        [[ ${line:0:1} == "#" ]] && continue 
        # skip empty lines
        [[ -z "$line" ]] && continue
        line=`echo $line | awk -F "#" '{print $1}'`
        # echo line=$line
        key=${line//=*}
        value=${line//*=}
        key=$(echo "$key" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
        value=$(echo "$value" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
        # echo key=$key 
        # echo value=$value
        export $key="$value"
    done < "$CONFIG"
}

#########################################################

CONFIG=/etc/TARWIS.conf

readconf 

cat > resetDatabase << EOF
DROP DATABASE IF EXISTS wsn_info;
EOF

# make the database accessible to the outside in case of UBERN
if [ `ifconfig eth0 | awk '/inet addr/ {split ($2,A,":"); print A[2]}'` == "130.92.66.182" ]; then
    echo "GRANT ALL ON wsn_info.* TO wisebeduser@'130.92.66.182' IDENTIFIED BY 'wisebed';" >> resetDatabase
    echo "GRANT ALL ON wsn_info.* TO wisebeduser@'130.92.66.132' IDENTIFIED BY 'wisebed';" >> resetDatabase
    echo "GRANT ALL ON wsn_info.* TO wisebeduser@'130.92.65.69'  IDENTIFIED BY 'wisebed';" >> resetDatabase
    iptables --flush  >/dev/null 2>&1
    iptables -A INPUT -i eth0 -p tcp --destination-port 3306 -j ACCEPT >/dev/null 2>&1
    iptables-save  >/dev/null 2>&1
fi;


wisebed_sql_error() {
    echo "#############################################################"
    echo "wisebed.sql not found! go to the TARWISServer directory      "
    echo "and run resetDB.sh again                                     "
    echo "cd TARWISServer && sh resetDB.sh                             "
    echo "#############################################################"
    exit
}

echo $TARWIS_DIRECTORY
FILE=$TARWIS_DIRECTORY/wisebed.sql
FILE=`echo $FILE | sed "s/\/\//\//"`


cat $FILE >> resetDatabase  || wisebed_sql_error
# setting up tables in db
echo "#############################################################"
echo "This script sets up the TARWIS mysql database tables!        "
echo "This requires that you know your root mysql user password!!  "
echo "#############################################################"
echo "IN CASE YOU DO NOT HAVE CONFIGURED YOUR ROOT MYSQL USER YET  "
echo "Do you want to set the password for your root mysql user NOW?"
echo "[enter y or n]" 

read answer

if [ $answer == "y" ]; then
    echo "enter desired mysql root user password and remember it:" 
    read pw
    mysqladmin -u root password $pw
fi



echo "#############################################################"
echo "Setting up TARWIS mysql tables now"
echo "#############################################################"
echo "ENTER THE ROOT MYSQL USER PASSWORD:"
mysql -h localhost -u root -p < resetDatabase

if [ $? == 0 ]; then
    echo "#############################################################"
    echo "set up of tables succeeded... exiting"
    echo "#############################################################"

rm -rf insertImage.pl

cat > insertImage.pl << EOF

use Env;
readConfiguration("/etc/TARWIS.conf");
sub readConfiguration {
my \$conf = \$_[0];
open(CONFIG, \$conf)  or return 0;   
while (<CONFIG>) {
chomp;                                    
s/#.*//;                                   
s/^\s+//;                                  
s/\s+\$//;                         
next unless length;
my (\$var, \$value) = split(/\s*=\s*/, \$_, 2);
\$ENV{"\$var"} = \$value;
} 
}


push(@INC, \$ENV{'TARWIS_DIRECTORY'});

require "DatabaseInterface.pm"; 
\$databaseInterface = new DatabaseInterface();
\$databaseInterface->registerDefaultImage(\$ENV{'DEFAULT_IMAGE_PATH'}, \$ENV{'DEFAULT_IMAGE_PLATFORM'});
EOF

rm -rf insertImage.pl

else 
    echo "#############################################################"
    echo "set up tables failed! run resetDB.sh again!"
    echo "#############################################################"
fi

rm resetDatabase





