<?php

class NetTreeExpSetup
{
	private static $instance = null;
	private $doc = null;
	private $network = null;

	private function __construct()
	{
	}

	public function getInstance()
	{
		if(self::$instance == null )
		{
			self::$instance = new NetTreeExpSetup();
		}
		return self::$instance;
	}

	public final function __clone()
	{
		trigger_error( "Cannot clone instance of XMLGenerator", E_USER_ERROR );
	}

	public function generateNetworkTree($network, $fileName)
	{
		$this->network = $network;

		$this->doc = new DOMDocument("1.0", "ISO-8859-1");

		$netTree = $this->createTree();
		$this->doc->appendChild($netTree);

		//$fileName = "../ExperimentSetup/" . $this->network->name . ".xml";

		file_put_contents($fileName, $this->doc->saveXML());
	}

	private function createTree()
	{
		$netTree = $this->doc->createElement("tree");
		$netTree->setAttribute( "id", "0" );

		$ntNetwork = $this->createNetwork();
		$netTree->appendChild($ntNetwork);

		return $netTree;
	}
	
	
	private function getAllTypes($nodeTypeMap)
	{
		$types = array();
		foreach ($nodeTypeMap as $urn => $type) {
		   if(!in_array($type, $types)) {
		       array_push($types, $type);
		   }
		}  
		return $types;
	}  
		
	
	private function createTypeMap($nodesArray)
    {
        $nodeTypeMap  = array();
        if(!empty($nodesArray))
        {
            foreach($nodesArray as $node)
            {  
              $nodeTypeMap[$node->id] = $node->type;
            }
        }
        return $nodeTypeMap;
    }  

	private function createNetwork()
	{
		$ntNetwork = $this->doc->createElement("item");
		$ntNetwork->setAttribute("id", "network");
		$ntNetwork->setAttribute("text", $this->network->name);
		$ntNetwork->setAttribute("im0", "../../../../pics/net.png");
		$ntNetwork->setAttribute("im1", "../../../../pics/net.png");
		$ntNetwork->setAttribute("im2", "../../../../pics/net.png");
		$ntNetwork->setAttribute("closeable", "0");
		$ntNetwork->setAttribute("open", "1");
 
    
	    $map = $this->createTypeMap($this->network->sensornodes);
		$types = getAllTypes($map);
		$nodes_sorted_by_nodetype = array();
	    foreach($types as $type) {
		    foreach($this->network->sensornodes as $sensornode) {
		        if($map[$sensornode->id]==$type){
		            array_push($nodes_sorted_by_nodetype, $sensornode);
		        } 
		    }
		}
	   
		foreach($nodes_sorted_by_nodetype as $sensornode)
		{
			if($sensornode->isNodeAlive()){
				$ntSN = $this->createTNSensornode($sensornode);
				$ntNetwork->appendChild($ntSN);
			}
			else {
				$ntSN = $this->createDeadTNSensornode($sensornode);
				$ntNetwork->appendChild($ntSN);
			}
		}

		return $ntNetwork;
	}
	
	private function createTNSensornode($sensornode)
	{
		$ntSN = $this->doc->createElement("item");
		$ntSN->setAttribute("id", "sn".$sensornode->id);
		$ntSN->setAttribute("text", $sensornode->id);
		$ntSN->setAttribute("im0", "../../../../pics/" . $sensornode->type . ".png");
		$ntSN->setAttribute("im1", "../../../../pics/" . $sensornode->type . ".png");
		$ntSN->setAttribute("im2", "../../../../pics/" . $sensornode->type . ".png");
		$ntSN->setAttribute("tooltip","node is alive");
		
		
		$tnSnID = $this->doc->createElement("item");
		$tnSnID->setAttribute("id", "sn" . $sensornode->id . "_" . "ID");
		$tnSnID->setAttribute("text", "sensor_node_id: " . $sensornode->id);
		$tnSnID->setAttribute("nocheckbox", "1");
		$tnSnID->setAttribute("im0", "../../../../pics/information.png");
		$tnSnID->setAttribute("im1", "../../../../pics/information.png");
		$tnSnID->setAttribute("im2", "../../../../pics/information.png");
		$ntSN->appendChild($tnSnID);

		foreach($sensornode->information as $snProp)
		{
			$tnSnProp = $this->createSensornodeProperty($snProp, $sensornode->id);
			$ntSN->appendChild($tnSnProp);
		}

		return $ntSN;
	}
	
	private function createDeadTNSensornode($sensornode)
	{
		$ntSN = $this->doc->createElement("item");
		$ntSN->setAttribute("id", "sn".$sensornode->id);
		$ntSN->setAttribute("nocheckbox", "1");
		$ntSN->setAttribute("text", $sensornode->id);
		$ntSN->setAttribute("im0", "../../../../pics/" . "dead.png");
		$ntSN->setAttribute("im1", "../../../../pics/" . "dead.png");
		$ntSN->setAttribute("im2", "../../../../pics/" . "dead.png");
		$ntSN->setAttribute("tooltip","node is currently dead");
		
		
//		$tnSnID = $this->doc->createElement("item");
//		$tnSnID->setAttribute("id", "sn" . $sensornode->id . "_" . "ID");
//		$tnSnID->setAttribute("text", "sensor_node_id: " . $sensornode->id);
//		$tnSnID->setAttribute("nocheckbox", "1");
//		$tnSnID->setAttribute("im0", "../../../../pics/information.png");
//		$tnSnID->setAttribute("im1", "../../../../pics/information.png");
//		$tnSnID->setAttribute("im2", "../../../../pics/information.png");
//		$ntSN->appendChild($tnSnID);
//
//		foreach($sensornode->information as $snProp)
//		{
//			$tnSnProp = $this->createSensornodeProperty($snProp, $sensornode->id);
//			$ntSN->appendChild($tnSnProp);
//		}

		return $ntSN;
	}

	private function createSensornodeProperty($snProp, $snID)
	{
		$tnSnProp = $this->doc->createElement("item");
		$tnSnProp->setAttribute("id", "sn" . $snID . "_" . $snProp['prop_id']);
		$tnSnProp->setAttribute("text", $snProp['description'] . ": " . $snProp['value']);
		$tnSnProp->setAttribute("nocheckbox", "1");
		$tnSnProp->setAttribute("im0", "../../../../pics/information.png");
		$tnSnProp->setAttribute("im1", "../../../../pics/information.png");
		$tnSnProp->setAttribute("im2", "../../../../pics/information.png");

		return $tnSnProp;
	}
}
?>
