<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: AttributeType.class.inc,v 1.16 2006/11/06 16:40:27 vlry Exp $
 */

 /**
 * AttributeType is a utility class for managing and accessing 
 * attribute types. 
 *
 * There is a <b>predefined list of attriute types</b>
 * whose IDs, names and descriptions are stored in the database
 * table AttributeType. The first name of a user is an example for
 * a predefined attribute type. It's ID is 2 and its LDAP name
 * is 'givenName'.
 *
 * At run time, Role Provider can define <b>role specific 
 * attribute types</b>. For instance, if a Role Provider requires
 * subscribers to submit a short motivation text, he could create
 * a role specific attribute type 'motivationText'. User would
 * then have to fill in a value for this attribute when they subscribe
 * for the respective role. 
 *
 *
 * @author Thomas Spreng  <spreng@iam.unibe.ch>
 * @author Valery Tschopp <tschopp@switch.ch>
 * @version $Revision: 1.16 $
 * 
 * @see MySQLConnection
 * 
 * @package Library 
 */ 
class AttributeType {
           
    /**
    * Replies the attribute type ID given the LDAP name of an
    * attribute.
    *
    * @param string $ldapName  the LDAP name of the attribute
    *
    * @return int the internal attributetype_id or -1, if the 
    *   there is no attribute with LDAP name $ldapName
    *
    * @static
    */  
    function getAttributeTypeIdByLdapName($ldapName) {
    
        $conn =& MySQLConnection::getConnection();
     
        $query = " SELECT attributetype_id "
               . " FROM  AttributeType "
               . " WHERE attributetype_ldapname = '$ldapName' ";
    
        $conn->query($query);
        $row = $conn->fetch_array();
        $conn->freeresult();
        if (! $row) {
            return -1;
        } 
        else {
            return $row["attributetype_id"];    
        }     
    }

    /**
    * Replies the name of an attribute type given an attribute type
    * ID. 
    *
    * @param mixed $attributeTypeId  either the internal database 
    *   ID or the unique LDAP name for the attribute type
    *
    * @return string the name of the attribute type 
    *
    * @static
    *
    */
    function getAttributeTypeName($attributeTypeId) {
        $conn =& MySQLConnection::getConnection();
    
        $name = false;
        if (is_int($attributeTypeId)) {
            $query  = "select attributetype_name from AttributeType ";
            $query .= "where attributetype_id = $attributeTypeId";
            
            $conn->query($query);
            $row = $conn->fetch_array();
            $conn->freeresult();
            $name = $row['attributetype_name'];
        }
        return $name;
    
    }
    

    /**
    * Replies the LDAP name of an attribute type given an attribute type
    * ID. 
    *
    * @param int  $attributeTypeId  the internal database 
    *   ID for the attribute type
    *
    * @return string the LDAP name of the attribute type 
  *
    * @static
    */
    function getAttributeTypeLdapName($attributeTypeId) {

        $conn =& MySQLConnection::getConnection();
    
        $name = false;
        if (is_int($attributeTypeId)) {
            $query  = "select attributetype_ldapname from AttributeType ";
            $query .= "where attributetype_id = $attributeTypeId";
            
            $conn->query($query);
            $row = $conn->fetch_array();
            $conn->freeresult();
            $name = $row['attributetype_ldapname'];
        }
        return $name;

    }

    /**
    * Replies the intenal attribute type ID given  the name 
    * of an attribute type.
    *
    * @param string  $attributeTypeName  the attribute type name  
    *
    * @return int  the internal database ID of the attribute type 
    *
    * @static
    *
    */

    function getAttributeTypeId($attributeTypeName) {

        $conn =& MySQLConnection::getConnection();
    
        $query  = "select attributetype_id from AttributeType ";
        $query .= "where attributetype_name = '$attributeTypeName'";
                    
        $conn->query($query);
        $conn->fetch_array();
        $conn->freeresult();
        return $row['attributetype_id'];

    }

    /**
    * Replies true if the given attribute type is a standard
    * attribute type.  
    *
    * @param mixed $attributeType  either the internal database 
    *   ID or the unique LDAP name for the attribute type
    *
    * @return int   1, if the given attribute type is a standard 
    *   attribute type; 0, otherwise 
    *
    * @static
    *
    */
    function isAttributeTypeStandard($attributeType) {

        if (!is_int($attributeType)) {
            $attributeTypeId = $this->getAttributeTypeId($attributeType);
        } else {
            $attributeTypeId = $attributeType;
        }
        
        $conn =& MySQLConnection::getConnection();
        
        $query  = "select attributetype_standard from AttributeType ";
        $query .= "where attributetype_id = $attributeTypeId";

        $conn->query($query);
        $row = $conn->fetch_array();
        $conn->freeresult();
        return $row['attributetype_standard'];

    }


    /**
    * Inserts a new attribute type in the  database 
    *
    * @param string $attributeTypeName  the attribute type
    *   name
    *
    * @param string $attributeTypeDescription  the attribute type
    *   description 
    *
    * @return int   the internal attribute type ID created for this
    *    new attribute type 
    *
    * @static
    *
    */
    function addAttributeType($attributeTypeName, $attributeTypeDescription = NULL) {

        $conn=& MySQLConnection::getConnection();
        $query  = "insert into AttributeType ";
        $query .= "(attributetype_name, attributetype_description, date_created, addr_created) ";
        $query .= "values ('" . $conn->escape_string($attributeTypeName) . "', ";
        $query .= is_null($attributeTypeDescription)  ? " NULL, "
                  : " '" . $conn->escape_string($attributeTypeDescription) . "', ";
        $query .= " NULL, '" . $_SERVER["REMOTE_ADDR"]."') ";

        $conn->query($query);
        return $conn->insert_id();
    }


    /**
    * Updates an attribute type in the  database 
    *
    * @param int $atid the internal attribute type ID 
    *
    * @param string $attributeTypeName  the attribute type
    *   name
    *
    * @param string $attributeTypeDescription  the attribute type
    *   description 
    *
    * @static
    *
    */
    function updateAttributeType($atid, $attributeTypeName, $attributeTypeDescription = NULL) {

        $conn =& MySQLConnection::getConnection();
        $query = " UPDATE AttributeType "
               . "   SET attributetype_name = '" . $conn->escape_string($attributeTypeName) . "', "
               . "       attributetype_description = " . (is_null($attributeTypeDescription) ? " NULL "
               : " '" . $conn->escape_string($attributeTypeDescription) . "' " )
               . " WHERE attributetype_id = $atid ";
     
        $conn->query($query);
    }



    /**
    * Deletes an attribute type from the database 
    *
    * This method removes the attribute type itself and all attribute
    * values of this type.
    *
    * @param mixed $attributeType  either the internal database 
    *   ID or the unique LDAP name for the attribute type
    *
    * @static
    *  
    */
    function deleteAttributeType($attributeType) {

        if (!is_int($attributeType)) {
            $attributeTypeId = $this->getAttributeTypeId($attributeType);
        } else {
            $attributeTypeId = $attributeType;
        }
        
        $conn =& MySQLConnection::getConnection();
        
        $query  = "delete from Attribute ";
        $query .= "where attribute_type = $attributeTypeId";

        $conn->query($query);
        
        $query  = "delete from AttributeType ";
        $query .= "where attributetype_id = $attributeTypeId";

        $conn->query($query);

    }


    /**
    * Replies an array of attribute IDs for all standard attributes. 
    *
    * @return array  an array of attribute type IDs 
    *
    * @static
    */
    function getStandardAttributeTypeIds() {
        return AttributeType::_getAttributeTypeIdsByClass(true);
    }


    /**
    * Replies an array of attribute IDs for all non-standard attributes. 
    *
    * @return array  an array of attribute type IDs 
    *
    * @static
    *
    */
    function getNonStandardAttributeTypeIds() {     
        return AttributeType::_getAttributeTypeIdsByClass(false);
    }


    /**
    * Replies an array of attribute IDs for all standard attributes. 
    *
    * @param $isStandard  true, if standard attributes are to be
    *   replies; false, if non-standard attributes are to be replied
    *
    * @return array  an array of attribute type IDs 
    *
    * @access private 
    */
    function _getAttributeTypeIdsByClass($isStandard) {

        $conn =& MySQLConnection::getConnection();
    
        $standard = ($isStandard) ? "1" : "0";
        
        $query =  " SELECT attributetype_id FROM AttributeType ";
        $query .= " WHERE attributetype_standard = $standard ";
        $query .= "  AND attributetype_id <> " . ATID_LOCALPASSWORD;
        $query .= " ORDER BY attributetype_id ";
            
        $conn->query($query);
        $atids= array();
        while ($row = $conn->fetch_array()) {
            $atids[]= $row["attributetype_id"];
        }
        $conn->freeresult();
        
        return $atids;
    }

   /**
    * Returns a hashtable (atid => attributetype_name) of attribute names. Used
    * without parameters, returns all ATID_* without the last (local password).
    *
    * @params var_args, comma-separated list of attribute type id (ATID_*), no param returns all atid.
    * @return array Hashtable (atid => name), with NULL value if not defined.  
    *
    * @see func_get_args()
    *
    * @access static 
    */
    function getAttributeTypeNames() {
        $atids= func_get_args();
        if (empty($atids)) {
            // no params, all ATID_* without local password
            $atids= range(ATID_SURNAME,ATID_EDUPERSONENTITLEMENT);
        }    
        $attr_names= array(); // hashtable (atid => name)
        $or_clause= array();
        foreach($atids as $atid) {
            $or_clause[]= "attributetype_id = $atid";
            // initialize hashtable value with NULL;
            $attr_names[$atid]= NULL;
        }
        $or_query= implode(' OR ', $or_clause);
        
        $query= " SELECT attributetype_id, attributetype_name "
              . " FROM AttributeType "
              . " WHERE $or_query "
              . " ORDER BY attributetype_id ";
        
        $conn=& MySQLConnection::getConnection();
        $conn->query($query);
        while( $row= $conn->fetch_array() ) {
            $attr_names[$row['attributetype_id']]= $row['attributetype_name'];
        }
        $conn->freeresult();
        
        return $attr_names;
    }

}

?>
