<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Invitation.class.inc,v 1.23 2006/11/06 16:34:23 vlry Exp $
 */

require_once('include/MySQLConnection.class.inc');

/**
 * Invitation for a particular role
 *
 * @version $Revision: 1.23 $
 * @author Valery Tschopp - SWITCH
 *
 * @package Library 
 */
class Invitation {

    //
    // static method
    //
    /**
     * Returns true iff the given invitation token is valid (stored in DB
     * table Invitation) for the given role id.
     *
     * @static
     */
    function isValid( $token, $rid ) {
        if (!is_numeric($rid)) return false;

        $conn=& MySQLConnection::getConnection();
        $token= $conn->escape_string($token);
        $query = " SELECT token ";
        $query.= " FROM Invitation ";
        $query.= " WHERE token = '$token' ";
        $query.= "   AND role_id = $rid";
        
        $conn->query($query);
        $valid= ($conn->num_rows() == 1);
        $conn->freeresult();
        return $valid; 
    }

    
    /**
     * @static
     */
    function delete( $token, $rid ) {
        if (!is_numeric($rid)) return false;
        $conn=& MySQLConnection::getConnection();
        $token= $conn->escape_string($token);
        $query= " DELETE FROM Invitation "
              . " WHERE token = '$token' "
              . "   AND role_id = $rid";
        
        $conn->query($query);
        $deleted= ($conn->affected_rows() == 1);
        return $deleted; 
    }

    /**
     * @static
     */
    function getEmail( $token, $rid ) {
        if (!is_numeric($rid)) return null;
        $conn=& MySQLConnection::getConnection();
        $token= $conn->escape_string($token);
        $query = " SELECT email ";
        $query.= " FROM Invitation ";
        $query.= " WHERE token = '$token' ";
        $query.= "   AND role_id = $rid";
        
        $conn->query($query);
        $email= null;
        $row= $conn->fetch_array();
        $conn->freeresult();
        if (isset($row['email'])) {
            $email= $row['email'];
        }
        return $email; 
    }

    
    /**
     * @static
     */
    function getInvitationsCount( $rid ) {
        $query = " SELECT token ";
        $query.= " FROM Invitation ";
        $query.= " WHERE role_id = $rid";
        
        $conn=& MySQLConnection::getConnection();
        $conn->query($query);
        $n_invitations= $conn->num_rows();
        $conn->freeresult();
        return $n_invitations;
    }

    /**
     * @param int rid The role ID.
     * @return array of invitations attributes: email, token, date_created, user_id_created
     *
     * @static
     */
    function getInvitationsAttributes( $rid, $orderby= null, $offset= null, $count= null ) {
        $query = " SELECT email, token, date_created, user_id_created ";
        $query.= " FROM Invitation ";
        $query.= " WHERE role_id = $rid ";
        if (!is_null($orderby)) {
            if (strpos($orderby, 'date_created') !== false) {
                $query.= " ORDER BY date_created DESC, email ";
            }
            elseif (strpos($orderby, 'email') !== false) {
                $query.= " ORDER BY email, date_created DESC ";
            }
            else {
                $query.= " ORDER BY $orderby ";        
            }
        }
        if (!is_null($offset) and !is_null($count)) {
            $query.= " LIMIT $offset,$count ";
        }

        $conn=& MySQLConnection::getConnection();
        $conn->query($query);

        $invs_attrs= array();
        while($row = $conn->fetch_array()) {
            $invs_attrs[]= $row;
        }
        $conn->freeresult();
        return $invs_attrs; 
    }    

    //
    // instance variables
    //
    /** One time token */
    var $token_;
    /** Role Id */
    var $rid_;
    /** Invitation receiver email */
    var $email_;
    /** Admin user_id who sent the invitation */
    var $user_id_= NULL;
    /** Underlying error */
    var $error_= NULL;
    
    //
    // methods
    //
    /**
     * Constructor
     *
     * @param $email The receiver email address.
     * @param $rid   The role id.
     */
    function Invitation( $email, $rid ) {
        $this->email_= $email;
        $this->rid_= $rid;
        $this->generateToken();
    }

    /**
     * Generates the one-time token.
     */
    function generateToken() {
        $digest= $this->rid_ . ":" . $this->email_;
        $token= md5($digest);
        $token= strtoupper($token);
        $this->token_= $token;
    }
    
    /**
     * Returns the one time token of this invitation.
     * @return String The one-time token.
     */
    function getToken() {
        return $this->token_;
    }
    
    /**
     * Sets the invitation creator (user_id)
     */
    function setCreatorUserId($user_id) {
        $this->user_id_= $user_id;
    }
    
    /**
     * Store this invitation in DB table Invitation.
     */
    function persist() {
    
        $remote= '';
        if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }

        $conn=& MySQLConnection::getConnection();
        $query= " INSERT INTO Invitation "
              . " (role_id, email, token, date_created, addr_created) "
              . " VALUES ("
              . "  " . (int)$this->rid_ . ", "
              . "  '" . $conn->escape_string($this->email_ ). "', "
              . "  '" . $conn->escape_string($this->token_) . "', "
              . "  NOW(), "
              . "  '" . $conn->escape_string($remote) . "' "
              . ")";
        
        $stored= $conn->query($query, false); // don't die
        if (!$stored) {
            $this->error_= $conn->getError();
        }
        
        if (!is_null($this->user_id_)) {
            $query= " UPDATE Invitation "
                  . " SET user_id_created = " . (int)$this->user_id_ . " "
                  . "     , date_created = NOW() "
                  . " WHERE role_id = " . (int)$this->rid_ . " "
                  . "   AND token = '" . $conn->escape_string($this->token_) . "'";
            $conn->query($query); // don't die

        }
        
        return $stored;
    }
    
   /**
    * Returns the underlying error message (MySQL).
    *
    * @return String The underlying error message or NULL.
    */
   function getError() {
       return $this->error_;
   }
    
}

?>
