<?php
/*
 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Logger.class.inc,v 1.4 2006/11/21 11:02:50 vlry Exp $
 */

/**
* Logger level constants
*/
define('LOGGER_LEVEL_DEBUG',    0);
define('LOGGER_LEVEL_NOTICE',   1);
define('LOGGER_LEVEL_INFO',     2);
define('LOGGER_LEVEL_WARNING',  3);
define('LOGGER_LEVEL_ERROR',    4);
define('LOGGER_LEVEL_CRITICAL', 5);

/**
 * Simple logger class.
 * Uses PHP error_log() function to log messages.
 *
 * @see error_log()
 *
 * @author Valery Tschopp - SWITCH
 * @version $Revision: 1.4 $
 * @package Library
 */
class Logger {

    var $level_;
    var $logfile_;

    // factory/singleton
    // @static
    function & getInstance() {
        static $instance_;
        if(!isset($instance_)) {
            $instance_= new Logger();
        }
        return $instance_;
    }

    // constructor
    // default log level: LOGGER_LEVEL_INFO
    // default log filename: syslog
    // @access private
    function Logger() {
        ini_set('log_errors', true);
        $this->setLevel( LOGGER_LEVEL_INFO );
        $this->setLogfile('syslog'); // default logging to Apache syslog
    }

    // @access public
    function setLevel($level) {
        if (LOGGER_LEVEL_DEBUG <= $level and $level <= LOGGER_LEVEL_CRITICAL) {
            $this->level_= $level;
        }
    }

    // @access public
    function getLevel() {
        return $this->level_;
    }

    // @access public
    function isDebugEnable() {
        return ($this->level_ <= LOGGER_LEVEL_DEBUG);
    }

    // @access public
    function setLogfile($filename) {
        // checks if filename exists and is writable
        if ($filename == 'syslog' or is_writable($filename)) {
            $this->logfile_= $filename;
            ini_set('error_log', $filename );
        }
    }

    // @access public
    function getLogfile() {
        return $this->logfile_;
    }

    // @access public
    function debug($message) {
        $this->log( LOGGER_LEVEL_DEBUG, 'DEBUG: ' . $message);
    }

    // @access public
    function info($message) {
        $this->log( LOGGER_LEVEL_INFO, 'INFO: ' . $message);
    }

    // @access public
    function notice($message) {
        $this->log( LOGGER_LEVEL_NOTICE, 'NOTICE: ' . $message);
    }

    // @access public
    function warn($message) {
        $this->log( LOGGER_LEVEL_WARNING, 'WARN: ' . $message);
    }

    // @access public
    function error($message) {
        $this->log( LOGGER_LEVEL_ERROR, 'ERROR: ' . $message);
    }

    // @access public
    function critical($message) {
        $this->log( LOGGER_LEVEL_CRITICAL, 'CRITICAL: ' . $message);
    }

    // @access protected
    function log( $level, $message ) {
        if ($level >= $this->level_ ) {
            error_log( $message, 0 );
        }
    }

}

?>
