<?PHP
/*
 * Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: WebCTCEAdapter.inc,v 1.7 2006/09/07 15:06:30 vlry Exp $
 */

/**
 * Define default request paths for WebCT CE AutoSignon adapter.
 */
define('WEBCTCE_DEFAULT_AS_REQUEST_PATH', '/webct/public/autosignon');

/**
 * Define default request paths for WebCT CE System Integration (API) adapter.
 */
define('WEBCTCE_DEFAULT_API_REQUEST_PATH', '/webct/public/serve_webctdb');

/**
 * Define default request paths for WebCT CE System Integration (IMS) adapter.
 */
define('WEBCTCE_DEFAULT_IMS_REQUEST_PATH', '/webct/systemIntegrationApi.dowebct');

/** 
 * Define default IMS source 
 */
define('WEBCTCE_DEFAULT_IMS_SOURCE', 'WISEBEDaai');

/**
 * The abstract WebCTCEAdapter is use to handle the API, IMS and AutoSignon request.
 *
 * @version $Revision: 1.7 $
 * @author Valery Tschopp - SWITCH
 *
 * @abstract
 * @package WebCTAPI
 */
class WebCTCEAdapter {

    /** 
	  * Base server url
	  * @var string
	  * @access private
	  */
    var $server_url_;
    /** 
	  * Shared secret for adapter
	  * @var string
	  * @access private
     */
    var $secret_;
    
    /**
     * Abstract constructor: Creates a new WebCT CE Adapter.
     * The server base URL is given without path extention in format: 
     * http[s]://<host>:<port>
     * 
     * @param string $server_url The WebCT CE server base URL.
     * @param string $secret The shared secret for this adapter (either AS or IMS).
     */
    function WebCTCEAdapter($server_url, $secret) {
    
        // abstract class
        if ( !is_subclass_of($this,"WebCTCEAdapter") ) {
            trigger_error("Abstract class cannot be instanciate", E_USER_ERROR);
            return;
        }

        $this->server_url_= $server_url;
        $this->secret_= $secret;
    }

    /**
    * Sets the shared secret.
    *
    * @param string $secret The shared secret (AS or IMS).
    */
    function setSecret( $secret ) {
        $this->secret_= $secret;   
    }
    
    /**
    * Builds and returns the default AutoSignon request URL. 
    * This URL is built with the WebCT CE server base URL 
    * and the WEBCTCE_DEFAULT_AS_REQUEST_PATH.
    *
    * @return string The default AutoSignon request URL 
    */
    function getASRequestURL() {
        $req_url= $this->server_url_ . WEBCTCE_DEFAULT_AS_REQUEST_PATH;
        return $req_url;
    }

    /**
    * Builds and returns the default API request URL. 
    * This URL is built with the WebCT CE server base URL 
    * and the WEBCTCE_DEFAULT_API_REQUEST_PATH.
    *
    * @return string The default API request URL 
    */
    function getAPIRequestURL() {
        $req_url= $this->server_url_ . WEBCTCE_DEFAULT_API_REQUEST_PATH;
        return $req_url;
    }


    /**
    * Builds and returns the default IMS request URL. 
    * This URL is built with the WebCT CE server base URL 
    * and the WEBCTCE_DEFAULT_IMS_REQUEST_PATH.
    *
    * @return string The default IMS request URL
    */
    function getIMSRequestURL() {
        $req_url= $this->server_url_ . WEBCTCE_DEFAULT_IMS_REQUEST_PATH;
        return $req_url;
    }
}


?>
