<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2006 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTIMS_IMSDataObjectNode.class.inc,v 1.10 2006/09/07 15:06:30 vlry Exp $
 */

require_once("include/pre-php-4.2.inc");

/**
 * WebCTIMS_IMSDataObjectNode  represents a data node in an IMS data
 * object.
 *
 * @package WebCTAPI
 * @version $Revision: 1.10 $
 *
 */
class WebCTIMS_IMSDataObjectNode {

    /* ***************************************************************** */
    /*                    private members                                */
    /* ***************************************************************** */
    
    /**
    * the array of child nodes
    *
    * @var array
    * @access private 
    */    
    var $_children = array();

    /**
    * the array of attributes
    *
    * @var array
    * @access private 
    */    
    var $_attributes = array();

    /**
    * the text value 
    *
    * @var string
    * @access private 
    */    
    var $_value = false;

    /**
    * the node name 
    *
    * @var string
    * @access private 
    */    
    var $_name = false;

    /* ***************************************************************** */
    /*                    constructor                                    */
    /* ***************************************************************** */
    /**
    * constructor
    *
    * @param string $name  the node name 
    * @param array  $attributes  the node attributes 
    */
    function WebCTIMS_IMSDataObjectNode($name=false, $value=false, $attributes=NULL) {
       if ($name != false) {
          $this->setName($name);
       }
       if (! is_null($attributes) && is_array($attributes)) {
         $this->_attributes = $attributes;
       }
       if ($value != false) {
         $this->setValue($value); 
       }
    }
    
    /* ***************************************************************** */
    /*                    methods                                        */
    /* ***************************************************************** */
    
    /**
    * Sets the name of this node.
    *
    * @param string $name the name
    */
    function setName($name) {
       $this->_name = $name;
    }
    
    /* ***************************************************************** */
    /**
    * Replies the name of this node.
    *
    * @return string the name
    */
    function getName() {
       return $this->_name;
    }

    /* ***************************************************************** */   
    /**
    * Sets the value of this node.
    *
    * @param string $value  the value
    */    
    function setValue($value) {
       $this->_value = $value;
    }

    /* ***************************************************************** */
    /**
    * Replies the value of this node.
    *
    * @return string the value of this node; false, if no value is
    *   defined 
    */    
    function getValue() {
       return $this->_value;
    }

    /* ***************************************************************** */ 
    /**
    * Adds a child node.
    *
    * @param IMSDataObjectNode  $child  the child node. Must not be NULL.
    *
    */
    function addChild(&$child) {
       if (is_null($child) || ! is_object($child) || ! is_a($child, "WebCTIMS_IMSDataObjectNode")) {
          trigger_error(
            "WebCTIMS_IMSDataObjectNode::addChild: expected instance of 'WebCTIMS_IMSDataObjectNode' as first parameter, got '$child'",
            E_USER_ERROR
          );
       }
       //REMIND: do NOT use array_push. array_push creates a copy of the object 
       //        instead of appending a reference ot the child at the end of
       //        the list of children
       //
       $last = count($this->_children);
       $this->_children[$last] =& $child;
       
             
    }

    /* ***************************************************************** */
    /**
    * Creates a child node,
    *
    * @param string $name   the name of the node 
    * @param string $value   the node value
    * @param array  $attributes   the attributes 
    *
    * @return IMSDataObjectNode  a reference to the the new child node 
    *
    */
    function &createChild($name, $value=false, $attributes=NULL) {
       $child = new WebCTIMS_IMSDataObjectNode($name,false,$attributes);
       $this->addChild(&$child);
       if ($value) {
         $child->setValue($value);  
       }
       return $child;
    }

    /* ***************************************************************** */
    /**
    * Replies the number of children.
    *
    * @return int the number of children 
    *
    */
    function getNumChildren() {
       return count($this->_children);
    }

    /* ***************************************************************** */
    /**
    * Replies a child node by positional index .
    *
    * @param int $idx  the index
    *
    * @return IMSDataObjectNode  the child node; NULL, if the index is out of range
    *
    */
    function getChild($idx) {
       if ($idx < 0 || $idx >= count($this->_children)) {
         $child= null;
         return $child; // null 
       }
       return $this->_children[$idx];
    }

    /* ***************************************************************** */   
    /**
    * Sets the value of an attribute. 
    *
    * @param string $name  the attribute name 
    * @param string $value  the attribute value
    *
    */    
    function setAttribute($name, $value) {
      $this->_attributes[$name] = $value;
    }

    /* ***************************************************************** */
    /**
    * Replies the value of an attribute.
    *
    * @param string $name  the attribute name 
    * @return string the attribute value; false, if no attribute $name exists
    *
    */    
    function getAttribute($name) {
      if (isset($this->_attributes[$name])) {
        return $this->_attributes[$name];
      } else {
        return false; 
      }     
    }

    /* ***************************************************************** */   
    /**
    * Converts the node and its children to an XML representation
    *
    * @param WebCTIMS_XMLWriter $xmlWriter   the XML writer 
    * 
    * @see toXML()
    */        
    function writeXML(&$xmlWriter) {
       if (! is_object($xmlWriter) || ! is_a($xmlWriter, "WebCTIMS_XMLWriter")) {
         trigger_error(
           "WebCTIMS_IMSDataObjectNode::toXML: expected instance of 'WebCTIMS_XMLWriter' as first argument, got '$xmlWriter'",
           E_USER_ERROR
         ); 
       } 
       
       if ($this->_value == false) {         
         if ($this->getNumChildren() == 0) {
           // no children, just write the empty tag
           //
           $xmlWriter->writeEmptyTag(
             $this->_name,
             $this->_attributes
           ); 
         } else {
           // at least one child. Write the open tag, the children, and finally
           // the closing tag
           //
           $xmlWriter->writeOpenTag(
             $this->_name,
             $this->_attributes
           ); 
           $xmlWriter->incIndentLevel();
           foreach($this->_children as $child) {
             $child->writeXML(&$xmlWriter);
           }
           $xmlWriter->decIndentLevel();
           $xmlWriter->writeCloseTag(
             $this->_name
           );
         }
       } else {
         // writes an element with text
         //
         $xmlWriter->writeElementWithText(
             $this->_name,
             $this->_value,
             $this->_attributes        
         );
       }
    }


    /**
    * Replies an XML representation for this node 
    *
    * @param boolean $includeHeader  true, if the IMS request header shall
    *   be emitted; false, otherwise
    *
    * @see WebCTIMS_XMLWriter::writeIMSRequestHeader()
    * @see writeXML()
    */        

    function toXML($includeHeader=false,$xml_encoding= null) {
        $writer = new WebCTIMS_XMLWriter();
        if ($includeHeader) {
           $writer->writeIMSRequestHeader($xml_encoding);
        }
        $this->writeXML(&$writer);
        return $writer->getBuffer();
    }
}

?>
