<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTIMS_Response.class.inc,v 1.12 2006/09/29 12:51:26 vlry Exp $
 */
/* ****************************************************************** */

require_once('include/Logger.class.inc');

/**
* WebCTIMS_Response represents the response of an WebCT IMS API
* invocation.
*
* @version $Revision: 1.12 $
* @package WebCTAPI
*/
class WebCTIMS_Response {

   /* ***************************************************** */
   /*                  private members                      */
   /* ***************************************************** */
  /**
  * the array of response HTTP headers 
  *
  * @var array
  * @access private
  */
  var $_headers        = array();  

  /**
  * the server response body 
  *
  * @var string
  * @access private
  */
  var $_serverResponse = "";

  /**
  * the status of the response 
  *
  * @var boolean 
  * @access private
  */
  var $_isOK           = false;

   /* ***************************************************** */
   /*                  constructor                          */
   /* ***************************************************** */
   
  /**
  * constructor
  */
  function WebCTIMS_Response() {    
  }
  
   /* ***************************************************** */
   /*                  methods                              */
   /* ***************************************************** */
   
   /* ***************************************************** */  
  /**
  * Reads the response from a socket.
  *
  * @param handle  $handle  the socket handle 
  */
  function read($handle) {
    // read the HTTP headers, up to an empty line
    //
    
    while(! feof($handle)) {
       $buffer= fgets($handle, 1024);
       $line = rtrim($buffer);
       
       if ($line == "") {
         break; 
       }
       // parse headers
       if (ereg("^(.*): (.*)$", $line, $regs)) {
         $name = $regs[1];
         $value = $regs[2];               
         $this->_headers[$name]= $value;   
       }
    }
    
    // read the body
    //
    while(! feof($handle)) {
      $this->_serverResponse .= @fread($handle, 512);
    }
     
    // check whether the response is OK
    $logger= Logger::getInstance();
    $logger->debug("WebCTIMS_Response:read:\n" . $this->_serverResponse);

    //   
    if (eregi('success',$this->_serverResponse)) {
      $this->_isOK = true; 
    } 
    elseif (ereg('Error',$this->_serverResponse)) {
      $this->_isOK = true;  
    } 
    elseif (eregi('membership import failed',$this->_serverResponse)) {
      $this->_isOK = true;  
    } 
    elseif (strpos($this->_serverResponse, 'Error(99):')) {
      $this->_isOK = true;  
    } 
    elseif (eregi('<consortiaid>',$this->_serverResponse)) {
      $this->_isOK = true;  
    } 
    elseif (eregi('<userid>',$this->_serverResponse)) {
      $this->_isOK = true;  
    } 
    else {
       $this->_isOK = false;  
    }
    $logger->debug("WebCTIMS_Response:read:_isOK: " . ($this->_isOK ? "true" : "false") );
    
  }

  /* ***************************************************** */  
  /**
  * Replies true, if the response indicates, that the IMS operation was successful.
  *
  * @retrun boolean  true, if the response indicates, that the IMS operation was successful.
  */
  
  function isOK() {
    return $this->_isOK;
  }
  
  /* ***************************************************** */    
  /**
  * replies the body part of the server response 
  *
  * @return string the body part of the server response
  */ 
  function getServerResponse() {
    return $this->_serverResponse;  
  }

  /* ***************************************************** */     
  /** 
  * Replies the array of response HTTP headers 
  *
  * @return array  an array of response HTTP headers 
  */
  function getResponseHeaders() {
    return $this->_headers;
  }  
}

?>
