<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTIMS_SwitchAAIAdapting.class.inc,v 1.26 2006/09/07 15:06:30 vlry Exp $
 */
/* ****************************************************************** */

require_once("include/webct/WebCTIMS.inc");

/**
 * ISO 8859-1 table to convert accentued characters
 */
$ISO_8859_1_ACCENT_FILTERS= array(
    192 => 'A', 193 => 'A', 194 => 'A', 195 => 'A', 196 => 'Ae', 197 => 'A', 
    198 => 'AE', 199 => 'C', 
    200 => 'E', 201 => 'E', 202 => 'E', 203 => 'E',
    204 => 'I', 205 => 'I', 206 => 'I', 207 => 'I',
    209 => 'N', 
    210 => 'O', 211 => 'O', 212 => 'O', 213 => 'O', 214 => 'Oe', 216 => 'O',
    217 => 'U', 218 => 'U', 219 => 'U', 220 => 'Ue', 
    221 => 'Y', 223 => 'ss',
    224 => 'a', 225 => 'a', 226 => 'a', 227 => 'a', 228 => 'ae', 229 => 'a',
    230 => 'ae', 231 => 'c',
    232 => 'e', 233 => 'e', 234 => 'e', 235 => 'e',
    236 => 'i', 237 => 'i', 238 => 'i', 239 => 'i',
    241 => 'n',
    242 => 'o', 243 => 'o', 244 => 'o', 245 => 'o', 246 => 'oe', 248 => 'o', 
    249 => 'u', 250 => 'u', 251 => 'u', 252 => 'ue',
    253 => 'y', 255 => 'y'
);

 /**
 * WebCTIMS_SwitchAAIAdapting is a static utitliy class only. It provides
 * methods for creating appropriate WebCT IMS objects given a set of 
 * authenticated SwitchAAI attributes.
 *
 *
 * @package WebCTAPI
 * @version $Revision: 1.26 $
 *
 * @static 
 * @abstract
 *
 */
class WebCTIMS_SwitchAAIAdapting {
  
   /* *************************************************************** */
    /**
    * Replies the current date in the format YYYY-MM-DD HH:MM:SS.
    *
    * @return string the current date in the format YYYY-MM-DD
    *
    * @static 
    */
    function getDateTime() {
       return date("Y-m-d H:i:s");
    }

    /* *************************************************************** */   
    /**
    * Creates a WebCT ID given a Switch AAI ID (a so called
    * Shib-WISEBEDPerson-personUniqueID). A Shib-WISEBEDPerson-personUniqueID has the format
    * &lt;unique-local-id&gt;@&lt;domain&gt;, i.e. ab1234@unibe.ch.
    *
    * This method replies a derived WebCT user ID in the format
    * wisebedaai_&lt;domain&gt;_&lt;unique-local-id&gt;, i.e.
    * wisebedaai_unibe.ch_ab1234.
    *
    * @param string $Shib-WISEBEDPerson-personUniqueID  the Switch AAI ID 
    * @return string   the WebCT ID 
    *
    * @static 
    */
    function createWebCTID($uniqueid) {
	   /* check if Shib-WISEBEDPerson-personUniqueID contains an '@' */
       if (strpos($uniqueid,'@')) {
          $webctID= strtr($uniqueid, '@', '_');
          //$webctID = "wisebedaai_";
          //ereg("^([^\@]+)\@(.+)", $uniqueid, $regs);
          //$webctID .= $regs[2];
          //$webctID .= "_";
          //$webctID .= $regs[1];
       } else {
          $webctID = "snaportal_";
          $webctID .= $uniqueid;
       }
       
       return $webctID;
    }

    /* *************************************************************** */   
    /**
    * Creates a user WebCT ID given a AAI UniqueID (Shib-WISEBEDPerson-personUniqueID),
    * a firstname and a lastname.
    * The function filters all accentued characters (ISO-8859-1) and
    * replace with non accentued character.
    * 
    * A Shib-WISEBEDPerson-personUniqueID has the format:
    * &lt;unique-id&gt;@&lt;domain&gt;, i.e. 3453245345@unizh.ch
    *
    * This method replies a derived WebCT user ID in the format:
    * &lt;domain&gt;_&lt;lastname&gt;_&lt;firstname&gt;_&lt;unique-id&gt;,
    * i.e. unizh.ch_mueller_hans_3453245345
    *
    * @param string $firstname     User firstname
    * @param string $lastname      User lastname
    * @param string $uniqueid      The AAI UniqueID (Shib-WISEBEDPerson-personUniqueID)
    * @return string   the WebCT ID
    *
    * @static 
    */
    function createUserWebCTID($firstname, $lastname, $uniqueid) {
       $webctID= "";
       /* check if Shib-WISEBEDPerson-personUniqueID contains an '@' */
       if (strpos($uniqueid,'@')) {
           ereg("^([^\@]+)\@(.+)", $uniqueid, $regs);
           $webctID .= $regs[2]; // domain
           $webctID .= "_" . $lastname . "_" . $firstname . "_";
           $webctID .= $regs[1]; // id
       } else {
           $webctID.= "snaportal_" . $lastname . "_" . $firstname . "_" . $uniqueid;
       }
       
       $webctID= strtolower($webctID);
       $webctID= WebCTIMS_SwitchAAIAdapting::filter_iso_8859_1($webctID);
       $webctID= WebCTIMS_SwitchAAIAdapting::filter_special_chars($webctID);
       return $webctID;
    }

    /**
    * Removes or converts special characters from a given string.
    *
    * @param string $in  The string to filter
    * @return string     The filtered string (ISO-8859-1)
    *
    * @static 
    */
    function filter_special_chars( $in ) {
        $out= $in;
        // PHP magic_quotes_gpc is default ON, quote and single quote are escaped!
        $out= stripslashes( $out );
        // remove: single quote are removed
        $out= str_replace("'", "", $out);
        // convert:  Jean-Pierre | Jean Pierre => Jean_Pierre
        $out= strtr($out, '- ', '__');
        return $out;
    }

    /**
    * Removes the accentued characters from a given string.
    *
    * @param string $str The string to filter
    * @return string     The filtered string (ISO-8859-1)
    *
    * @static 
    */
    function filter_iso_8859_1( $in ) {
       global $ISO_8859_1_ACCENT_FILTERS;
       $unknown= '#';
       $str_l= strlen( $in );
       $out= "";
//XXX echo "XXX: ISO in= '$in', l=$str_l\n";

       for( $i= 0; $i<$str_l; $i++ ) {
          $ord= ord( $in[$i] );
//XXX echo "XXX: in[$i]=$ord\n";

          if (array_key_exists($ord, $ISO_8859_1_ACCENT_FILTERS)) {
             // in conversion table, replace
             $filtered= $ISO_8859_1_ACCENT_FILTERS[$ord];
             $out .= $filtered;
          }
          elseif ($ord >= 160) {
             // not in conversion table, replace with ?
             $out .= $unknown;
          }
          else {
             $out .= chr( $ord );
          }
       }

       return $out;
    }

    /* *************************************************************** */   
    /**
    * Creates a random password consisting of 20 randomly chosen
    * characters from the set {a-z}.
    *
    * @param int $length  An optional length (default is 20)
    * @return string   the random password 
    *
    * @static 
    */    
    function createPassword($length= 20) {
       $pw = "";
       for ($i = 0; $i < $length; $i++) {
         $pw .= chr(rand(ord("a"), ord("z")));  
       }
       return $pw;
    }

    /* *************************************************************** */
    /**
    * Checks whether the array $aaiParams includes the four mandatory
    * SwitchAAI attributes 'Shib-WISEBEDPerson-personUniqueID', 'surname',
    * 'givenName', and 'Shib-WISEBEDPerson-homeOrganization'. 
    *
    * @param array   $aaiParams  a parameter array 
    * 
    * @param string  &$errorMessage  OUT-parameter for an error message
    *
    * @return boolean true, if all required parameters are included; false
    *   otherwise 
    *
    * @static 
    *
    * @access private 
    */        
    function _checkAAIParams($aaiParams, &$errorMessage) {
      if (!isset($aaiParams["Shib-WISEBEDPerson-personUniqueID"])) {
         $errorMessage = "mandatory WISEBEDaai attribute 'Shib-WISEBEDPerson-personUniqueID' missing";
         return false;
      }
      if (!isset($aaiParams["surname"])) {
         $errorMessage = "mandatory WISEBEDaai attribute 'surname' missing";
         return false;
      }
      if (!isset($aaiParams["givenName"])) {
         $errorMessage = "mandatory WISEBEDaai attribute 'givenName' missing";
         return false;
      }
      if (!isset($aaiParams["Shib-WISEBEDPerson-homeOrganization"])) {
         $errorMessage = "mandatory WISEBEDaai attribute 'Shib-WISEBEDPerson-homeOrganization' missing";
         return false;
      }
      return true; 
    }


    /* *************************************************************** */
    /**
    * Prepares an Enterprise Data Object.
    *
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node 
    *
    * @static 
    */            
    function prepareEnterpriseDataObject($apiParams = NULL) {
      if (! is_null($apiParams) && isset($apiParams["WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER"])) {
        $sourceSystemIdentifier = $apiParams["WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER"];
      } else {
        $sourceSystemIdentifier = WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER;
      }

      $enterprise = new WebCTIMS_IMSDataObjectNode("enterprise");
      
      // create and initialize node "enterprise/properties"
      //      
      $properties =& $enterprise->createChild("properties");
      $properties->createChild(
          "datasource",
          $sourceSystemIdentifier 
      );
      $properties->createChild(
             "datetime",
             WebCTIMS_SwitchAAIAdapting::getDateTime()
      );
      
      return $enterprise;
    }


    /* *************************************************************** */
    /**
    * Prepares an IMS data object for creating or updating a WebCT user
    * given a set of SwitchAAI attributes. This method creates an
    * object of type "Person" and initializes it with data from the
    * SwitchAAI attributes.
    *
    *
    * @param array $aaiParams the SwitchAAI attributes. This array should
    *   include the fields 'Shib-WISEBEDPerson-personUniqueID', 'surname',
    * 'givenName', and 'Shib-WISEBEDPerson-homeOrganization'. 
    *
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node 
    *
    * @static 
    */            
    function prepareCreateOrUpdateUserRequest($webctid, $aaiParams, $apiParams = NULL) {
      $person= null;
      if (is_null($aaiParams) ||! is_array($aaiParams)) {
        trigger_error(
          "WebCTIMS_SwitchAAIAdapting::prepareCreateOrUpdateUserRequest: expected array in argument \$aaiParams, got '$aaiParams'",
          E_USER_ERROR
        );
        return $person; // null
      }
      if (! WebCTIMS_SwitchAAIAdapting::_checkAAIParams($aaiParams, &$errorMessage)) {
        trigger_error(
          "WebCTIMS_SwitchAAIAdapting::prepareCreateOrUpdateUserRequest: illegal or incomplete SwitchAAI parameters. Details: " . $errorMessage,
          E_USER_ERROR
        );   
        return $person; // null
      }

      if (! is_null($apiParams) && isset($apiParams["WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER"])) {
        $ims_person_source = $apiParams["WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER"];
      } else {
        $ims_person_source = WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER;

      }

      // create and initialize node "enterprise/person"
      //      
      $person = new WebCTIMS_IMSDataObjectNode("person");

      // read aai parameters
      $aai_uniqueid= $aaiParams['Shib-WISEBEDPerson-personUniqueID'];
      $aai_firstname= $aaiParams['givenName'];
      $aai_lastname= $aaiParams['surname'];
      $aai_homeorg= $aaiParams['Shib-WISEBEDPerson-homeOrganization'];
 
      // unescape single quote
      $aai_firstname= stripslashes( $aai_firstname ); 
      $aai_lastname= stripslashes( $aai_lastname ); 
 
      $sourcedid =& $person->createChild("sourcedid");
      $sourcedid->createChild(
         "source",
         $ims_person_source
      );       
      $sourcedid->createChild(
         "id",
         $aai_uniqueid
      );  

      // don't overwrite password for designer or teaching assistant (ta)
      // it will allow WebDAV password
      $is_designer_or_ta= isset($aaiParams['roletype']) and
                          ( $aaiParams['roletype'] == WEBCT_IMS_OBJECT_ROLETYPE_DESIGNER or
                            $aaiParams['roletype'] == WEBCT_IMS_OBJECT_ROLETYPE_TEACHINGASSISTANT );
      if ($is_designer_or_ta) {
        $person->createChild(
            "userid", 
            $webctid
        );
      }
      else {
        // normal user, generate new unknown password
        $password= WebCTIMS_SwitchAAIAdapting::createPassword();      
        $person->createChild(
            "userid", 
            $webctid, 
            array("password" =>  $password)
        );  
      }
              
      $nameNode =& $person->createChild("name");                  
      // formatted name
      $nameNode->createChild(
         "fn",
         "$aai_firstname $aai_lastname"
      );  
        
      $n =& $nameNode->createChild("n");          
      $n->createChild(
          "given",
          $aai_firstname
      );  
      $n->createChild(
          "family",
          $aai_lastname
      );  

      $person->createChild(
         "datasource",
         $aai_homeorg
      );  
                  
      return $person;     
    }
    

    /* *************************************************************** */
    /**
    * Prepares an IMS data object for creating or updating membership
    * of a user to a WebCT course.
    *
    * @param string $Shib-WISEBEDPerson-personUniqueID  the SwitchAAI unique ID 
    *
    * @param string $courseID  the WebCT Course ID 
    *
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node 
    *
    * @static 
    */                  
    function prepareCourseMembershipRequest($ims_id, 
                                            $ims_source,
                                            $courseid, 
                                            $webctid= null, 
                                            $roletype= null,
                                            $recstatus= null) {
       $membership = new WebCTIMS_IMSDataObjectNode("membership");

      $sourcedid =& $membership->createChild("sourcedid");
      $sourcedid->createChild(
         "source",
         "WebCT"
      );       
      $sourcedid->createChild(
         "id",
         $courseid
      );  
      
      $member =& $membership->createChild("member");

      $sourcedid =& $member->createChild("sourcedid");
      $sourcedid->createChild(
         "source",
         $ims_source
      );       
      $sourcedid->createChild(
         "id",
         $ims_id
      );  
       
      $member->createChild("idtype", "1"); // person 
      //WARNING:
      //  TA should only be added (recstatus= 1) otherwise will be overwritten
      $role_options= array();
      if (! is_null($roletype)) {
         $role_options['roletype']= $roletype;
      }
      else {
         // default student
         $role_options['roletype']= WEBCT_IMS_OBJECT_ROLETYPE_STUDENT;
      }
      
      if ( !is_null($recstatus) and (1 <= $recstatus) and ($recstatus <= 3) ) {
         $role_options['recstatus']= "$recstatus";
      }

      $role =& $member->createChild("role", false, $role_options);
      $role->createChild("status", "1");

      if (!is_null($webctid)) {
         $role->createChild("userid", "$webctid");
      }
      //XXX var_dump($role);
      
      return $membership;
    }
    

    /* *************************************************************** */   
    /**
    * constructor 
    * @access private
    */
    function WebCTIMS_SwitchAAIAdapting() {
        trigger_error(
          "WebCTIMS_SwitchAAIAdapting::constructor: creating instances of this class is not allowed",
          E_USER_ERROR
        );      
    }
}

?>
