<?php
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaIMS_SwitchAAIAdapting.inc,v 1.14 2006/09/07 15:06:30 vlry Exp $
*/

require_once("include/webct/WebCTVista.inc");
require_once("include/webct/WebCTVistaIMS.inc");
require_once("include/webct/WebCTIMS_SwitchAAIAdapting.class.inc");
require_once("include/webct/WebCTIDFactory.inc");


/**
* WebCTIMS_SwitchAAIAdapting is a static utitliy class only. It provides
* methods for creating appropriate WebCT IMS objects given a set of
* authenticated SwitchAAI attributes.
*
* @author Valery Tschopp - SWITCH
* @package WebCTAPI
* @version $Revision: 1.14 $
*
* @static
*/
class WebCTVistaIMS_SwitchAAIAdapting extends WebCTIMS_SwitchAAIAdapting {
    
    /**
     * Creates a password.
     *
     * @static
     */
    function createPassword( $size= 15 ) {
        $passwd= '';
        $chars= "23456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMPQRSTUVWXYZ";
        $length= strlen($chars);
        for ($i= 0; $i<$size; $i++) {
            $rand= mt_rand(0, $length - 1);
            $passwd .= $chars{$rand};
        }
        return $passwd;

    }
    
    /**
    * Prepares an Enterprise Data Object.
    *
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node
    *
    * @static
    */
    function prepareEnterpriseDataObject($apiParams = NULL) {
        if (! is_null($apiParams) && isset($apiParams["WEBCTVISTA_IMS_ENTERPRISE_SOURCE"])) {
            $datasource= $apiParams["WEBCTVISTA_IMS_ENTERPRISE_SOURCE"];
        }
        else {
            $datasource= WEBCTVISTA_DEFAULT_IMS_ENTERPRISE_SOURCE;
        }
        
        $enterprise= new WebCTVistaIMS_IMSDataObjectNode("enterprise");
        
        // create and initialize node "enterprise/properties"
        //
        $properties=& $enterprise->createChild("properties");
        $properties->createChild(
            "datasource",
            $datasource
        );
        $datetime= WebCTVistaIMS_SwitchAAIAdapting::getISO8601DateTime();
        $properties->createChild(
            "datetime",
            $datetime
        );
        
        return $enterprise;
    }
    
    /**
    * Replies the current ISO-8601 date. ISO-8601 Format YYYY-MM-DDThh:mm:ss+GGGG
    * @return string The current ISO-8601 date
    * @static 
    */
    function getISO8601DateTime() {
       return date("Y-m-d\TH:i:sO");
    }
    
    
    /**
    * Prepares an IMS data object for creating or updating a WebCT user
    * given a set of SwitchAAI attributes. This method creates an
    * object of type "Person" and initializes it with data from the
    * SwitchAAI attributes.
    *
    *
    * @param array $aaiParams the SwitchAAI attributes. This array should
    *   include the fields 'Shib-WISEBEDPerson-personUniqueID', 'surname',
    * 'givenName', and 'Shib-WISEBEDPerson-homeOrganization'.
    *
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node
    *
    * @static
    */
    function prepareCreateOrUpdateUserRequest($aaiParams, $apiParams = NULL) {
        $mynull= null;
        if (is_null($aaiParams) ||! is_array($aaiParams)) {
            trigger_error(
            "WebCTVistaIMS_SwitchAAIAdapting::prepareCreateOrUpdateUserRequest: expected array in argument \$aaiParams, got '$aaiParams'",
            E_USER_ERROR
            );
            return $mynull;
        }
        if (! parent::_checkAAIParams($aaiParams, &$errorMessage)) {
            trigger_error(
            "WebCTVistaIMS_SwitchAAIAdapting::prepareCreateOrUpdateUserRequest: illegal or incomplete SwitchAAI parameters. Details: " . $errorMessage,
            E_USER_ERROR
            );
            return $mynull;
        }
        
        if (! is_null($apiParams) && isset($apiParams["WEBCTVISTA_IMS_PERSON_SOURCE"])) {
            $person_source= $apiParams["WEBCTVISTA_IMS_PERSON_SOURCE"];
        }
        else {
            $person_source= WEBCTVISTA_DEFAULT_IMS_PERSON_SOURCE;
            
        }
        
        // create and initialize node "enterprise/person"
        if (!is_null($apiParams) && isset($apiParams['WEBCTVISTA_IMS_OBJECT_RECSTATUS'])) {
            // person recstatus="1" => Create User
            // person recstatus="2" => Update User
            // person recstatus="3" => Delete User
            $recstatus= $apiParams['WEBCTVISTA_IMS_OBJECT_RECSTATUS'];
            $person= new WebCTVistaIMS_IMSDataObjectNode("person", false, array("recstatus" => "$recstatus"));
        }
        else {
            $person= new WebCTVistaIMS_IMSDataObjectNode("person");
        }
        
        $sourcedid=& $person->createChild("sourcedid");
        $sourcedid->createChild(
            "source",
            $person_source
        );
        $person_id= $aaiParams["Shib-WISEBEDPerson-personUniqueID"];
        $sourcedid->createChild(
            "id",
            $person_id
        );
        
        $webctid= $aaiParams['webctid'];        
        $person->createChild(
            "userid",
            $webctid
        );

        $name= $aaiParams['surname'];
        $firstname= $aaiParams['givenName'];
        $nameNode=& $person->createChild("name");
        $nameNode->createChild(
            "fn",
            $firstname . " " . $name
        );
        
        $n =& $nameNode->createChild("n");
        $n->createChild(
            "family",
            $name
        );
        $n->createChild(
            "given",
            $firstname
        );
        // person.email OPTIONAL
        if ( isset($aaiParams['email']) ) {
            $email= $aaiParams['email'];
            $person->createChild(
                "email",
                $email
            );
        }
        
        $datasource= $aaiParams["Shib-WISEBEDPerson-homeOrganization"];
        $person->createChild(
            "datasource",
            $datasource
        );
        
        // create optional <extension> element?
        $extensions= false;
        $extension_columns= array();
        // user password
        // <extension><WEBCREDENTIAL ...>password</WEBCREDENTIAL><extension>
        $extension_password= null;
        if (!is_null($apiParams) and isset($apiParams['WEBCTVISTA_IMS_PERSON_EXTENSION_PASSWORD'])) {
            $extension_password= $apiParams['WEBCTVISTA_IMS_PERSON_EXTENSION_PASSWORD'];
            $extensions= !empty($extension_password);
        }
        // custom columns
        // <extension><customcolumns ...><column ...></customcolumns><extension>
        if (!is_null($apiParams) and isset($apiParams['WEBCTVISTA_IMS_PERSON_EXTENSION_COLUMNS'])) {
            // get the custom columns array (name=>value,...)
            $extension_columns= $apiParams['WEBCTVISTA_IMS_PERSON_EXTENSION_COLUMNS'];
            $extensions= true;
        }
        

        if ($extensions) {
            // create the <extension> element
            $extension=& $person->createChild('extension');
            // add custom columns if any
            if (!empty($extension_columns)) {
                $customcolumns=& 
                    $extension->createChild('customcolumns', false, 
                                            array('xmlns' => 'http://www.webct.com/IMS'));
                foreach($extension_columns as $name => $value) {
                    $customcolumns->createChild('column', $value, array('name' => $name) );
                }
            }
            // add webcredential if any
            if (!empty($extension_password)) {
                $extension->createChild(
                    "WEBCREDENTIAL",
                    $extension_password,
                    array('xmlns' => 'http://www.webct.com/IMS')
                );
            }
        }
        
        return $person;
    }
    
    
    /* *************************************************************** */
    /**
    * Prepares an IMS data object for creating or updating membership
    * of a user to a WebCT Vista course.
    *
    * @param TODO
    * @return WebCTIMS_IMSDataObjectNode an instance of an IMS data object node
    *
    * @static
    */
    function prepareCreateOrUpdateMembershipRequest($aaiParams, $apiParams= null) {
        if (is_null($aaiParams) ||! is_array($aaiParams)) {
            trigger_error(
            "WebCTVistaIMS_SwitchAAIAdapting::prepareCourseMembershipRequest: expected array in argument \$aaiParams, got '$aaiParams'",
            E_USER_ERROR
            );
        }
        
        if (! is_null($apiParams) && isset($apiParams["WEBCTVISTA_IMS_MEMBERSHIP_SOURCE"])) {
            $membership_source= $apiParams["WEBCTVISTA_IMS_MEMBERSHIP_SOURCE"];
        }
        else {
            $membership_source= WEBCTVISTA_DEFAULT_IMS_MEMBERSHIP_SOURCE;
        }
        
        if (! is_null($apiParams) && isset($apiParams["WEBCTVISTA_IMS_MEMBER_SOURCE"])) {
            $member_source= $apiParams["WEBCTVISTA_IMS_MEMBER_SOURCE"];
        }
        else {
            $member_source= WEBCTVISTA_DEFAULT_IMS_MEMBER_SOURCE;
        }

        
        // <membership>
        //   <sourcedid>
        //     <source>$membership_source</source>
        //     <id>$courseid</id>
        //   </sourcedid>
        //   <member>
        //     <sourcedid>
        //       <source>$member_source</source>
        //       <id>$unique_id</id>
        //     </sourcedid>
        //     <idtype>1</idtype>
        //     <role roletype="01">
        //       <status>1</status>
        //       <userid>$webctid</userid>
        //       <email>$email</email>
        //       <datasource>AAI homeOrg</datasource>
        //     </role>
        //   </member>
        // </membership>
        
        $membership= new WebCTIMS_IMSDataObjectNode("membership");
        
        $sourcedid=& $membership->createChild("sourcedid");
        $sourcedid->createChild(
        "source",
        $membership_source
        );
        $course_id= $aaiParams['courseid'];
        $sourcedid->createChild(
        "id",
        $course_id
        );
        
        $member=& $membership->createChild("member");
        
        $sourcedid=& $member->createChild("sourcedid");
        $sourcedid->createChild(
        "source",
        $member_source
        );
        $person_id= $aaiParams['Shib-WISEBEDPerson-personUniqueID'];
        $sourcedid->createChild(
        "id",
        $person_id
        );
        
        // member.idtype=1 => Person
        $member->createChild("idtype", "1");

        $recstatus= false;
        if (!is_null($apiParams) && isset($apiParams['WEBCTVISTA_IMS_OBJECT_RECSTATUS'])) {
            // person recstatus="1" => Create User
            // person recstatus="2" => Update User
            // person recstatus="3" => Delete User
            $recstatus= $apiParams['WEBCTVISTA_IMS_OBJECT_RECSTATUS'];
        }
        // member.role.recstatus="1" => Create
        // member.role.recstatus="2" => Update
        // member.role.recstatus="3" => Delete
        // member.role.roletype="01" => Student
        $role_params['roletype']= '01';
        if ($recstatus) {
            $role_params['recstatus']= "$recstatus";   
        }
        $role=& $member->createChild("role", false, $role_params);
        // member.role.status=1 => active
        $role->createChild("status", "1");
        
        // member.role.userid := webctid
        $webctid= $aaiParams['webctid'];        
        $role->createChild(
        "userid",
        $webctid
        );
        
        $datasource= $aaiParams["Shib-WISEBEDPerson-homeOrganization"];
        $role->createChild(
        "datasource",
        $datasource
        );
        
        return $membership;
    }
    
  
    /**
    * constructor
    *
    * @access private
    */
    function WebCTVistaIMS_SwitchAAIAdapting() {
        trigger_error(
        "WebCTVistaIMS_SwitchAAIAdapting::constructor: creating instances of this class is not allowed",
        E_USER_ERROR
        );
    }
}

?>
