<?php
//
// $Id: create-roles.php,v 1.6 2006/11/07 11:46:16 vlry Exp $
// Copyright (c) 2005 SWITCH
// @author Valery Tschopp - SWITH
//

// bootstrap
//error_reporting(E_ALL);
error_reporting(E_ALL);
$include_path= '.' . PATH_SEPARATOR . '..' . PATH_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..';
ini_set('include_path',$include_path);
require_once("include/application.inc");


// minimal set of attribute require
$STANDARD_POLICY_ATTRIBUTE_IDS = array(
  ATID_SURNAME,
  ATID_GIVENNAME,
  ATID_EDUPERSONHOMEORGANIZATION,
  ATID_EDUPERSONHOMEORGANIZATIONTYPE,
  ATID_EDUPERSONAFFILIATION,
  ATID_MAIL
);


// creates and return a new role (url, title, adapter) for the 
// user (provider_id)
// default: hidden, open for subscription role
//          no description
//          default policy
function & createRole($provider_id, $url, $title, $adapter, 
                          $visible= false, $state= 'open', $description= null) {	
   global $STANDARD_POLICY_ATTRIBUTE_IDS;
   $role=& Role::create($provider_id, $title, $url );
   $role->setRoleAdapter($adapter);

   // default policy
   foreach($STANDARD_POLICY_ATTRIBUTE_IDS as $atid) {
      $role->addToPolicy($atid); 
   }

   if (!is_null($description)) {
      $role->setDescription($description);
   }
   
   // default: hidden, open for surbscription role
   $role->setVisible($visible);
   $state= strtolower($state);
   if ($state == 'free') {
      $role->setRoleAccessState(RESOURCE_ACCESS_STATE_FREE);
   }
   elseif ($state == 'closed') {
      $role->setRoleAccessState(RESOURCE_ACCESS_STATE_CLOSED);
   }
   elseif ($state == 'suspended') {
      $role->setRoleAccessState(RESOURCE_ACCESS_STATE_CLOSED);
   }
   else {
      $role->setRoleAccessState(RESOURCE_ACCESS_STATE_OPEN);
   }

   // store in DB
   $role->persist();
   return $role;
}

// set the adapter parameters for the role
function setAdapterParameters(&$role, $parameters) {
   $role->deleteAdapterParameters();
   foreach($parameters as $parameter => $value) {
      $value= trim( $value );
      if (!empty($value)) {
         $role->insertAdapterParameter($parameter, $value);
      }
   }
}

// parse the arguments list -> options
function parseArgs($argv= array(), $options) {
   for($x= 0; $x<count($argv); $x++) {
      $opt=NULL;
      if($argv[$x]{0} == "-") {
         $opt= $argv[$x];
         $options[substr($opt,1,strlen($opt))]= true;
      }
      if ($opt != NULL) {
         if ( isset($argv[$x+1]) && ($argv[$x+1] != "") && ($argv[$x+1]{0} != "-")) {
            $options[substr($opt,1,strlen($opt))]= $argv[$x+1];
         } 
      }
   }
}

function validArgs($options) {
   if (empty($options)) {
      echo "ERROR: Missing arguments\n";
      return false;
   }
   if (isset($options['h'])) return false;
   if (!isset($options['n']) or !is_numeric($options['n'])) {
      echo "ERROR: Missing or invalid argument '-n'\n";
      return false;
   }
   if (!isset($options['provider']) or !is_string($options['provider'])) {
      echo "ERROR: Missing or invalid argument '-provider'\n";
      return false;
   }
   if (!isset($options['url']) or !is_string($options['url'])) {
      echo "ERROR: Missing or invalid argument '-url'\n";
      return false;
   }
   if (!isset($options['api_secret']) or !is_string($options['api_secret'])) {
      echo "ERROR: Missing or invalid argument '-api_secret'\n";
      return false;
   }
   if (!isset($options['as_secret']) or !is_string($options['as_secret'])) {
      echo "ERROR: Missing or invalid argument '-as_secret'\n";
      return false;
   }
   return true;
}

function printUsage() {
   echo "Generates N new dummy roles, based on adapter 'webct-ce-net'.\n";
   echo "Usage:\n";
   echo " php generate-roles.php [arguments] ...\n"; 
   echo "  mandatory arguments:\n";
   echo "    -n <n_roles> \n";
   echo "    -provider <user_id> \n";
   echo "    -url <webct_server_url>\n";
   echo "    -api_secret <webct_api_secret>\n";
   echo "    -as_secret <webct_autosignon_secret>\n";
   echo "  optional arguments:\n";
//   echo "    -adapter <adapter_name>  default: 'webct-ce-net'\n";
   echo "    -description <description>\n";
   echo "    -title_prefix <prefix>   default: 'WebCT CE:'\n";
   echo "    -ims_source <ims_source> default: 'WISEBEDaai'\n";
   echo "    -state <state>           default: 'open'\n";
   echo "    -visible                 creates visible roles\n";
   echo "    -mywebct                 enable myWebCT login\n";
   echo "    -v                       verbose\n";
   echo "    -h                       usage\n";
}

//
// main
//
$argv= $_SERVER['argv'];
$options= array();

// set default arguments
//$options['adapter']= 'webct-ce-net';
$options['title_prefix']= 'WebCT CE:';
$options['ims_source']= 'WISEBEDaai';
$options['state']= 'open';


// parse and validate args
parseArgs($argv,&$options);
if( !validArgs($options)) {
   printUsage();
	exit;
}
$verbose= isset($options['v']);


// role provider
$uid= $options['provider'];
if ($verbose)  echo "INFO: checks user $uid\n";
$provider= Administrator::reconstituteByUid($uid);
if (is_null($provider) or ! $provider->canCreateRole()) {
   echo "ERROR: $uid is not a Portal or Role Administrator.\n";
   exit;
}
$provider_id= $provider->getID();


//
// adapter := webct-ce-net
//
$adapter= 'webct-ce-net';

// invariant role parameters
//$adapter= $options['adapter'];
$title_prefix= null;
if (isset($options['title_prefix']) and is_string($options['title_prefix'])) {
   $title_prefix= $options['title_prefix'];
}
$url= $options['url'];
$ims_source= $options['ims_source'];
$api_secret= $options['api_secret'];
$as_secret= $options['as_secret'];
$visible= isset($options['visible']);
$state= $options['state'];
$mywebct= isset($options['mywebct']);

// optional description
$description= null;
if (isset($options['description']) and is_string($options['description'])) {
   $description= $options['description'];
}

// how many roles to generate
$n_roles= $options['n'];
if ($verbose)  echo "INFO: generates $n_roles role adapters\n";


// adapter parameters
$adapter_params= array();
$adapter_params['WEBCTCE_SERVER_URL']= $url;
$adapter_params['WEBCTCE_IMS_SOURCE']= $ims_source;
$adapter_params['WEBCTCE_IMS_SHARED_SECRET']= $api_secret;
$adapter_params['WEBCTCE_AS_SHARED_SECRET']= $as_secret;

// NET-ETHZ group specific params
$adapter_params['WEBCTCE_NET_FIELD_STUDYBRANCH3']= 'Studiengang';
$adapter_params['WEBCTCE_NET_FIELD_STUDYLEVEL']= 'Studienstufe';
if ($mywebct)  {
   $adapter_params['WEBCTCE_NET_ENABLE_MYWEBCT_LOGIN']= true;
}
   
if ($verbose) {
   echo "INFO: adapter '$adapter' parameters:\n";
   foreach ($adapter_params as $param => $value) {
      echo "      $param = $value\n";
   }
}

// create N role adapters with parameters
for ($i= 0; $i < $n_roles; $i++) {
   // dynamic role parameters
   $title= date("Ymd:His") . ":$i";
   if ($title_prefix) {
      $title= "$title_prefix $title";
   }
   if ($verbose)  echo "INFO: creates role ($provider_id,'$title','$url',$adapter," . ($visible?"true":"false") . ",$state)\n";
   if ($verbose and $description)  echo "      $description\n";
  
   $role= createRole($provider_id,$title,$url,$adapter,$visible,$state,$description);
   setAdapterParameters(&$role, $adapter_params);   
}

if ($verbose)  echo "INFO: done.\n";





?>
