<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: change-password.php,v 1.23 2005/10/14 15:57:20 vlry Exp $
 */


/**
* This is the start page for the administration part of the SNA-Portal.
*
* <b>Request Parameters</b>
* <ul>
*   <li>uid=<uid>: OPTIONAL. The internal user ID of the user whose
*     password shall be changed. If present, it is ensured that the current 
*     administrator is actually allowed to change the password user <uid>.
*     If missing, changes the password of the current administrator.
*
*   <li>url=<follow-up-url>: OPTIONAL. The URL a user is redirect to
*      after having changed the password. If missing, the URL is
*      initialized with the value of HTTP_REFERER.
*  </ul>
*
* 
* @version $Revision: 1.23 $
* @package AdminPortal
*/


/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */

/**
* Validates the input supplied as HTTP request parameters.
* 
* If necessary, replies error messages in &$input_errors for the
* following request parameters:
* <ul>
*   <li>$input_errors["new_password_1"]  - errors related to the first 
*      password (missing password, illegal format)
*   </li>
*   <li>$input_errors["new_password_2"]  - errors related to the second
*      password (inconsistency with the first password)
*   </li>
*
* </ul>
*
* @param array &$input_errors  an array with error messages. Initially empty,
*   the function uses it as out-paramter in which error messages are replied.
*
* @access private
*/
function validate_input(&$input_errors) {
  $ret = true; 
  if (ereg("[ \t\r\n]+", $_REQUEST["new_password_1"])) {
    $ret = false;
    $input_errors["new_password_1"] = "The password must not include blanks, tabs, carriage returns and/or line feeds. Please enter the password again.";
  }
  elseif (empty($_REQUEST['new_password_1']) ) {
    $ret = false;
    $input_errors["new_password_1"] = "The password is empty. Please enter a password.";
  }
  if (strcmp($_REQUEST["new_password_1"], $_REQUEST["new_password_2"]) != 0) {
    $ret = false;
    $input_errors["new_password_2"] = "The repeated password is not identical to the first password. Please enter the password again.";
  }

  return $ret; 
}





// bootstrap application
//
require_once("include/application.inc");

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("current user is not priviledged to enter this page"); 
}
Session::init($admin);


// if request parameter uid missing, initialize it with the internal User ID
// of the current administrator user
// 
if (! isset($_REQUEST["uid"])) {
  $_REQUEST["uid"] = $admin->getUserId();
}
if (! isset($_REQUEST["url"])) {
  $_REQUEST["url"] = $_SERVER["HTTP_REFERER"];
}

//  load the user object
//
$user=& User::reconstituteById($_REQUEST["uid"]);
if (is_null($user)) {
  ErrorHandling::fatalErrorAndExit("no user with user ID '$uid' found");
}

// check wheter the current user is allowed to see details about
// this user
//
if (! $admin->canChangePasswordOf($user)) {
  ErrorHandling::fatalErrorAndExit("you do not have enough privileges to change the password of this user");
}


$input_errors = array();
if (isset($_REQUEST["do_update"])) {
  if (validate_input(&$input_errors)) {
    $new_password= $_REQUEST['new_password_1'];
    if (empty($new_password)) {
      $user->deleteAttribute(ATID_LOCALPASSWORD);
    } else {
      $user->storeAttribute(ATID_LOCALPASSWORD, $new_password); 
    }
    header("Location: " . $_REQUEST["url"]);
  } 
}


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

  <h1>Change Local Password</h1>
 
  <?php if ($admin->getUid() == $user->getUid()): ?>
    Please enter your new local password twice and click on <span class="button-explained">Update</span>.  
  <?php else: ?>
    Please enter the new local password of user <strong><?= $user->getUid() ?></strong>
    twice and click on <span class="button-explained">Update</span>. 
  <?php endif; ?> 
  <p>
  Changing the password of a AAI user will not change it. It will only set a local password.
  <p>
  
  <form action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  
  <table width="500" border="0" cellspacing="0" class="form-table">
    <input type="hidden" name="uid" value="<?= $_REQUEST["uid"] ?>" />
    <input type="hidden" name="do_update" value="1" />
    <input type="hidden" name="url" value="<?= $_REQUEST["url"] ?>" />
    <tr>
      <td width="30%" valign="top" class="form-label form-first">
      New Password
      </td>
      <td width="70%" valign="top" class="form-entry form-first">
      <input type="password" name="new_password_1" value="" size="10" />
     <?php if (isset($input_errors["new_password_1"])): ?>
       <br/><span class="error-message"><?= $input_errors["new_password_1"] ?></span>
     <?php endif; ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      New Password (repeated)
      </td>
      <td width="70%" valign="top" class="form-entry">
      <input type="password" name="new_password_2" value="" size="10" />      
     <?php if ( isset($input_errors["new_password_2"]) ): ?>
       <br/><span class="error-message"><?= $input_errors["new_password_2"] ?></span>
     <?php endif; ?>

     </td>
      </tr>

    <tr>
      <td width="100%" valign="top" align="center" class="form-control" colspan="2">
      <input type="submit" value="Update" />
      </td>    
    </tr>
  </table>
  </form>

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
