<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: edit-custom-role-aap.php,v 1.24 2005/10/11 07:55:01 vlry Exp $
 */

/**
* This page edit the list of required custom attributes for a role.
*
* <b>Request Parameters</b>
* <ul>
*   <li>rid=<rid>: MANDATORY. The internal role ID. <rid> must be
*    a valid integer > 0 and the current administrator must be allowed
*    to edit the role with ID <rid>.
*  </ul>
*
* 
* @version $Revision: 1.24 $
* @package AdminPortal
*/

/**
 * Ensures request parameters  be valid.
 *
 * @access private
 */
function ensure_request_parameters_valid() {
  if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing");	
  }
  $rid = $_REQUEST["rid"];
  if (! is_numeric($rid) || $rid < 0) {
    ErrorHandling::fatalErrorAndExit("illegal value '$rid' for request parameter 'rid'. Aborting.");	
  }
}

/**
 * Updates the AAP for a role from the current  request parameters in $_REQUEST
 * 
 * @param array $attribute_type_ids an array of attribute type IDS
 *
 * @param Role $role     the role whose AAP is to
 *                                   be updated 
 *
 * @access private
 */
function update_aap($attribute_type_ids, $role) {
 
        foreach ($attribute_type_ids as $atid) {
                $role->removeFromPolicy((int)$atid);
        }
 
        foreach ($_REQUEST as $key=>$value) {        	
                if (substr($key, 0, 6) == "sattr-" && $value) {                	
                	$atid=substr($key, 6);                	
                        $role->addToPolicy((int)$atid);
                }
        }
}

/**
 * Action handler for action do_create
 *
 * @param MySQLConnection $conn        a MySQL database connection
 * @param Role   $role    the parent role for the custom attribute
 *
 * @access private
 */
function handle_do_create($conn, $role) {
   $name = $_REQUEST["attr-name-new"];
   $desc = $_REQUEST["attr-description-new"];
   
   $atid = AttributeType::addAttributeType($name, $desc);
   $role->addToPolicy($atid);
}

/* **************************************************************** */
/**
 * Action handler for action do_update
 *
 * @param MySQLConnection $conn    a MySQL database connection
 * @param Role  $role   the parent role for the custom attribute
 *
 * @access private
 */
function handle_do_update($conn, $role) {
   $atid = (int)$_REQUEST["edited-atid"];
   $name = $_REQUEST["attr-name-edit"];   
   $desc = $_REQUEST["attr-description-edit"];
   AttributeType::updateAttributeType($atid,$name, $desc);
}

/* **************************************************************** */
/**
 * Action handler for action do_delete
 *
 * @param MySQLConnection $conn    a MySQL database connection
 * @param Role $role   the parent role for the custom attribute
 *
 * @access private
 */
function handle_do_delete($conn, $role) {
   foreach ($_REQUEST as $name=>$value) {
     if (ereg("^atid-([0-9]+)$", $name, $regs)) {
       $atid = (int)$regs[1];
       $role->removeFromPolicy($atid);
       AttributeType::deleteAttributeType($atid);	
     }
   }
}

//
// TODO: remove embedded SQL
//

/* **************************************************************** */
/**
 * Replies a list of custom attribute details for a given role 
 *
 * @param MySQLConnection $conn   a MySQL database connection
 *
 * @param Role $role   the parent role for the custom
 *                           attribute
 *
 * @return array  an array of arrays; each row is a detail records. Field names
 *   of these data records are equal to column names in the database
 *   table 'AttributeType'
 * @access private
 */
function query_custom_attributes($conn, $role) {  
   $query =   " SELECT AttributeType.* "
          .   " FROM AttributeType , Policy "
          .   " WHERE Policy.policy_attributetype = AttributeType.attributetype_id "
          .   "   AND Policy.policy_role = " . $role->getID()
          .   "   AND AttributeType.attributetype_standard = 0 "
          .   " ORDER BY AttributeType.attributetype_name ";
    
   $ret = array();
   $conn->query($query);
   $i=0;
   while($row = $conn->fetch_array()) {
      $ret[$i++] = $row;
   }
   $conn->freeresult();
   return $ret;
}

// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);

if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

// make sure 'rid' identifies an existing role
//
$rid= $_REQUEST["rid"];
$role =& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("Illegal role id ($rid) submitted");
}

// make sure the current administrator can edit this role
//
if (! $admin->canEditRole($role)) {
   ErrorHandling::fatalErrorAndExit("You do not have enough privileges to edit role ($rid)");  	
}	

$conn=& MySQLConnection::getConnection();
$edited_atid = -1;

// dispatch request
//
if (isset($_REQUEST['action']) ) {
    if ($_REQUEST["action"] == "do_create") {
      handle_do_create($conn, $role);	
    } 
    elseif ($_REQUEST["action"] == "do_update") {
      handle_do_update($conn, $role);	
    } 
    elseif ($_REQUEST["action"] == "do_delete") {
      handle_do_delete($conn, $role);	
    } 
    elseif ($_REQUEST["action"] == "do_edit") {
      $edited_atid = $_REQUEST["atid"];
    }
}

$attrs = query_custom_attributes($conn, $role);

/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>
<script language="JavaScript">

function is_empty(v) {
  //alert("is_empty: v=<<" + v + ">>");
  if (!v) return true;
  if (v.length == 0) return true;
  for (i=0; i < v.length; i++) {
    if (! (v[i] == ' ' || v[i] == '\t' || v[i] == '\r' || v[i] == '\n')) {
    	return false; 
    }
  }	
  return true;
}

function create_attribute() {
  f = document.forms["formattr"];
  if (is_empty(f.elements["attr-name-new"].value)) {
    alert("Attribute name must not be empty.");
    return;	
  }	
  f.action.value = "do_create";
  f.submit();
}

function update_attribute() {
  f = document.forms["formattr"];
  if (is_empty(f.elements["attr-name-edit"].value)) {
    alert("Attribute name must not be empty.");
    return;	
  }	
  f.action.value = "do_update";
  f.submit();
}

function select_all() {
  for(i=0; i < document.forms["formattr"].elements.length;i++) {
    var el = document.forms["formattr"].elements[i];
    if (el.name.substr(0,5) == "atid-") {
      el.checked =  true; 
    }
  }
}

function unselect_all() {
  for(i=0; i < document.forms["formattr"].elements.length;i++) {
    var el = document.forms["formattr"].elements[i];
    if (el.name.substr(0,5) == "atid-") {
      el.checked =  false;
    }
  }
}
function num_selected() {
  var ret = 0;
  for(i=0; i < document.forms["formattr"].elements.length;i++) {
    var el = document.forms["formattr"].elements[i];
    if (el.name.substr(0,5) == "atid-") {
      if (el.checked)ret++; 
    }
  }
  return ret;
}

function delete_selected() {
  if (num_selected() == 0) {
    alert("Please select at least one attribute first.")
    return;
  }
  if (confirm("Do you really want to delete these attributes?")) {
    f = document.forms["formattr"];
    f.action.value="do_delete";
    f.submit();
  }
}

function delete_attribute(atid) {
  if (! confirm("Do you really want to delete this attribute ?")) {
    return;	
  }	
  window.open(
    "<?= $_SERVER['PHP_SELF'] ?>?action=do_delete&atid-" + atid + "=1&rid=<?= $rid ?>",
    "_self"
  );
}


</script>

<!-- navigation menu -->
<p align="center">
  <span class="button-reference"><a href="edit-role.php?rid=<?= $rid ?>">Edit Role</a></span>
| <span class="button-reference"><a href="edit-role-managers.php?rid=<?= $rid ?>">Edit Role Administrators</a></span>
| <span class="button-reference"><a href="edit-role-aap.php?rid=<?= $rid ?>">Edit Standard Attributes</a></span>
</p>


<h1>Role Authorization Policy - Custom Attributes</h1>

  You can define a list of custom attributes a Role
  User <strong>must</strong> provide when he submits a 
  subscription request for this role.
  

  
  <form name="formattr" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="action" value="unknown">
  <input type="hidden" name="rid" value="<?= $rid ?>" />
  <input type="hidden" name="edited-atid" value="<?= $edited_atid  ?>" />
  <table width="90%" border="0" cellspacing="0" class="form-table">

    <tr>
      <th class="data" width="5">
      &nbsp;
      </th>
      <th class="data" width="40%">
      Name
      </th>
      <th class="data" width="40%">
      Description
      </th>
      <th class="data" width="20%">
      Action
      </th>
    </tr>
    
    <?php if (count($attrs) == 0):  ?>
    <tr>
      <td class="data-odd" colspan=4>
	There are no custom attributes defined yet for this role.
      </td>
    </tr>
    
    <?php else: ?>
      <?php $i= 0; ?>
      <?php foreach($attrs as $attr): ?>
         <?php 
          $style = ($i++ % 2 == 0) ? "data-even" : "data-odd";
         ?>
         <?php if ($edited_atid == $attr["attributetype_id"]): ?>
            <tr>
              <td class="<?= $style ?>" width="5">
                <input type="checkbox" name="atid-<?= $attr["attributetype_id"] ?>" />
              </td>
              <td class="<?= $style ?>" width="30%">
                <input type="text" name="attr-name-edit" value="<?= $attr["attributetype_name"] ?>" size=30 />
              </td>
              <td class="<?= $style ?>" width="30%">
                <input type="text" name="attr-description-edit" value="<?= $attr["attributetype_description"] ?>" size=30 />
              </td>
              <td class="<?= $style ?> last" width="30%">
                <input type="button" name="btnUpdate" value="Update" onclick="update_attribute();" />
              </td>
            </tr>
      
          <?php else: ?> 
            <tr>
              <td class="<?= $style ?>" width="5">
                <input type="checkbox" name="atid-<?= $attr["attributetype_id"] ?>" />
              </td>
              <td class="<?= $style ?>" width="30%">
                <?= $attr["attributetype_name"] ?>
              </td>
              <td class="<?= $style ?>" width="30%">
                <?= $attr["attributetype_description"] ?>
              </td>
              <td class="<?= $style ?> last" width="30%">
                <a href="<?= $PHP_SELF ?>?action=do_edit&atid=<?= $attr["attributetype_id"] ?>&rid=<?= $rid ?>">Edit</a> 
                | <a href="javascript:delete_attribute(<?= $attr["attributetype_id"] ?>);">Delete</a>
              </td>
            </tr>
          <?php endif; ?>
      <?php endforeach; ?>
  <?php endif; ?>
   
    
    
   <tr>
      <td class="data-even" width="5">
        &nbsp;
      </td>
      <td class="data-even" width="40%">
        <input type="text" name="attr-name-new" value=""  size=30 />
      </td>
      <td class="data-even" width="40%">
        <input type="description" name="attr-description-new" value="" size=30 />
      </td>
      <td class="data-even last" width="20%">
        <input type="button" name="btnSave" value="Create New" onClick="create_attribute();"/>
      </td>
    </tr>
		
    <tr>
      <td class="data-even last" colspan="4">
        <a href="javascript:select_all();">Select All</a> 
        | <a href="javascript:unselect_all();">Unselect All</a>
      </td>
    </tr>
    <tr>
      <td class="form-control" colspan="4">
        <input type=button name="btnDeleteAll" value="Delete Selected" onclick="delete_selected();"/>
      </td>
    </tr>   	
  </table>	
  </form>
  
  
  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
