<?php
	/*
	 * SNA-Portal mediates between one or more Authentication and Authorisation
	 * Infrastructures and non-AAI-enabled roles.
	 *
	 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
	 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch) 
	 *
	 * This program is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License
	 * as published by the Free Software Foundation; either version 2
	 * of the License, or (at your option) any later version.
	 *
	 * This program is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with this program; if not, write to the Free Software
	 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	 *
	 *
	 * $Id: edit-role.php,v 1.58 2007/10/18 08:47:24 vlry Exp $
	 */
	/* ****************************************************************** */
	
	
	/**
	 * This page edits a role.
	 *
	 * <b>Request Parameters</b>
	 * <ul>
	 *   <li>rid=<rid>: MANDATORY. The internal role ID. <rid> must be
	 *    a valid integer > 0 and the current administrator must be allowed
	 *    to edit the role with ID <rid>.
	 *    </li>
	 *   <li>do_save: OPTIONAL. If present updates the role with the
	 *      values provided in the HTTP request. If abstent, just displays
	 *      the input form initialized with the current values from the
	 *      database.
	 *   </li>
	 *   <li>role_title=<value>: OPTIONAL. the role title 
	 *   </li>
	 *
	 *   <li>role_description=<value>: OPTIONAL. the role description
	 *   </li>
	 *   <li>role_url=<value>: OPTIONAL. the role URL
	 *   </li>
	 *   <li>role_is_visible=<value>: OPTIONAL. flag, indicating whether the role
	 *      is visible or not. Must be "0" or "1".
	 *   </li>
	 *   <li>role_state=<value>: OPTIONAL. int value with the role
	 *      access state. 
	 *   </li>
	 *
	 *  </ul>
	 *
	 * 
	 * @version $Revision: 1.58 $
	 * @package AdminPortal
	 */
	
	/* ***************************************************************** */
	/*                        functions                                  */
	/* ***************************************************************** */
	
	/*
	 * Validates the input parameters 
	 *
	 * @param MySQLConnction  $conn     a MySQL connection
	 * @param array &$input_errors  an output parameter with the error messages. The 
	 *  following error messages are replies:
	 *  <ul>
	 *    <li>$input_errors["role_title"]: errors related to the role title
	 *        (title missing)
	 *    </li>
	 *    <li>$input_errors["role_url"]: errors related to the role URL
	 *        (URL missing)
	 *    </li>
	 *  </ul>
	 * 
	 * @return boolean  true, if the input is valid; false, otherwise.
	 *   The out-parameter $input_errors contains specific error messages
	 *   for each parameter.
	 * 
	 * @access private
	 */
	function validate_input( &$input_errors,&$role) {
		$valid= true;
		$title= $_REQUEST["role_title"];
		$title= trim($title);
		if ( empty($title) ) {
			$valid = false;
			$input_errors["role_title"] = "Please enter a non-empty Title.";    
		}
		
		$url= $_REQUEST["role_url"];    
		$url= trim($url);
		if ( empty($url) ) {
			$valid= false;
			$input_errors["role_url"] = "Please enter a non-empty URL.";    
		}
		
		$adapter= $_REQUEST["role_adapter"];
		$adapter= trim($adapter);
		if ( empty($adapter) ) {
			$valid= false;
			$input_errors["role_adapter"]= "Please choose a Role type.";    
		}
		
		$rkey= $_REQUEST['role_key'];
		$rkey= trim($rkey);
		if (!empty($rkey)) {
			$rkey_length= strlen($rkey);
			if ($rkey_length < 6 or $rkey_length > 40) {
				$valid= false;
				$input_errors['role_key']= "The Role Key size must be between 6 and 40 characters.";
			}
			elseif ( $role->getRoleKey() != $rkey and Role::existsRoleKey($rkey)) {
				$valid= false;
				$input_errors['role_key']= "This Role Key already exists, please choose another one.";
			}
		}  
		return $valid;
	}
	
	
	/**
	 * Updates a role with current values in the
	 *    $_REQUEST object 
	 *
	 * @param Role &$res  the role object to be updated
	 *
	 * @access private
	 */
	function updateRole(&$res) {
		
		$res->setTitle(trim($_REQUEST["role_title"]));
		$res->setUrl(trim($_REQUEST["role_url"]));
		$res->setDescription(trim($_REQUEST["role_description"]));
		$res->setRoleAdapter(trim($_REQUEST["role_adapter"]));
		$res->setVisible((boolean)trim($_REQUEST["role_is_visible"]));
		$res->setRoleAccessState((int)trim($_REQUEST["role_state"]));
		// === ANWANDER ===
		$all_actions= Role::getAllActions();
		$i = 0;
		foreach($all_actions as $all_actions):
		$action_id= $all_actions['action_id'];
		$res->setRoleActionState($action_id,(int)trim($_REQUEST["action_id_$action_id"]));
		endforeach;
		// ================
		
		if (isset($_REQUEST['role_key'])) {
			$rkey= $_REQUEST['role_key'];
			$rkey= trim($rkey);
			if (!empty($rkey) and $rkey != $res->getRoleKey()) {
				$res->setRoleKey($rkey);
			}
		}
		
		$res->persist();
	}
	
	function changeRoleAdapter(&$res) {
		if (!empty($_REQUEST["role_adapter"])) {
			$role_adapter= $_REQUEST["role_adapter"];
			$res->setRoleAdapter($role_adapter);
		}
	}
	
	/**
	 * Updates the adapter parameters   
	 *
	 * @param Role  &$res  the role object whose role adapter
	 *    parameters are updated.  
	 *
	 * @access private
	 */
	function updateRoleAdapterParameters(&$res) {
		$res->deleteAdapterParameters();
		foreach($_REQUEST as $name => $value) {
			if (ereg("^ap-(.*)$", $name, $reg)) {
				$value= trim( $value );
				if (!empty($value)) {
					$res->insertAdapterParameter($reg[1], $value);
				}
			}
		}
	}
	
	
	/* **************************************************************** */
	/**
	 * Ensures that request parameters are valid. If the request is invalid
	 * an error message is displayed and the page aborts with a fatal
	 * error.
	 *
	 * @access private
	 */
	function ensure_request_parameters_valid() {
		if (! isset($_REQUEST["rid"])) {
			ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing"); 
		}
		
		$rid = $_REQUEST["rid"];
		if (! is_numeric($rid) || $rid < 0) {
			ErrorHandling::fatalErrorAndExit("illegal value '$rid' for request parameter 'rid'. Aborting.");    
		}
		if (isset($_REQUEST["role_is_visible"])) {
			$v = $_REQUEST["role_is_visible"];
			if ($v != "0" && $v != "1") {
				ErrorHandling::fatalErrorAndExit("illegal value '$v' for request parameter 'role_is_visible'. Aborting.");  
			}
		}
		if (isset($_REQUEST["role_state"])) {
			$v = $_REQUEST["role_state"];
			if ($v < 0 or $v > 4) {
				ErrorHandling::fatalErrorAndExit("illegal value '$v' for request parameter 'role_state'. Aborting.");   
			}
		}
		// ANWANDER -> role_state
		$all_actions= Role::getAllActions();
		$i = 0;
		foreach($all_actions as $all_actions):
		$action_id= $all_actions['action_id'];
		if (isset($_REQUEST["action_id_$action_id"])){
			$v = $_REQUEST["action_id_$action_id"];
			if ($v < 0 or $v > 1) {
				ErrorHandling::fatalErrorAndExit("illegal value '$v' for request parameter 'action_id_$action_id'. Aborting.");
			}
		}	
		endforeach;
	}
	
	// bootstrap application
	//
	require_once("include/application.inc");
	
	ensure_request_parameters_valid();
	
	//
	// resume the current session and make sure we have a valid session
	//
	Session::ensureAdminSessionValid();
	
	$uid= Session::getUID();
	$admin=& Administrator::reconstituteByUid($uid);
	if (is_null($admin)) {
		ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
	}
	Session::init($admin);
	
	$rid= $_REQUEST["rid"];
	$role =& Role::reconstituteById($rid);
	if (is_null($role)) {
		ErrorHandling::fatalErrorAndExit("Illegal role id ($rid) submitted");
	}
	
	if (! $admin->canEditRole($role)) {
		ErrorHandling::fatalErrorAndExit("You do not have enough privileges to edit role ($rid)");   
	}   
	
	//
	// process save operation
	//
	$input_errors = array();
	if (isset($_REQUEST['do_save'])) {
		if (validate_input(&$input_errors,&$role)) {     
			updateRole(&$role); 
			updateRoleAdapterParameters(&$role);
		}
	}
	// process change of adapter (without loss)
	elseif (isset($_REQUEST['action']) and $_REQUEST['action'] == 'change_adapter' ) {
		if (validate_input(&$input_errors,&$role)) {     
			changeRoleAdapter(&$role);
		}
	}
	
	
	/* ****************************************************************** */
	/*                     page content                                   */
	/* ****************************************************************** */
	include("include/adminHeader.inc");
	
	?>
<script>
<!--
function delete_role(rid) {
	if (! confirm("Do you really want to delete this role ?")) return;
	window.open('delete-role.php?rid=' + rid + '&url=', '_self');
}

function reloadForm() {
	document.editrole.action.value= "change_adapter";
	document.editrole.submit();
}
// -->
</script>

<!-- navigation menu -->
<p align="center">
<span class="button-reference"><a href="edit-role-managers.php?rid=<?= $rid ?>">Edit Role Administrators</a></span>
| <span class="button-reference"><a href="edit-role-aap.php?rid=<?= $rid ?>">Edit Standard Attributes</a></span>
| <span class="button-reference"><a href="edit-custom-role-aap.php?rid=<?= $rid ?>">Edit Custom Attributes</a></span>
| <span class="button-reference"><a href="javascript:delete_role(<?= $rid ?>)">Delete</a></span> 
</p>


<h1>Edit Role</h1>

Fields marked with
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
must be filled in.
<p/>
Your role is not visible to others unless you select <i>Visible</i>
in the field <b>Role Visibility</b>.<br/>
Users cannot subscribe to your role unless you select <i>Free</i>,  
<i>Managed</i>, <i>Passwords</i> or <i>Suspended</i> in the field 
<b>Role Subscription State</b>. 
<p/>

<form name="editrole" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">

<input type="hidden" name="rid" value="<?= $rid ?>" />
<input type="hidden" name="action" value="" />

<table width="90%" border="0" cellspacing="0" class="form-table">

<!-- ************* DATE ************************** -->

<tr>
<td width="30%" valign="top" class="form-label form-first">
Date Created
</td>
<td width="70%" valign="top" class="form-entry form-first">
<?= Formatting::formatMysqlDate($role->getDateCreated(),true) ?>
</td>
</tr>
<!-- ************* Role Key ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Key<br>(Password)
</td>
<td width="70%" valign="top" class="form-entry">
<input type="text" name="role_key" value="<?= $role->getRoleKey() ?>" size="40" maxlength="40"/>
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
<br>You can define a unique role key and use it as password if the subscription state is set to Password.
<br>The length of the key must be between 6 and 40 characters. 
<?php if ( isset($input_errors["role_key"]) ): ?>
<br/><span class="error-message"><?= $input_errors["role_key"] ?></span>
<?php endif; ?>
</td>
</tr>
<!-- ************* Role Title    ************************** -->

<tr>
<td width="30%" valign="top" class="form-label">
Role Title
</td>
<td width="70%" valign="top" class="form-entry">
<input type="text" name="role_title" value="<?= $role->getTitle() ?>" size="40"/>
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
<?php if ( isset($input_errors["role_title"]) ): ?>
<br/><span class="error-message"><?= $input_errors["role_title"] ?></span>
<?php endif; ?>
</td>
</tr>
<!-- ************* Role URL      ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role URL
</td>
<td width="70%" valign="top" class="form-entry">
<input type="text" name="role_url" value="<?= $role->getUrl() ?>" size="40" />
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
<?php if ( isset( $input_errors["role_url"] ) ): ?>
<br/><span class="error-message"><?= $input_errors["role_url"] ?></span>
<?php endif; ?>

</td>
</tr>
<!-- ************* Role Visibility    ********************* -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Description
</td>
<td width="70%" valign="top" class="form-entry">
<textarea name="role_description" cols="40" rows="5"><?= $role->getDescription() ?></textarea>
</td>
</tr>
<!-- ************* Role Visibility    ********************* -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Visibility
</td>
<td width="70%" valign="top" class="form-entry">
<?php $is_visible = $role->isVisible(); ?>
<input type="radio" name="role_is_visible" value="1" <?= $is_visible ? "CHECKED" : "" ?> /> Visible
<input type="radio" name="role_is_visible" value="0" <?= $is_visible ? "" : "CHECKED"?> /> Invisible
</td>
</tr>
<!-- ************* Role Access State   ********************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Subscription State
</td>
<td width="70%" valign="top" class="form-entry">
<?php $state = $role->getRoleAccessState();  ?>
<input type="radio" name="role_state" value="3" <?= $state == 3 ? "CHECKED" : "" ?> /> Free
<input type="radio" name="role_state" value="2" <?= $state == 2 ? "CHECKED" : "" ?> /> Managed
<input type="radio" name="role_state" value="4" <?= $state == 4 ? "CHECKED" : "" ?> /> Password
<input type="radio" name="role_state" value="1" <?= $state == 1 ? "CHECKED" : "" ?> /> Suspended
<input type="radio" name="role_state" value="0" <?= $state == 0 ? "CHECKED" : "" ?> /> Closed           
</td>
</tr>

<!-- ************* Action  ************************** -->

<tr>
<td width="30%" valign="top" class="form-label">
Allowed Actions 
</td>
<td width="100%" class="form-entry">


<table>
<tr>
<th class="data last">State</th>
<th class="data last">Action</th>
<th class="data last">Title</th>
</tr>
<?
//   ANWANDER
$all_actions= Role::getAllActions();
$selected_actions= Role::getAllSelectetActionsForThisRessoure($role->getID());
$i = 0;
foreach($all_actions as $all_actions):
$selected_actions_tmp= $selected_actions;
$is_selected = 0;
$action_name= $all_actions['action_name'];
$action_title= $all_actions['action_title'];
$action_id= $all_actions['action_id'];
foreach($selected_actions_tmp as $selected_actions_tmp){
	$action_id_selected = $selected_actions_tmp['action_id'];
	if($action_id_selected == $action_id){
		$is_selected = 1;
	}
}
$style= ($i++ % 2 == 0) ? "data-even" : "data-odd";
?>

<tr>
<td valign="top" class="<?= $style ?>">
<input type="checkbox" name="action_id_<?= $action_id  ?>" value="1" <?= $is_selected ? "CHECKED" : "" ?>/>
</td>
<td valign="top" class="<?= $style ?>">
<?= $action_name ?> 
</td>
<td valign="top" class="<?= $style ?>">
<?= $action_title ?>
</td>
</tr>

<?php endforeach; ?>

</table>

</td>	 
</tr>

<!-- ************* Role Adapter  ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Adapter
</td>
<td width="70%" valign="top" class="form-entry">
<select name="role_adapter" onChange="reloadForm()">
<?php
	$roleAdapters= Adapting::listRoleAdaptors();
	?>
<?php foreach ($roleAdapters as $adapterConfig): ?>
<?php $selected = ($adapterConfig['name'] == $role->getRoleAdapter()) ? " selected " : ""; ?>
<option value="<?= $adapterConfig['name'] ?>" <?= $selected ?> ><?= $adapterConfig['display-name'] ?></option>
<?php endforeach; ?>
</select> <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">

<?php if (isset($input_errors["role_adapter"])): ?>
<br/><span class="error-message"><?= $input_errors["role_adapter"] ?></span>
<?php endif; ?>
<?php 
	$adapter_name= $role->getRoleAdapter();
	$help= Adapting::getAdapterHelp($adapter_name);
	$parameters=& Adapting::getAdapterParameterDescriptors($adapter_name);
	
	if ($help) {
        echo "<p>$help</p>";
	}
	
	// adapter parameters form
	if (is_null($parameters)) {
		// no parameters
		$parameters= array();
	}
	else {
        echo "<p style='font-weight: bold;'>This Role requires you to provide additional parameters. ";
        echo "Please fill in the following fields.<p/>";
		
        $paramValues =& $role->getAdapterParameters();
		
	}
	?>  
<table width="100%">
<?php foreach($parameters as $pardesc_id => $pardesc): ?>
<tr>
<td valign="top" class="form-entry form-first" style="font-weight: bold;">
<?= $pardesc["display-name"] ?>
</td>
<?php 
	$param_value= '';
	if (isset($paramValues[$pardesc_id])) {
		$param_value= $paramValues[$pardesc_id];                
	}
	
	$input_type= "text";
	$checked= '';
	if( eregi('SECRET',$pardesc_id) or eregi('PASSWORD',$pardesc_id) ) { 
		$input_type= "password"; 
	}
	elseif ( eregi('ENABLE', $pardesc_id) ) {
		$input_type= "checkbox";
		if (!empty($param_value)) {
			$checked= 'checked';
		}
		$param_value="1";
	}
	
	
	?>
<td valign="top" class="form-entry form-first">
<input type="<?= $input_type ?>" <?= $checked ?> name="ap-<?= $pardesc_id ?>" size=40 value="<?= $param_value ?>">
<br/><?= $pardesc["description"]?>
</td>
</tr>
<?php endforeach; ?>
</table> 

</td>
</tr>



<!-- ************* Control Buttons ************************** -->    

<tr>
<td width="100%" valign="top" align="center" class="form-control" colspan="2">
<input type="submit" name="do_save" value="Save" />
</td>    
</tr>

</table>
</form>



<?php
	// include common page footer
	//
	include("include/adminFooter.inc");
	?>
