<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: edit-user.php,v 1.35 2006/09/29 10:51:29 vlry Exp $
 */

/**
* This page provides a form for editing user information.
*
* <b>Request Parameters</b>
* <ul>
*   <li>uid=<uid>: MANDATORY. The internal userID. <uid> must be
*    a valid integer > 0 and the current administrator must be allowed
*    to edit the user <uid>.
*    </li>
*   <li>do_save=On: OPTIONAL. If present saves the data submitted in 
*      the HTTP request. If abstent, just loads the data from the
*      database and displays an input form.
*   </li>
*
*  </ul>
*
* 
* @version $Revision: 1.35 $
* @package AdminPortal
*/

/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */

/**
 * Ensures that all expected
 *   request parameters are actually passed in and that they have
 *   the acceptable values
 *
 *  Aborts with exit() if an error is detected. 
 *
 * @access private
 */ 
function ensure_request_parameters_valid() {

  // make sure mandatory uid is set
  //
  if (! isset($_REQUEST["uid"])) {
     ErrorHandling::fatalErrorAndExit("request parameter 'uid' is missing");
  }

  // make sure the uid is numeric and positive
  //
  if (! is_numeric($_REQUEST["uid"]) || $_REQUEST["uid"] < 0) {
     ErrorHandling::fatalErrorAndExit("illegal value '". $_REQUEST["uid"] . "' for request parameter 'uid'");
  }   
}


/**
 * Updates the user attributes of user
 *   $user with the values posted in $_REQUEST
 *
 * @param User  $user  the user whose attributes to update
 *
 * 
 * @access private
 */
function update_user_attributes($user) {
  foreach ($_REQUEST as $key=>$value) {
      $value= trim($value);
      if (substr($key, 0, 10) == "attribute-") {

        // $attribute_id needs to be an integer (check User::storeAttribute interface)
        $attributeType_id = (int) substr($key, 10, strlen($key) - 10);

        if (!empty($value)) {
          if (!$user->isAttributeTrusted($attributeType_id)) {
                $user->storeAttribute($attributeType_id, $value);
          }
        } else {
          $user->deleteAttribute($attributeType_id);
        }
      }
  }
}

/* **************************************************************** */

//
// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

//  load the user object
//
$user_id= $_REQUEST["uid"];
$user =& User::reconstituteById($user_id);
if (is_null($user)) {
  ErrorHandling::fatalErrorAndExit("No user with ID ($user_id) found");
}


// check wheter the current user is allowed to see details about
// this user
//
if (! $admin->canEditUser($user)) {
  ErrorHandling::fatalErrorAndExit("You do not have enough privileges to edit details of this user");
}

if (isset($_REQUEST["action"])) {
    $action= $_REQUEST["action"];
    if ($action == "do_update") {
      update_user_attributes($user);
      if (isset($_REQUEST['user_privilege']) and $admin->canEditUserPrivileges() ) {
            $user->setPrivilege($_REQUEST["user_privilege"]);
      }
    } 
    elseif ($action == "do_delete" and $admin->canDeleteUser()) {
      $user->delete($admin->getID());
      header("Location: " . PORTAL_BASE_URL . "/admin/list-all-users.php");
      exit;
    }
}


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>
<script>
function delete_user(uid) {
    if (! confirm("Do you really want to delete this user ?"))  return;     
    document.form_user.action.value= "do_delete";
    document.form_user.submit();
}

</script>

<!-- Action Menu -->
<p align="center">
  <span class="button-reference"><a href="list-user-subscriptions.php?uid=<?= $user_id ?>" title="List User Subscriptions">List Subscriptions</a></span>
  | <span class="button-reference"><a href="javascript:delete_user(<?= $user_id ?>)">Delete User</a></span>
  | <span class="button-reference"><a href="change-password.php?uid=<?= $user_id ?>">Change Local Password</a></span>
</p>  
  
<h1>Edit User</h1>


  <!-- ******************* Required Attributes ******************* -->
     
  <form name="form_user" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
    <input type="hidden" name="action" value="do_update" />
    <input type="hidden" name="uid" value="<?= $user_id  ?>" />

  <table width="90%" border="0" cellspacing="0" class="form-table">

   <tr>
      <td width="30%%" valign="top" class="form-label form-first">
      Last Login
      </td>
      <td width="70%%" valign="top" class="form-entry form-first">
       <?= Formatting::formatMysqlDate($user->getLastLogin(), true) ?>
      </td>
    </tr>

   <tr>
      <td width="30%%" valign="top" class="form-label form-first">
      User ID
      </td>
      <td width="70%%" valign="top" class="form-entry form-first">
       <?= $user->getUid() ?>
      </td>
    </tr>
 
   <tr>
      <td width="30%%" valign="top" class="form-label">
      User Role
      </td>
      <td width="70%%" valign="top" class="form-entry">
      <?php 
         if ($admin->canEditUserPrivileges()) {         
          $privileges = UserPrivilege::getPrivileges();
           foreach($privileges as $privilege)  {
             $name = $privilege["userprivilege_description"];
             $user_privilege= $privilege['userprivilege_id'];
             if ($user->getPrivilege() == $user_privilege) {
                $checked = "checked"; 
             }
             else {
                $checked = "";
             }
        ?>
             <input type="radio" name="user_privilege" value="<?= $user_privilege ?>" <?= $checked ?> /><?= $name ?><br>
        <?php
           } // end for 
         } else {
           echo $user->getPrivilegeName();
         }
      ?>
      </td>
    </tr>

  <?php
    // get attribute names
    $attr_names= AttributeType::getAttributeTypeNames();
    // and user attributes
    $attrs= $user->getAttributes();
    foreach( $attr_names as $atid => $name):
        if (!empty($attrs[$atid])) {
            $value= $attrs[$atid]['attribute_value'];
            $trusted= $attrs[$atid]['attribute_trusted'];
        }
        else {
            $value= null;
            $trusted= false;
        } 
        ?>
        
         <tr>
            <td width="30%" valign="top" class="form-label">
                <?= $name ?>
            </td>
            <td width="70%" valign="top" class="form-entry">
            <?php if ($trusted): ?>
                 <?= Formatting::formatTableCellContent($value) ?>
            <?php else: ?>
                 <input type="text" name="attribute-<?= $atid ?>" value="<?= $value ?>" >
            <?php endif; ?>
            </td>
         </tr>
   <?php endforeach; ?>

  <tr>
    <td valign="top" align="center" class="form-control" colspan="2">
      <input type="submit" value="Save" class="button">
    </td>    
  </tr>    

  </table>
  </form>
  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
