<?php 
	/*
	 * SNA-Portal mediates between one or more Authentication and Authorisation
	 * Infrastructures and non-AAI-enabled roles.
	 *
	 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
	 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch) 
	 *
	 * This program is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License
	 * as published by the Free Software Foundation; either version 2
	 * of the License, or (at your option) any later version.
	 *
	 * This program is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with this program; if not, write to the Free Software
	 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	 *
	 * $Id: new-role.php,v 1.53 2007/03/06 17:20:24 vlry Exp $
	 */
	
	/**
	 * This page provides an input form for creating a new role. 
	 *
	 * <b>Request Parameters</b>
	 * <ul>
	 *   <li>do_save=on: OPTIONAL. If present, creates a new role
	 *     with the parameters supplied in the request parameters 
	 *    </li>
	 *   <li>role_title=<value>: OPTIONAL. the role title 
	 *   </li>
	 *
	 *   <li>role_description=<value>: OPTIONAL. the role description
	 *   </li>
	 *   <li>role_url=<value>: OPTIONAL. the role URL
	 *   </li>
	 *   <li>role_is_visible=<value>: OPTIONAL. flag, indicating whether the role
	 *      is visible or not. Must be "0" or "1".
	 *   </li>
	 *   <li>role_state=<value>: OPTIONAL. the role access state 
	 *   </li>
	 *  
	 *  </ul>
	 *
	 * @version $Revision: 1.53 $
	 * @package AdminPortal
	 */
	
	
	$STANDARD_REQUIRED_ATTRIBUTES = array(
										  "surname",
										  "givenName",
										  "mail",
										  "Shib-WISEBEDPerson-homeOrganization",
										  "Shib-WISEBEDPerson-homeOrganizationType",
										  "eduPersonAffiliation"
	);
	
	
	/* ***************************************************************** */
	/*                        functions                                  */
	/* ***************************************************************** */
	
	
	/**
	 * Validates the input parameters 
	 *
	 * @param array  &$input_errors  an output parameter with the
	 *                         error messages 
	 * 
	 * @return boolean  true, if the input is valid; false, otherwise
	 *   the OUT parameter $input_errors contains specific error messages
	 *   for each parameter 
	 * 
	 * @access private
	 */
	function validate_input(&$input_errors) {
		$valid= true;
		$title= $_REQUEST["role_title"];
		$title= trim($title);
		if ( empty($title) ) {
			$valid = false;
			$input_errors["role_title"] = "Please enter a non-empty Title.";    
		}
		
		$url= $_REQUEST["role_url"];    
		$url= trim($url);
		if ( empty($url) ) {
			$valid= false;
			$input_errors["role_url"] = "Please enter a non-empty URL.";    
		}
		
		$adapter= $_REQUEST["role_adapter"];
		$adapter= trim($adapter);
		if ( empty($adapter) ) {
			$valid= false;
			$input_errors["role_adapter"]= "Please choose a Role Adapter type.";    
		}
		
		if (isset($_REQUEST['role_key'])) {
			$rkey= $_REQUEST['role_key'];
			$rkey= trim($rkey);
			if (!empty($rkey)) {
				$rkey_length= strlen($rkey);
				if ($rkey_length < 6 or $rkey_length > 40) {
					$valid= false;
					$input_errors['role_key']= "The Role Key size must be between 6 and 40 characters.";
				}
				elseif (Role::existsRoleKey($rkey)) {
					$valid= false;
					$input_errors['role_key']= "This Role Key already exists, please choose another one.";
				}
			}
		}
		
		return $valid;
	}
	
	/**
	 * Creates a new role from the values provided
	 *   in the $_REQUEST object 
	 *
	 * @param Administrator $admin       the current administrator user (an instance
	 *                         of class User)
	 *
	 * @return  Role  an initialized role already written to the database
	 * 
	 * @access private
	 */
	function createRole(&$admin) {  
		
		$provider_id= $admin->getID();
		$url= $_REQUEST["role_url"];
		$url= trim($url);
		
		$title= $_REQUEST["role_title"];
		$title= trim($title);
		
		$res=& Role::create($provider_id, $url, $title );
		
		$res->setDescription(trim($_REQUEST["role_description"]));
		$res->setRoleAdapter(trim($_REQUEST["role_adapter"]));
		$res->setVisible((boolean)trim($_REQUEST["role_is_visible"]));
		$res->setRoleAccessState((int)trim($_REQUEST["role_state"]));
		// === ANWANDER ===
		$all_actions= Role::getAllActions();
		$i = 0;
		foreach($all_actions as $all_actions):
		$action_id= $all_actions['action_id'];
		$res->setRoleActionState($action_id,(int)trim($_REQUEST["action_id_$action_id"]));
		endforeach;
		// ================
		if (isset($_REQUEST['role_key'])) {
			$rkey= $_REQUEST['role_key'];
			$rkey= trim($rkey);
			if (!empty($rkey)) {
				$res->setRoleKey($rkey);
			}
		}
		
		$res->persist();
		return $res;
	}
	
	/**
	 * Persists the role adapter parameters for the current role.
	 * Values to be persisted in the database are read from the global
	 * $_REQUEST object.
	 *
	 * @param Role &$res the role object currently being edited
	 *
	 * @access private
	 */ 
	function persistRoleAdapterParameters(&$res) {
		$res->deleteAdapterParameters();
		foreach($_REQUEST as $name => $value) {
			if (ereg("^ap-(.*)$", $name, $reg)) {
				$value= trim( $value );
				if (!empty($value)) {
					$res->insertAdapterParameter($reg[1], $value);
				}
			}
		}
	}
	
	
	/**
	 * Displays the form fields for entering role adapter parameter
	 * values.
	 *
	 * @param string $adapterName  the name of the adapter 
	 *
	 * @access private
	 */ 
	function displayAdapterParameterForm($adapterName) {
		$parameters =& Adapting::getAdapterParameterDescriptors($adapterName);
		if (! $parameters || count($parameters) == 0) {
			return;
		}
	?>
<p/>
This role adapter requires you to provide additional parameters specific
for this role. Please fill in the following fields.
<p/>
<table width="100%">
<?php foreach($parameters as $pardesc_id => $pardesc): ?>
<tr>
<td valign="top" class="form-entry form-first" style="font-weight: bold;"><?= $pardesc["display-name"]?></td>
<td valign="top" class="form-entry form-first"><input name="ap-<?= $pardesc_id ?>" size=40 >
<br/><?= $pardesc["description"]?></td>
</tr>
<?php endforeach; ?>
</table>
<?php
	}
	
	// bootstrap application
	//
	require_once("include/application.inc");
	
	//
	// resume the current session and make sure we have a valid session
	//
	Session::ensureAdminSessionValid();
	
	$uid= Session::getUID();
	$admin=& Administrator::reconstituteByUid($uid);
	if (is_null($admin) ) {
		ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
	}
	
	if ( ! $admin->canCreateRole() ) {
		ErrorHandling::fatalErrorAndExit("You have not enough privilege to create role."); 
	}
	
	
	Session::init($admin);
	
	
	// process save operation 
	//
	$input_errors= array(); 
	if ( isset($_REQUEST['do_save']) ) {  
		if (validate_input(&$input_errors)) {
			$role =& createRole(&$admin);     
			foreach($STANDARD_REQUIRED_ATTRIBUTES as $ldapName) {
				$role->addToPolicy($ldapName); 
			}
			if (isset($_REQUEST['role_adapter'])) {
				persistRoleAdapterParameters(&$role);
			}
			header("Location: " . PORTAL_BASE_URL . "/admin/edit-role.php?rid=" . $role->getID());
			exit;
		}
	}
	
	// process REQUEST parameters
	$req_role_title= "";
	if (isset($_REQUEST['role_title'])) {
		$req_role_title= trim($_REQUEST['role_title']);
	}
	$req_role_url= "";
	if (isset($_REQUEST['role_url'])) {
		$req_role_url= trim($_REQUEST['role_url']);
	}
	$req_role_description= "";
	if (isset($_REQUEST['role_description'])) {
		$req_role_description= trim($_REQUEST['role_description']);
	}
	$req_role_is_visible= 0; // default: invisible
	if (isset($_REQUEST['role_is_visible'])) {
		$req_role_is_visible= trim($_REQUEST['role_is_visible']);
	}
	$req_role_state= 2; // default: open
	if (isset($_REQUEST['role_state'])) {
		$req_role_state= trim($_REQUEST['role_state']);
	}
	$req_role_key= '';
	if (isset($_REQUEST['role_key'])) {
		$req_role_key= trim($_REQUEST['role_key']);
	}
	
	// === ANWANDER ==================
	$checkedActions = array();
	$all_actions= Role::getAllActions();
	foreach($all_actions as $all_actions){
		$action_id= $all_actions['action_id'];
		$checkedActions[$action_id] = isset($_REQUEST["action_id_$action_id"]);
	}
	// ===============================

	
	/* ****************************************************************** */
	/*                     page content                                   */
	/* ****************************************************************** */
	include("include/adminHeader.inc");
	?>

<?php
	/* ****************************************************************** */
	/*                     client side java script                        */
	/* ****************************************************************** */
	?>

<script>
function reloadForm() {
    document.newRole.submit();
}

</script>

<h1>New Role</h1>

To create a new role enter the form below and click on
<span class="button-reference">Save</span>. Fields marked with
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
must be filled in.
<p/>
Your role is not visible to others unless you select <i>Visible</i>
in the field <b>Role Visibility</b>.<br/>
Users cannot subscribe to your role unless you select <i>Free</i>, 
<i>Managed</i>, <i>Passwords</i> or <i>Suspended</i> in the field 
<b>Role Subscription State</b>. 
<p/>

<form name="newRole" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">

<table width="90%" border="0" cellspacing="0" class="form-table">

<!-- ************* Role Key ************************** -->
<tr>
<td width="30%" valign="top" class="form-label form-first">
Role Key<br>(Password)
</td>
<td width="70%" valign="top" class="form-entry form-first">
<input type="text" name="role_key" value="<?= $req_role_key ?>" size="40" maxlength="40"/>
<?php if (isset($input_errors["role_key"])): ?>
<br/><span class="error-message"><?= $input_errors["role_key"] ?></span>
<?php endif; ?> 
<br>You can define a unique role key and use it as password if the subscription state is set to <i>Password</i>. The length of the key must be between 6 and 40 characters.
<br>If not defined, the system will generate a random key.
</td>
</tr>

<!-- ************* Role Title    ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Title
</td>
<td width="70%" valign="top" class="form-entry">
<input type="text" name="role_title" value="<?= $req_role_title ?>" size="40"/>
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">

<?php if (isset($input_errors["role_title"])): ?>
<br/><span class="error-message"><?= $input_errors["role_title"] ?></span>
<?php endif; ?>

</td>
</tr>

<!-- ************* Role URL      ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role URL
</td>
<td width="70%" valign="top" class="form-entry">
<input type="text" name="role_url" value="<?= $req_role_url ?>" size="40" />
<img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
<?php  if (isset($input_errors["role_url"])):  ?>
<br/><span class="error-message"><?= $input_errors["role_url"] ?></span>
<?php  endif; ?>

</td>
</tr>

<!-- ************* Role Description    ********************* -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Description
</td>
<td width="70%" valign="top" class="form-entry">
<textarea name="role_description" cols="40" rows="5"><?= $req_role_description ?></textarea>
</td>
</tr>

<!-- ************* Role Visibility    ********************* -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Visibility
</td>
<td width="70%" valign="top" class="form-entry">
<input type="radio" name="role_is_visible" value="1" <?= $req_role_is_visible ? "CHECKED" : "" ?> /> Visible
<input type="radio" name="role_is_visible" value="0" <?= $req_role_is_visible ? "" : "CHECKED" ?> /> Invisible      
</td>
</tr>

<!-- ************* Role Access State   ********************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Subscription State 
</td>
<td width="70%" valign="top" class="form-entry">
<input type="radio" name="role_state" value="3" <?= $req_role_state == 3 ? "CHECKED" : "" ?> /> Free
<input type="radio" name="role_state" value="2" <?= $req_role_state == 2 ? "CHECKED" : "" ?> /> Managed
<input type="radio" name="role_state" value="4" <?= $req_role_state == 4 ? "CHECKED" : "" ?> /> Password
<input type="radio" name="role_state" value="1" <?= $req_role_state == 1 ? "CHECKED" : "" ?> /> Suspended  
<input type="radio" name="role_state" value="0" <?= $req_role_state == 0 ? "CHECKED" : "" ?> /> Closed
</td>
</tr>

<!-- ************* Action  ************************** -->

<tr>
<td width="30%" valign="top" class="form-label">
Allowed Actions 
</td>
<td width="100%" class="form-entry">


<table>
<tr>
<th class="data last">State</th>
<th class="data last">Action</th>
<th class="data last">Title</th>
</tr>
<?

//   ANWANDER
$all_actions= Role::getAllActions();
foreach($all_actions as $all_actions):
  $action_name= $all_actions['action_name'];
  $action_title= $all_actions['action_title'];
  $action_id= $all_actions['action_id'];
  $style= ($i++ % 2 == 0) ? "data-even" : "data-odd";
  $is_selected = $checkedActions[$action_id];
?>

<tr>
  <td valign="top" class="<?= $style ?>">
    <input type="checkbox" name="action_id_<?= $action_id  ?>" value="1" <?= $is_selected ? "CHECKED" : "" ?>/>
  </td>
  <td valign="top" class="<?= $style ?>">
    <?= $action_name ?> 
  </td>
  <td valign="top" class="<?= $style ?>">
    <?= $action_title ?>
  </td>
</tr>

<?php endforeach; ?>

</table>

</td>	 
</tr>

<!-- ************* Role Adapter  ************************** -->
<tr>
<td width="30%" valign="top" class="form-label">
Role Adapter 
</td>
<td width="70%" valign="top" class="form-entry">
<select name="role_adapter" onChange="reloadForm()">
<option value="" selected>(None)</option>
<?php 
	$roleAdapterList = Adapting::listRoleAdaptors();
	$selected= ""; 
	foreach ($roleAdapterList as $adapter):
	if (isset($_REQUEST['role_adapter'])) {
		$selected = ($adapter["name"] == $_REQUEST['role_adapter']) ? " selected " : "";
	} 
	?>
<option value="<?= $adapter["name"] ?>" <?= $selected ?> ><?= $adapter["display-name"] ?></option>
<?php  endforeach; ?>
</select> <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">

<?php if (isset($input_errors["role_adapter"])): ?>
<br/><span class="error-message"><?= $input_errors["role_adapter"] ?></span>
<?php endif; ?>

<?php if (isset($_REQUEST['role_adapter'])):
	$adapter_name= $_REQUEST['role_adapter'];
	$help= Adapting::getAdapterHelp($adapter_name); 
	if ($help) {
		echo "<p>$help</p>";
	}
	$parameters =& Adapting::getAdapterParameterDescriptors($adapter_name);
	if (is_null($parameters)) {
		// no parameters
		$parameters= array();
	}
	else {
		echo "<p style='font-weight: bold;'>This role adapter requires you to provide additional parameters specific";
		echo "for this role. Please fill in the following fields.<p/>";
	}
	?>

<table width="100%">
<?php foreach($parameters as $pardesc_id => $pardesc): ?>
<tr>
<td valign="top" class="form-entry form-first" style="font-weight: bold;">
<?= $pardesc["display-name"]?>
</td>
<?php             
	$param_value= '';
	$input_type= "text";
	$checked= '';
	
	// default value?
	if (isset($pardesc['default-value']) and empty($param_value)) {
		$param_value= trim($pardesc['default-value']);
	}
	
	if( eregi('SECRET',$pardesc_id) or eregi('PASSWORD',$pardesc_id) ) {
		$input_type= "password"; 
	}
	elseif ( eregi('ENABLE', $pardesc_id) ) {
		$input_type= "checkbox";
		$param_value="1";
	}
	
	?>
<td valign="top" class="form-entry form-first">
<input name="ap-<?= $pardesc_id ?>" type="<?= $input_type ?>" size=40 value="<?= $param_value ?>">
<br/><?= $pardesc["description"]?></td>
</tr>
<?php endforeach; ?>
</table>
<?php endif; ?>

</td>
</tr>

<!-- ************* Control Buttons ************************** -->    
<tr>
<td width="100%" valign="top" align="center" class="form-control" colspan="2">
<input type="submit" name="do_save" value="Save" />
</td>    
</tr>

</table>
</form>



<?php 
	// include common page footer
	//
	include("include/adminFooter.inc");
	?>
