<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: new-user.php,v 1.32 2005/10/13 15:17:39 vlry Exp $
 */


/**
* This page provides a form for editing user details.
*
* <b>Request Parameters</b>
* <ul>
*   <li>do_save=On: OPTIONAL. If present saves the data submitted in 
*      the HTTP request. If abstent, just loads the data from the
*      database and displays an input form.
*   </li>
*  </ul>
*
* 
* @version $Revision: 1.32 $
* @package AdminPortal
*/

/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */

/**
 * Ensures that all expected
 *   request parameters are actually passed in and that they have
 *   the acceptable values
 *
 *  Aborts with exit() if an error is detected. 
 *
 * @access private
 */ 
function ensure_request_parameters_valid() {

  // make sure mandatory uid is set
  //
  if (! isset($_REQUEST["uid"])) {
     ErrorHandling::fatalErrorAndExit("request parameter 'uid' is missing");
  }

  // make sure the uid is numeric and positive
  //
  if (! is_numeric($_REQUEST["uid"]) || $_REQUEST["uid"] < 0) {
     ErrorHandling::fatalErrorAndExit("illegal value '". $_REQUEST["uid"] . "' for request parameter 'uid'");
  }   
}

/**
 * Updates the user attributes of user
 *   $user with the values posted in $_REQUEST
 *
 * @param User $user  the user 
 *
 * @access private
 */
function update_user_attributes($user) {
  foreach ($_REQUEST as $key=>$value) {
      $value= trim($value);
      if (substr($key, 0, 10) == "attribute-") {

        // $attribute_id needs to be an integer (check User::storeAttribute interface)
        $attributeType_id = (int) substr($key, 10, strlen($key) - 10);

        if (!empty($value)) {
          if (!$user->isAttributeTrusted($attributeType_id)) {
                $user->storeAttribute($attributeType_id, $value);
          }
        } else {
          $user->deleteAttribute($attributeType_id);
        }
      }
  }
  // and store password for user
  $password= null;
  if (isset($_REQUEST["admin_password_1"])) {
    $password= $_REQUEST["admin_password_1"];
    $password= trim($password);
  }
  if ( empty($password) ) {
      $user->deleteAttribute(ATID_LOCALPASSWORD);
  } 
  else {
      $user->storeAttribute(ATID_LOCALPASSWORD, $password); 
  }
}

/**
 * Replies true, if $user_id is a valid user ID
 *
 * @param string $user_id   a user ID 
 *
 * @access private
 */

function is_valid_user_uid($user_uid) {
  // username cannot be empty 
  //
  $user_uid= trim($user_uid);
  if (empty($user_uid)) return false;

  // username cannot contain white space
  //
  if (ereg("[ \t\r\n]+", $user_uid)) return false; 

  return true; 
}



/**
 * Validates the input supplied in the global $_REQUEST array 
 *
 * @param array &$input_errors  OUT parameters with the validation
 *               error message
 * @access private
 */

function validate_input(&$input_errors) {
   $ret = true; 

  // validate user name
  //
  $user_uid = $_REQUEST["user_uid"];
  if (! is_valid_user_uid($user_uid)) {
    $ret = false;
    $input_errors["user_uid"] = "A user name must not be empty and must not contain blanks, tabs, carriage returns or line feeds. Please provide a new User ID.";
  }    
  if (User::existsUid($user_uid)) {
     $ret = false; 
     $input_errors["user_uid"] = "A user with this User ID already exists. Please provide a new User ID.";
  }
  
  // check password
  //
  $pw1 = $_REQUEST["admin_password_1"];
  $pw2 = $_REQUEST["admin_password_2"];
  if (trim($pw1) != trim($pw2)) {
    $ret = false;
    $input_errors["admin_password_2"] = "Repeated password is not identical to first password. Please enter passwords again."; 
  }

  return $ret; 
}


/* ***************************************************************** */
/*                        HTML                                       */
/* ***************************************************************** */

// bootstrap application
//
require_once("include/application.inc");

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);


// check wheter the current user is allowed to create a new user
//
if (! $admin->canCreateUser()) {
  ErrorHandling::fatalErrorAndExit("You do not have enough privileges to create a  user");
}

$input_errors = array();
if (isset($_REQUEST["do_save"])) {
  if (validate_input(&$input_errors)) {
     $user =& User::create($_REQUEST["user_uid"]);
     if (isset($_REQUEST['user_privilege'])) {
        $user->setPrivilege($_REQUEST["user_privilege"]);
     }     
     update_user_attributes($user);
     header("Location: " . PORTAL_BASE_URL . "/admin/edit-user.php?uid=" . $user->getUserId());
     exit;
  }
}


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

  <h1>Create a new User</h1>
 
  As a Portal Administrator you can create two type of new users: 
  <ol>
    <li><strong>Local Users:</strong> Local User have specific local user ID
      and a local password. They can log into the SNA-Portal using the local
      web-based authentication form.

    <li><strong>AAI Users:</strong> AAI Users have a unique AAI User ID.
      They can log in through the authentication mechansims provided by
      the unterlying Authentication and Authorization Infrastructur.

  </ol>

  <!-- ******************* Required Attributes ******************* -->
     
  <form action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
    <input type="hidden" name="do_save" value="1" />

  <table width="90%" border="0" cellspacing="0" class="form-table">

   <tr>
      <td align="center" width="100%" valign="top" class="form-head" colspan="2">
      Required Attributes 
      </td>
    </tr>
 
   <tr>
      <td width="30%%" valign="top" class="form-label form-first">
      User ID 
      </td>
      <td width="70%%" valign="top" class="form-entry form-first">
     <?php
        $req_user_uid= ""; 
        if( isset($_REQUEST['user_uid']) ) $req_user_uid= $_REQUEST['user_uid'];
     ?>
      <input type="text" name="user_uid" value="<?= $req_user_uid ?>" size="40"/> <br/>
     <?php if (isset($input_errors["user_uid"]) ):  ?>
        <span class="error-message"><?= $input_errors["user_uid"] ?></span><br/>
     <?php endif; ?>
     
      Enter either a <strong>Local User ID</strong> (i.e. <code>courseadmin1</code> or <code>paul</code>) or an 
      <strong>AAI User ID</strong> (i.e. <code>773443@aaitest.switch.ch</code>)

      </td>
    </tr>

   <tr>
      <td width="30%%" valign="top" class="form-label">
      Local Password 
      </td>
      <td width="70%%" valign="top" class="form-entry">
      <input type="password" name="admin_password_1" value="" size="10"/> <br/>
      <?php if (isset($input_errors["admin_password_1"])): ?>
        <span class="error-message"><?= $input_errors["admin_password_1"] ?></span><br/>
     <?php endif; ?>
      Fill in if this user shall be able to login using the local login
      form without going through AAI. <br/>
      <strong>Note:</strong> you must
      enter a password if you create a user with a <strong>Local User ID</strong>.
      </td>
    </tr>

   <tr>
      <td width="30%%" valign="top" class="form-label">
      Local Password (repeated)
      </td>
      <td width="70%%" valign="top" class="form-entry ">
      <input type="password" name="admin_password_2" value="" size="10"/> 
      <?php if (isset($input_errors["admin_password_2"])):  ?>
        <br/><span class="error-message"><?= $input_errors["admin_password_2"] ?></span><br/>
      <?php endif; ?>

      </td>
    </tr>
   <tr>
      <td width="30%%" valign="top" class="form-label">
      User Role
      </td>
      <td width="70%%" valign="top" class="form-entry">
      <?php
          $privileges = UserPrivilege::getPrivileges();
          if (! isset($_REQUEST["user_privilege"])) {
            $_REQUEST["user_privilege"] = 1;
          }
          foreach($privileges as $privilege)  {
             $name = $privilege["userprivilege_description"];
             $user_privilege= $privilege['userprivilege_id'];
             if ($_REQUEST["user_privilege"] == $user_privilege) {
                $checked = "checked"; 
             }
             else {
                $checked = "";
             }
         ?>
             <input type="radio" name="user_privilege" value="<?= $user_privilege ?>" <?= $checked ?> /><?= $name ?><br>
      <?php
           } // end for 
      ?>
      </td>
    </tr>
    <tr>
      <td align="center" width="100%" valign="top" class="form-control" colspan="2">
      <input type="submit" value="Create" />
      </td>
    </tr>
    
    </table>
    <p/>
    For both Local Users and AAI Users you can enter additional 
    user attributes. It is recommended, that you enter at least
    <strong>Family Name</strong>, <strong>Given Name</strong> and 
    <strong>Email</strong> for a Local User.  
    <p/>
    In case of a AAI User, the values you enter here may be replaced
    by values authenticated by the AAI when the user logs in the first
    time. Values you provide are considered non-authenticated unless
    the AAI will provide the same value as well.
    <p/>

  <table width="90%" border="0" cellspacing="0" class="form-table">
   <tr>
      <td align="center" width="100%" valign="top" class="form-head" colspan="2">
      Optional Attributes
      </td>
    </tr>


  <?php for ($id = 1; $id < 23; $id++): ?>
        <?php 
        $name = AttributeType::getAttributeTypeName($id);
        ?>

         <tr>
            <td width="30%" valign="top" class="form-label">
                <?= $name ?>
            </td>
            <td width="70%" valign="top" class="form-entry">
               <?php 
                   $req_attribute_value= "";
                   if (isset($_REQUEST["attribute$id"])) $req_attribute_value= $_REQUEST["attribute$id"];
               ?>
               <input type="text" name="attribute-<?=$id ?>" value="<?= $req_attribute_value ?>" /> 
            </td>
         </tr>

   <?php endfor; ?>
  <tr>
    <td valign="top" align="center" class="form-control" colspan=2>
      <input type="submit" value="Create" class="button">
    </td>    
  </tr>    

  </table>
  </form>
  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
