<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: login.php,v 1.34 2006/09/26 12:28:31 vlry Exp $
 */

/**
 * This page displays an authentication form and checks whether
 *   the current user can be authenticated as Local SNA-Portal
 *   User.
 *
 * <b>Request Parameters</b>
 * <ul>
 *   <li>username=<username>: OPTIONAL. The username 
 *    </li>
 *   <li>do_auth=on: OPTIONAL. If present, performs the authentication. If
 *      absent, just displays the input form.
 *    </li>
 *
 *  </ul>
 *
 * @version $Revision: 1.34 $
 * @package UserPortal
 */

// application include for documentation database
require_once("include/application.inc");
$logger=& Logger::getInstance();

/** 
 * Checks the request parameters 
 *
 * @param array  &$errors   an OUT-parameter with an array of error message
 *              The following keys are defined:
 *              <ul>
 *                <li>'username'  -> the error message for input supplied
 *                    in $_REQUEST["username"]
 *                <li>'password'  -> the error message for input supplied
 *                    in $_REQUEST["password"]
 *              </ul>
 * 
 * @return boolean  true, if input is accepted; false, otherwise
 *
 * @access private
 */
function check_request_parameter(&$errors) {
  $ret = true;

  // make sure we have a non-empty username
  //
  if (empty($_REQUEST["username"])) {
    $errors["username"] = "Username must not be empty.";
    $ret = false;
  }

  // make sure we have a non-empty password
  if (empty($_REQUEST["password"])) {
    $errors["password"] = "Password must not be empty.";
    $ret = false; 
  }
  return $ret;
}


/**
 * Authenticates a user against the SNA-Portal user database
 *
 * @param string $username    the username
 * @param string $password    the password
 * @param array  &$errors     an OUT-parameter with an array of error message
 *               The following keys are defined:
 *                'auth-error'  -> the error message for an authentication
 *                    error
 *
 * 
 * @return boolean true, if the user could be authenticated; false,
 *   otherwise
 *
 * @access private 
 */
function do_authenticate($username, $password, &$errors) {
  
  $logger=& Logger::getInstance();
  $user=& User::reconstituteByUid($username);
  if (is_null($user)) {
    $errors["auth-error"] = "The username or password is not correct. Please try again.";
    $logger->error("Local login: unknown user: $username");
    return false;
  }

  if (! $user->checkPassword($password)) {
      // uid & password match, therefore set $auth = true
     $errors["auth-error"] = "Invalid credential. Please try again.";
     $logger->error("authentication failure for $username:$password");
     return false; 
  }
  $logger->debug("Local login: user $username authenticated");
  return true; 
}



//
// form processing
//
if (isset($_REQUEST["do_auth"])) {
  $errors = array();

  if (check_request_parameter(&$errors)) {
    $username = $_REQUEST["username"];
    $password = $_REQUEST["password"];
    if (do_authenticate($username, $password, &$errors)) {

       $user=& User::reconstituteByUid($username);
       Session::create();
       Session::init($user);

       $logger->info("Local login: user $username login");
       Event::storeAuthEvent("Local user login: $username",$user->getID());
       $user->updateLoginDate();

       // cache current locale in the session object
       //
       if ( isset($_REQUEST["_locale"]) ) {
          $locale= $_REQUEST["_locale"];
          $_SESSION['_locale']= $locale; 
       }
      
       // redirect to the start page
       //
       $location= PORTAL_BASE_URL . "/user/start-page.php";
       header("Location: $location");
       exit;
    }
    else {
        Event::storeAuthEvent("Local user wrong username or password: $username",
                               1, EVENT_LEVEL_WARNING);
    }
  }
}

//
// i18n init
//
I18N::initLocale();
I18N::loadMessageBundle("user/login");

//
// HTML content
//
require_once("lookandfeel/current/lf-specific-functions.inc");
?>
<html>
<head>
<link rel="stylesheet" href="<?= lf_get_user_style_sheet_url() ?>" type="text/css">
<style type="text/css">
<!--
    /* WISEBEDaai Login pages style */
    a.wisebedaai-link {
        font-family: Verdana, Arial, Helvetica, sans-serif; 
        color: #203781; 
        font-size: 12;
    }
    .wisebedaai-link { 
        font-family: Verdana, Arial, Helvetica, sans-serif; 
        color: #203781; 
        font-size: 12; 
    }
    img.wisebedaai-login-button {
        padding-bottom: 10; 
        padding-top: 12;
    }
    img.non-aai-login-button {
        padding-bottom: 8; 
        padding-top: 40;
    }
    input.non-aai-login-button {
	   padding-bottom: 8; 
	   padding-top: 12; 
	   float: right;
	}
    .fullheight {
      height: 100%; 
      min-height: 100%;
    }
    
    /* boxing styles */
    td.wisebedaai {
      font-family: Verdana, Arial, Helvetica, sans-serif; 
      background: #FFFFFF; 
      color: #000000; 
      font-size: 12;
    }
    td.top-left-x {
        background-image: url('../lookandfeel/current/obenlinksx.gif');
        height: 14px;
        width: 14px;
    }
    td.top-middle-x {
        background-image: url('../lookandfeel/current/obenmittex.gif');
        height: 14px;
    }
    td.top-right-x {
        background-image: url('../lookandfeel/current/obenrechtsx.gif');
        height: 14px;
        width: 14px;
    }
    td.middle-left-x {
        background-image: url('../lookandfeel/current/mittelinksx.gif');
        width: 14px;
    }
    td.middle-right-x {
        background-image: url('../lookandfeel/current/mitterechtsx.gif');
        width: 14px;
    }
    td.bottom-left-x {
        background-image: url('../lookandfeel/current/untenlinksx.gif');
        height: 14px;
        width: 14px;
    }
    td.bottom-middle-x {
        background-image: url('../lookandfeel/current/untenmittex.gif');
        height: 14px;
    }
    td.bottom-right-x {
        background-image: url('../lookandfeel/current/untenrechtsx.gif');
        height: 14px;
        width: 14px;
    }
-->
</style>

<script language="JavaScript">
function switchLanguage(sel) {
  window.open("login.php?_locale=" + sel.options[sel.selectedIndex].value, "_self"); 
}
</script>
</head>
<body>

<p class="welcome-title" style="padding-top: 1cm; text-align: center;"><?= I18N::getMessage("login.pagetitle") ?></p>


<p style="margin-left: 1cm; margin-right: 1cm; text-align: center">
<?= I18N::getMessage("login.text1") ?>
</p>
<p style="margin-left: 1cm; margin-right: 1cm; text-align: center">
<?= I18N::getMessage("login.text2") ?>
</p>


<!-- local login box -->
<div align="center">
  <table width="300" border="0" cellpadding="0" cellspacing="0">
    <tr>
      <td class="top-left-x"></td>
      <td class="top-middle-x"></td>
      <td class="top-right-x"></td>
    </tr>
     <!-- links + content + right -->
    <tr>
       <!-- links border -->
      <td class="middle-left-x"></td>
       <!-- content -->
      <td class="wisebedaai" valign="top">
         <p><b><?= I18N::getMessage("login.formtitle") ?></b></p>
         <?php if (isset($errors["auth-error"])): ?>
            <span class="error-message"><?= $errors["auth-error"] ?></span>
            <p/>
         <?php endif; ?>
         <form method="post" action="<?= $_SERVER['PHP_SELF'] ?>">
         <input type="hidden" name="do_auth" value="1" />
         <table width="90%">
         <!-- lang -->
         <tr>
           <td class="wisebedaai"><?= I18N::getMessage("login.label1") ?></td>
           <td class="wisebedaai">
            <select name="_locale" onChange="switchLanguage(this);">
            <?php
               $languages= I18N::getSupportedLocales();
               foreach($languages as $langid => $langdisplayname): 
                  $selected= (I18N::getLocale() == $langid) ? "SELECTED" : "";
               ?>
                  <option value="<?= $langid ?>" <?= $selected ?> ><?= $langdisplayname ?></option> 
               <?php endforeach; ?>
            </select>
           </td>           
         </tr>
         <!-- username -->
         <tr>
           <td class="wisebedaai"><?= I18N::getMessage("login.label2") ?></td>
           <td class="wisebedaai"><input type="text" name="username" size="20" maxlength="30" tabindex="1">
           <?php if (isset($errors["username"])):  ?>
               <br/><span class="error-message"><?= $errors["username"] ?></span>
           <?php endif; ?>
           </td>
         </tr>
         <!-- password -->
         <tr>
           <td class="wisebedaai"><?= I18N::getMessage("login.label3") ?></td>
           <td class="wisebedaai"><input type="password" name="password" size="20" maxlength="30" tabindex="2">
           <?php if (isset($errors["password"])):  ?>
               <br/><span class="error-message"><?= $errors["password"] ?></span>
           <?php endif; ?>
           </td>
         </tr>
         </table>
         <input type="image" src="../lookandfeel/current/button-login-gray.gif" width="52" height="19" border="0" class="non-aai-login-button" alt="login" align="bottom">
         </form>
      </td>
       <!-- right border -->
      <td class="middle-right-x"></td>
    </tr>
    <tr>
      <td class="bottom-left-x"></td>
      <td class="bottom-middle-x"></td>
      <td class="bottom-right-x"></td>
    </tr>
  </table>
</div>
     
</body>
</html>
