<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: my-roles.php,v 1.25 2007/03/06 17:22:02 vlry Exp $
 */
/* ****************************************************************** */

/**
* This page isplays the list of roles the current user is subscribed to.
*
* @version $Revision: 1.25 $
* @package UserPortal
*/

/**
 */
 

// bootstrap application 
//
require_once("include/application.inc");


/* *********************************************************** */
/**
* Returns a display name for a subscription state ID
*
* @param int $subscriptionStateID  the subscription state ID
*
* @return string a display name for this ID
* 
* @access private
*/
function getSubscriptionStateDisplayName($subscriptionStateID, $roleState) {
	switch($roleState) {
		case RESOURCE_ACCESS_STATE_OPEN: 
	    case RESOURCE_ACCESS_STATE_FREE: 
	    case RESOURCE_ACCESS_STATE_PASSWORD: 
		  $role_state = I18N::getMessage("my-roles.resstate.open");
		  break;
		  
		case RESOURCE_ACCESS_STATE_SUSPENDED: 
		  $role_state = I18N::getMessage("my-roles.resstate.suspended");
		  break;

		case RESOURCE_ACCESS_STATE_CLOSED: 
		  $role_state = I18N::getMessage("my-roles.resstate.closed");
		  break;
	}
	
  switch($subscriptionStateID) {
    case SUBSCRIPTION_STATE_ACCEPTED: 
      //$msg = I18N::getMessage("my-roles.substate.accepted") . " / $role_state";
      $msg = I18N::getMessage("my-roles.substate.accepted");
      break;
    case SUBSCRIPTION_STATE_REJECTED:
      $msg =  I18N::getMessage("my-roles.substate.rejected"); 
      break;
    case SUBSCRIPTION_STATE_SUSPENDED:
      $msg =  I18N::getMessage("my-roles.substate.suspended");
      break;
    case SUBSCRIPTION_STATE_PENDING:
      $msg =  I18N::getMessage("my-roles.substate.pending");
      break;
    default:
      $msg =  I18N::getMessage("my-roles.substate.unknown"); 
      break;
  }  
  return $msg;
}

/* *********************************************************** */
/**
* Return true, if a role with role state $roleState
* and subscription state $subscriptionStateID is currently
* accessible. The role is accessible if
* <ul>
*   <li>the role is in state {@link RESOURCE_ACCESS_STATE_OPEN} and</li>
*   <li>the subscription is in state {@link SUBSCRIPTION_STATE_ACCEPTED}</li>
* </ul>
*
* @param int $subscriptionStateID  the subscription state ID
* @param int $roleState the role state 
*
* @return boolean  true, if the role is accessible; false, otherwise 
*
* @access private
*
*/
function is_subscribed_role_accessible($subscriptionStateID, $roleState) {
	$ret = ($roleState == RESOURCE_ACCESS_STATE_OPEN);
	$ret |= ($roleState == RESOURCE_ACCESS_STATE_FREE);
	$ret |= ($roleState == RESOURCE_ACCESS_STATE_PASSWORD);
	$ret &= ($subscriptionStateID == SUBSCRIPTION_STATE_ACCEPTED);
	return $ret;
}

// start session and ensure access restrictions
//
Session::ensureUserSessionValid();
I18N::initLocale();
I18N::loadMessageBundle("user/my-roles");

//
// make sure the current user exists
//
$uid= Session::getUID();
$user=& User::reconstituteByUid($uid);
if (is_null($user)) {
    ErrorHandling::fatalErrorAndExit("Could not reconstitute user: $uid");
}



include("include/header.inc");
?>
  <h1><?= I18N::getMessage("my-roles.pagetitle") ?></h1>

 <?= I18N::getMessage("my-roles.text1") ?>
  
  <img src="<?= PORTAL_BASE_URL . "/lookandfeel/current/go.gif" ?>" border="0" alt="Goto role" /> 
  <?= I18N::getMessage("my-roles.redirect") ?>
  <p/>
  
  
  <table width="90%" border="0" cellspacing="0" class="data-table">

    <tr>
      <th class="data" width="40%" valign="top"><?= I18N::getMessage("my-roles.table1.coltitle1") ?></th>
      <th class="data" width="30%"><?= I18N::getMessage("my-roles.table1.coltitle2") ?></th>
      <th class="data last" width="30%"><?= I18N::getMessage("my-roles.table1.coltitle3") ?></th>
    </tr>  
  <?php
  $my_roles = $user->getSubscribedRoles();
  ?>
  <?php if (! $my_roles || count($my_roles) == 0): ?>
    <tr>
      <td width="100%" valign="top" class="data-odd last" colspan=3>
      <?= I18N::getMessage("my-roles.no-sub") ?>
      </td>
    </tr>
    
  <?php else: ?>
   <?php
     $numSubscribedRoles = count($my_roles);     
     for ($i=0; $i < $numSubscribedRoles; $i++):
           $role = $my_roles[$i];
           $style = ($i % 2 == 0) ? "data-even" : "data-odd";
    ?>
    <tr>
      <td width="40%" valign="top" class="<?= $style ?>"><?= $role["role_title"] ?>     
      &nbsp;<a href="go.php?rid=<?= $role["role_id"] ?>" target="role">
      <?php if (is_subscribed_role_accessible($role["subscription_state"], $role["role_state"])): ?><img src="<?= PORTAL_BASE_URL . "/lookandfeel/current/go.gif" ?>" border="0" alt="Goto role"></a><?php endif; ?>
      </td>
      <td width="30%" valign="top" class="<?= $style ?>">
        <?= getSubscriptionStateDisplayName($role["subscription_state"],$role["role_state"]) ?>
      </td>
      <td width="30%" valign="top" class="<?= $style ?> last">
      <a href="<?= PORTAL_BASE_URL . "/user/role-details.php?rid=" . $role["role_id"] ?>"><?= I18N::getMessage("my-roles.act.view-details") ?></a> 
      | <a href="<?= PORTAL_BASE_URL . "/user/unsubscribe.php?rid=" . $role["role_id"] ?>"><?= I18N::getMessage("my-roles.act.unsubscribe") ?></a> 
      </td>
    </tr>
    <?php  endfor; ?>
  <?php endif; ?>
  </table>


<?php
// include common footer
//
include("include/footer.inc");
?>
