<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: role-details.php,v 1.30 2007/03/06 17:22:02 vlry Exp $
 */
/* ****************************************************************** */


/**
* This page displays details about a specific role.
*
* <b>Request Parameters</b>
* <ul>
*  <li>rid=<rid>: MANDATORY. An internal role ID. <rid> must be a valid
*     integer > 0. The role ID <rid> must exist and must be visible.
*   </li>
* </ul>
*
* @version $Revision: 1.30 $
* @package UserPortal
*/

/**
 */


// Headerfile for documentation database
//
require_once("include/application.inc");



// start session and ensure access restrictions
//
Session::ensureUserSessionValid();
I18N::initLocale();
I18N::loadMessageBundle("user/role-details");

//
// make sure the current user exists
//
$uid= Session::getUID();
$user=& User::reconstituteByUid($uid);
if (is_null($user)) {
    ErrorHandling::fatalErrorAndExit("Could not reconstitute user: $uid");
}

$rid = $_REQUEST["rid"];
if (! isset($rid)) 
{
  ErrorHandling::fatalErrorAndExit("request argument 'rid' is missing");
}

if ( ! is_numeric($rid) || $rid < 0) 
{
  ErrorHandling::fatalErrorAndExit("illegal value '$rid' for request argument 'rid'");
}

$role =& Role::reconstituteById($rid);
if (is_null($role)) {
	ErrorHandling::fatalErrorAndExit("role with role id '$rid' not found");
}

$subscription =& Subscription::reconstitute(&$user, &$role);

// subscribed user can see invisible role
if (! $role->isVisible() and is_null($subscription)) {
  ErrorHandling::fatalErrorAndExit("role with role id '$rid' is not visible");
}



// ##############################################
// # HTML OUTPUT
// ##############################################

// include  common header
//
include("include/header.inc");
?>


  <h1><?= I18N::getMessage("role-details.pagetitle") ?></h1>
  
  <table width="90%" border="0" cellspacing="0" >
  <tr>
    <th valign="top" class="form-head" colspan=2><?= I18N::getMessage("role-details.form1.title") ?></td>
  </tr>

  <tr>
    <td width="30%" valign="top" class="form-label form-first"><?= I18N::getMessage("role-details.form1.label1") ?></td>
    <td valign="top" class="form-entry form-first">
      <?= $role->getTitle() ?>
    </td>
  </tr>
  
  <tr>
    <td valign="top" class="form-label"><?= I18N::getMessage("role-details.form1.label2") ?></td>
    <td valign="top" class="form-entry">
    <?= Formatting::formatTableCellContent($role->getDescription()) ?>
    </td>
  </tr>  
  
  <!-- ********** Role Status  ************** -->
  <tr>
    <td valign="top" class="form-label"><?= I18N::getMessage("role-details.form1.label3") ?></td>
    <td valign="top" class="form-entry">
    <?php
       $role_state= $role->getRoleAccessState();
    ?>
    <?php if ($role_state == RESOURCE_ACCESS_STATE_CLOSED):  ?>
      <?= I18N::getMessage("role-details.resstate.closed") ?>
    <?php elseif ($role_state == RESOURCE_ACCESS_STATE_OPEN):  ?>
      <?= I18N::getMessage("role-details.resstate.open") ?>
    <?php elseif ($role_state == RESOURCE_ACCESS_STATE_FREE):  ?>
      <?= I18N::getMessage("role-details.resstate.free") ?>
    <?php elseif ($role_state == RESOURCE_ACCESS_STATE_PASSWORD):  ?>
      <?= I18N::getMessage("role-details.resstate.password") ?>
    <?php elseif ($role_state == RESOURCE_ACCESS_STATE_SUSPENDED):  ?>
      <?= I18N::getMessage("role-details.resstate.suspended") ?>
    <?php endif; ?>
    </td>
  </tr>  
  
  <!-- ********** Subscription Status ********** -->
  <tr>
    <td valign="top" class="form-label"><?= I18N::getMessage("role-details.form1.label4") ?></td>
    <td valign="top" class="form-entry">
    <?php if (is_null($subscription)):  ?>
       <?= I18N::getMessage("role-details.substate.notsub") ?>

    <?php elseif ($subscription->getState() == SUBSCRIPTION_STATE_ACCEPTED): ?>
      <?= I18N::getMessage("role-details.substate.accepted") ?>
      <?php if ($role_state == RESOURCE_ACCESS_STATE_OPEN):  ?>
        <?= I18N::getMessage("role-details.res.accesslink", PORTAL_BASE_URL . "/user/go.php?rid=" . $role->getID()) ?>
      <?php endif; ?>
     
    <?php elseif ($subscription->getState() == SUBSCRIPTION_STATE_REJECTED):  ?>
      <?= I18N::getMessage("role-details.substate.rejected") ?>

   <?php elseif ($subscription->getState() == SUBSCRIPTION_STATE_SUSPENDED):  ?>
      <?= I18N::getMessage("role-details.substate.suspended") ?>

    <?php else:  ?>
      <?= I18N::getMessage("role-details.substate.pending") ?>

    <?php endif;  ?>
    </td>
  </tr>    
  </table>




  <?php
  $provider=& $role->getRoleProvider();
  $provider_attrs= $provider->getAttributes(ATID_MAIL, ATID_TELEPHONENUMBER);
  ?>
        
  <p/>
  <table width="90%" border="0" cellspacing="0" >
  <tr>
    <th valign="top" class="form-head" colspan=2><?= I18N::getMessage("role-details.form2.title") ?></td>
  </tr>

  <tr>
    <td width="30%" valign="top" class="form-label form-first">
      <?= I18N::getMessage("role-details.form2.label1") ?>  
    </td>
    <td valign="top" class="form-entry form-first">
      <?= $provider->getNameOrUid() ?>
    </td>
  </tr>
  <tr>
    <td width="30%" valign="top" class="form-label">
       <?= I18N::getMessage("role-details.form2.label2") ?>
    </td>
    <td valign="top" class="form-entry">
      <?php
         if ( !empty($provider_attrs[ATID_MAIL])) {
            $mail= $provider_attrs[ATID_MAIL]['attribute_value'];
            echo "<a href=\"mailto:$mail\" >$mail</a>";
         }
         else {
            echo I18N::getMessage("role-details.form2.unknown-email");
         } 
      ?>
    </td>
  </tr>
  <tr>
    <td width="30%" valign="top" class="form-label">
       <?= I18N::getMessage("role-details.form2.label3") ?>
    </td>
    <td valign="top" class="form-entry">
      <?php 
         if ( !empty($provider_attrs[ATID_TELEPHONENUMBER]) ) {
            $phone=  $provider_attrs[ATID_TELEPHONENUMBER]['attribute_value'];
            echo $phone;
         }
         else {
            echo I18N::getMessage("role-details.form2.unknown-phone");
         }
      ?>
    </td>
  </tr>
  </table>

<?php
$manager_ids= $role->getRoleManagerIDs();
if ( !empty($manager_ids) ): ?>

    <table width="90%" border="0" cellspacing="0" >
    <tr>
        <th valign="top" class="form-head" colspan=2><?= I18N::getMessage("role-details.form3.title") ?></td>
    </tr>
    
    <?php
    foreach($manager_ids as $manager_id):
        $manager= Administrator::reconstituteById($manager_id);
        $manager_attrs= $manager->getAttributes(ATID_MAIL, ATID_TELEPHONENUMBER); ?>
        
        <tr>
        <td width="30%" valign="top" class="form-label form-first">
          <?= I18N::getMessage("role-details.form2.label1") ?>  
        </td>
        <td valign="top" class="form-entry form-first">
          <?= $manager->getNameOrUid() ?>
        </td>
        </tr>
        <tr>
        <td width="30%" valign="top" class="form-label">
           <?= I18N::getMessage("role-details.form2.label2") ?>
        </td>
        <td valign="top" class="form-entry">
        <?php
        if ( !empty($manager_attrs[ATID_MAIL])) {
            $mail= $manager_attrs[ATID_MAIL]['attribute_value'];
            echo "<a href=\"mailto:$mail\" >$mail</a>";
        }
        else {
            echo I18N::getMessage("role-details.form2.unknown-email");
        } ?>
        </td>
        </tr>
        <tr>
        <td width="30%" valign="top" class="form-label">
           <?= I18N::getMessage("role-details.form2.label3") ?>
        </td>
        <td valign="top" class="form-entry">
          <?php 
             if ( !empty($manager_attrs[ATID_TELEPHONENUMBER]) ) {
                $phone=  $manager_attrs[ATID_TELEPHONENUMBER]['attribute_value'];
                echo $phone;
             }
             else {
                echo I18N::getMessage("role-details.form2.unknown-phone");
             }
          ?>
        </td>
        </tr>

    <?php endforeach; ?>      
  
    </table>

<?php endif; ?>
    



<?php
include("include/footer.inc");
?>
